************************************************************************
* This routine computes the amplitude for all processes with
*  6 quarks + 1 lepton and one antineutrino, making use of the
* two routines fourw.f and twoztwow.f
* It makes use of the four momenta p(0:3,8) (all considered
* outgoing) and of the particle identities idp(8) (all outgoing)
* Moreover it makes use of the igroup information
* to set the various flags. This implies that the order of the particles
* is the one specified by the following table.
* Finally symfact is the simmetry factor corresponding to n1!*n2!*n3!...
*  where n1,n2,n3,.. are the numbers of identical particles of type
*  1,2,3,... in the final state
************************************************************************
*  igroup =1   c s~ d u~ c s~ mu- vm~       4w     202  
*  igroup =2   u u~ u u~ c s~ mu- vm~      2z2w    422
*  igroup =3   u u~ c c~ c s~ mu- vm~      2z2w    422
*  igroup =4   u u~ s s~ c s~ mu- vm~      2z2w    422
*  igroup =5   u u~ b b~ c s~ mu- vm~      2z2w    233
*  igroup =6   d d~ d d~ c s~ mu- vm~      2z2w    422
*  igroup =7   d d~ c c~ c s~ mu- vm~      2z2w    422
*  igroup =8   d d~ s s~ c s~ mu- vm~      2z2w    422
*  igroup =9   d d~ b b~ c s~ mu- vm~      2z2w    233       
*  igroup =10  c c~ c c~ c s~ mu- vm~      2z2w   1266
*  igroup =11  c c~ b b~ c s~ mu- vm~      2z2w    466
*  igroup =12  s s~ s s~ c s~ mu- vm~      2z2w   1266
*  igroup =13  s s~ b b~ c s~ mu- vm~      2z2w    466
*  igroup =14  b b~ b b~ c s~ mu- vm~      2z2w    610
*  igroup =15  u u~ d d~ c s~ mu- vm~      mix     312
*  igroup =16  c c~ s s~ c s~ mu- vm~      mix    1046
************************************************************************
      
      subroutine amp(p,idp,igroup,symfact,res,iord)

      implicit real*8 (a-b,d-h,o-z)
      implicit double complex (c)

      dimension p(0:3,8)
      dimension idp(1:8)

      parameter (namp2max=6)
      parameter (rm1o3=-1.d0/3.d0,r1o9=1.d0/9.d0)
      parameter (czero=(0.d0,0.d0))

      dimension ii(8)

* declaration for record of double structure in fow 
*    at the end we have cres(6).z(2,2,2,2).z(2,2,2,2)
      structure/zz/
         double complex z(2,2,2,2)
      end structure

      structure/fow/
         double complex w,zaux(2,2,2,2)
         record/zz/z(2,2,2,2)
      end structure
      record/fow/cres(namp2max)

      dimension amp2(namp2max), ipos(8,namp2max),
     &     factint(namp2max,namp2max)

      dimension iord(8)

      COMMON /pharand/ idum

* namp2= number of ampsquared from which one  extracts the 
*  configuration to pass to hadronization routines
* namp2_2w2z=number of ampsquared for routine twoztwow.f
* namp2_4w  =number of ampsquared for routine    fourw.f
* ipos(8,namp2max) gives the particles which have to be
*  in the first,second,..eighth position in the call to
*  the various partial amplitudes
* iord(8) is the inverse of ipos(8,nchosen), where nchosen
*  is the number of the colour configuration which is extracted
*  among all with probability proportional to the respective
*   partial amplitude squared.

* factint= factors in the interferences, carrying sign and corrections
*  to the color factor 27 of every amp squared
 
      do i=1,namp2max
        do k=1,8
          ipos(k,i)=k
        enddo
      enddo



      if (igroup.eq.1) then
        namp2_2w2z=0
        namp2_4w=2
        ipos(1,2)=5
        ipos(5,2)=1
        factint(1,2)=rm1o3

      elseif (igroup.eq.2) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(1,2)=3
        ipos(3,2)=1
        factint(1,2)=rm1o3

      elseif (igroup.eq.3) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(3,2)=5
        ipos(5,2)=3
        factint(1,2)=rm1o3

      elseif (igroup.eq.4) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(4,2)=6
        ipos(6,2)=4
        factint(1,2)=rm1o3
        
      elseif (igroup.eq.5) then
        namp2_2w2z=1
        namp2_4w=0
        
      elseif (igroup.eq.6) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(1,2)=3
        ipos(3,2)=1
        factint(1,2)=rm1o3
        
      elseif (igroup.eq.7) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(3,2)=5
        ipos(5,2)=3
        factint(1,2)=rm1o3
        
      elseif (igroup.eq.8) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(4,2)=6
        ipos(6,2)=4
        factint(1,2)=rm1o3
        
      elseif (igroup.eq.9) then
        namp2_2w2z=1
        namp2_4w=0
        
      elseif (igroup.eq.10) then
* exchange of positions 1 3 5          
        namp2_2w2z=6
        namp2_4w=0
        ipos(3,2)=5
        ipos(5,2)=3
        ipos(1,3)=3
        ipos(3,3)=1
        ipos(1,4)=5
        ipos(5,4)=1
        ipos(1,5)=3
        ipos(3,5)=5
        ipos(5,5)=1
        ipos(1,6)=5
        ipos(3,6)=1
        ipos(5,6)=3
        
        factint(1,2)=rm1o3
        factint(1,3)=rm1o3
        factint(1,4)=rm1o3
        factint(1,5)=r1o9
        factint(1,6)=r1o9
* different sign = 1 exchange (odd permutation)
* equal sign = 2 exchanges (even permutation)
* same sign factor 9 different sign factor 3
        factint(2,3)=r1o9
        factint(2,4)=r1o9
        factint(2,5)=rm1o3
        factint(2,6)=rm1o3
        factint(3,4)=r1o9
        factint(3,5)=rm1o3
        factint(3,6)=rm1o3
        factint(4,5)=rm1o3
        factint(4,6)=rm1o3
        factint(5,6)=r1o9
          
      elseif (igroup.eq.11) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(1,2)=5
        ipos(5,2)=1
        factint(1,2)=rm1o3
        
      elseif (igroup.eq.12) then
        namp2_2w2z=6
        namp2_4w=0
        ipos(4,2)=6
        ipos(6,2)=4
        ipos(2,3)=4
        ipos(4,3)=2
        ipos(2,4)=6
        ipos(6,4)=2
        ipos(2,5)=4
        ipos(4,5)=6
        ipos(6,5)=2
        ipos(2,6)=6
        ipos(4,6)=2
        ipos(6,6)=4

        factint(1,2)=rm1o3
        factint(1,3)=rm1o3
        factint(1,4)=rm1o3
        factint(1,5)=r1o9
        factint(1,6)=r1o9
* different sign = 1 exchange (odd permutation)
* equal sign = 2 exchanges (even permutation)
* same sign factor 9 different sign factor 3
        factint(2,3)=r1o9
        factint(2,4)=r1o9
        factint(2,5)=rm1o3
        factint(2,6)=rm1o3
        factint(3,4)=r1o9
        factint(3,5)=rm1o3
        factint(3,6)=rm1o3
        factint(4,5)=rm1o3
        factint(4,6)=rm1o3
        factint(5,6)=r1o9
          
      elseif (igroup.eq.13) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(2,2)=6
        ipos(6,2)=2
        factint(1,2)=rm1o3
          
      elseif (igroup.eq.14) then
        namp2_2w2z=2
        namp2_4w=0
        ipos(1,2)=3
        ipos(3,2)=1
        factint(1,2)=rm1o3
      
      elseif (igroup.eq.15) then
        namp2_2w2z=1
        namp2_4w=1
        ipos(2,2)=4
        ipos(4,2)=2
        
        factint(1,2)=rm1o3
          
      elseif (igroup.eq.16) then
        namp2_2w2z=4
        namp2_4w=2
        ipos(1,2)=5
        ipos(5,2)=1
        ipos(4,3)=6
        ipos(6,3)=4
        ipos(1,4)=5
        ipos(4,4)=6
        ipos(5,4)=1
        ipos(6,4)=4
        
        ipos(2,5)=4
        ipos(4,5)=2
        ipos(1,6)=5
        ipos(2,6)=4
        ipos(4,6)=2
        ipos(5,6)=1
        
        factint(1,2)=rm1o3
        factint(1,3)=rm1o3
        factint(1,4)=r1o9
        factint(1,5)=rm1o3
        factint(1,6)=r1o9
* different sign = 1 exchange (odd permutation)
* equal sign = 2 exchanges (even permutation)
* same sign factor 9 different sign factor 3
        factint(2,3)=r1o9
        factint(2,4)=rm1o3
        factint(2,5)=r1o9
        factint(2,6)=rm1o3
        factint(3,4)=rm1o3
        factint(3,5)=r1o9
        factint(3,6)=rm1o3
        factint(4,5)=rm1o3
        factint(4,6)=r1o9
        factint(5,6)=rm1o3

      endif

      namp2=namp2_2w2z+namp2_4w

      do i=1,namp2_2w2z
        call twoztwow(p(0,ipos(1,i)),p(0,ipos(2,i)),p(0,ipos(3,i)),
     &         p(0,ipos(4,i)),p(0,ipos(5,i)),p(0,ipos(6,i)),
     &         p(0,ipos(7,i)),p(0,ipos(8,i)),idp(ipos(1,i)),
     &         idp(ipos(2,i)),idp(ipos(3,i)),idp(ipos(4,i)),
     &         idp(ipos(5,i)),idp(ipos(6,i)),idp(ipos(7,i)),
     &         idp(ipos(8,i)),cres(i).zaux)
      enddo

* set to zero to avoid previous unwanted components
      do i=1,namp2_2w2z
        do i1=1,2
          do i2=1,2
            do i3=1,2
              do i4=1,2
                do i5=1,2
                  do i6=1,2
                    do i7=1,2
                      do i8=1,2
                        cres(i).z(i1,i2,i3,i4).z(i5,i6,i7,i8)=czero
                      enddo
                    enddo
                  enddo
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo



      do i=1,namp2_2w2z
        do i1=1,2
          do i2=1,2
            do i3=1,2
              do i4=1,2
                ii(ipos(1,i))=i1
                ii(ipos(2,i))=i2
                ii(ipos(3,i))=i3
                ii(ipos(4,i))=i4
                ii(ipos(5,i))=2
                ii(ipos(6,i))=2
                ii(ipos(7,i))=2
                ii(ipos(8,i))=2
                cres(i).z(ii(1),ii(2),ii(3),
     &               ii(4)).z(ii(5),ii(6),
     &               ii(7),ii(8))=
     &               cres(i).zaux(i1,i2,i3,i4)
              enddo
            enddo
          enddo
        enddo
      enddo


      do i=namp2_2w2z+1,namp2
        call fourw(p(0,ipos(1,i)),p(0,ipos(2,i)),p(0,ipos(3,i)),
     &         p(0,ipos(4,i)),p(0,ipos(5,i)),p(0,ipos(6,i)),
     &         p(0,ipos(7,i)),p(0,ipos(8,i)),idp(ipos(1,i)),
     &         idp(ipos(2,i)),idp(ipos(3,i)),idp(ipos(4,i)),
     &         idp(ipos(5,i)),idp(ipos(6,i)),idp(ipos(7,i)),
     &         idp(ipos(8,i)),cres(i).w)
      enddo

      do i=1,namp2max
        factint(i,i)=1.d0
      enddo
      do i=1,namp2max-1
        do j=i+1,namp2max
          factint(j,i)=factint(i,j)
        enddo
      enddo

* compute res

      res=0.d0
      do i1=1,2
        do i2=1,2
          do i3=1,2
            do i4=1,2
            do i5=1,2
            do i6=1,2
            do i7=1,2
            do i8=1,2
              do i=1,namp2_2w2z
                do j=1,namp2_2w2z
                  res=res+cres(i).z(i1,i2,i3,i4).z(i5,i6,i7,i8)*
     &                 conjg(cres(j).z(i1,i2,i3,i4).z(i5,i6,i7,i8))
     &                 *factint(i,j) 
                enddo
              enddo
            enddo
            enddo
            enddo
            enddo
            enddo
          enddo
        enddo
      enddo

      do i=namp2_2w2z+1,namp2
        do j=1,namp2_2w2z
          res=res+2.d0*factint(i,j)*cres(i).w*
     &         conjg(cres(j).z(2,2,2,2).z(2,2,2,2))
        enddo
      enddo

      do i=namp2_2w2z+1,namp2
        do j=namp2_2w2z+1,namp2_2w2z+namp2_4w
          res=res+factint(i,j)*cres(i).w*conjg(cres(j).w)
        enddo
      enddo

* spin average 1/4
* color average 1/9
* overall color factor 27

      res=res*3.d0/4.d0/symfact

* compute amp2(i) (i=1,namp2) summed over all helicity configurations

      do i=1,namp2_2w2z
        do i1=1,2
          do i2=1,2
            do i3=1,2
              do i4=1,2
              do i5=1,2
              do i6=1,2
              do i7=1,2
              do i8=1,2
                amp2(i)=amp2(i)+cres(i).z(i1,i2,i3,i4).z(i5,i6,i7,i8)*
     &                 conjg(cres(i).z(i1,i2,i3,i4).z(i5,i6,i7,i8))
              enddo
              enddo
              enddo
              enddo
              enddo
            enddo
          enddo
        enddo
      enddo

      do i=namp2_2w2z+1,namp2
          amp2(i)=cres(i).w*conjg(cres(i).w)
      enddo

* extract the configuration nchosen for color flow

      amp2tot=0.d0
      do i=1,namp2
        amp2tot=amp2tot+amp2(i)
      enddo

      do i=1,namp2
        amp2(i)=amp2(i)/amp2tot
      enddo

      i=1
      am2=amp2(1)
      zzran=ran2(idum)
      do while (zzran.gt.am2.and.i.lt.namp2)
        i=i+1
        am2=am2+amp2(i)
      enddo      

      nchosen=i

* compute iord as the inverse of ipos(i,nchosen) 

      do i=1,8
        iord(ipos(i,nchosen))=i
      enddo


      return
      end













