#!/usr/bin/perl
#
# Script to report on results of two set of integrations using SGE
# in PHASE
# USES:
# array process_lnu which contains a string with the set of particles for each
# group, all outgoing, separated by spaces.
# Particles are          d  u  s  c  b  mu  num
# Antiparticles are      d_ u_ s_ c_ b_ mu_ num_
# The leptons DO NOT APPEAR in directory names, the group number DOES. 
#
# Help is available entering:
# perl compareresultsSGE.pl -h
#


# DEFAULT DIRECTORIES AND FILES
$dirtreeroot1_def = "mh0";
$Dirtreeroot2_def = "mh1";

#INITIALIZATION TO GET RID OF WARNING MESSAGES
$dirtreeroot1="";
$Dirtreeroot2="";

# PARSE COMMAND OPTIONS

while ( $_ = $ARGV[0]) {
	shift;
	last if (/^--$/);
	if (/^-d/)		{ $dirtreeroot1 = &get_option("-dirtreeroot1");	}
	elsif (/^-D/)		{ $Dirtreeroot2 = &get_option("-Dirtreeroot2");	}

	elsif (/^-h/)		{ &usage("help")	} 
	elsif (/^[a-zA-Z]*/)    { 
	  print "can't specify more then one target!" if ($dirtreeroot ne "");
	}
	else 		        { print "unknown argument";          }
	
}

if ($dirtreeroot1 eq ""){ $dirtreeroot1=$dirtreeroot1_def;}
if ($Dirtreeroot2 eq ""){ $Dirtreeroot2=$Dirtreeroot2_def;}
print "dirtreeroot1= $dirtreeroot1\n";
print "Dirtreeroot2= $Dirtreeroot2\n";

# BASIC DEFINITIONS

%pdgnumber = (  
     "d" => 1, "u" => 2, "s" => 3, "c" => 4, "b" => 5,
     "d_" => -1, "u_" => -2, "s_" => -3, "c_" => -4, "b_" => -5, 
     "e" => 11, "nue" => 12 , "mu" => 13, "num" => 14, "tau" => 15, "nut" => 17,
     "e_" => -11, "nue_" => -12 , "mu_" => -13, "num_" => -14,
     "tau_" => -15, "nut_" => -17 
     );
                 
                 
%antiparticle = ( 
     "d" => "d_", "u" => "u_", "s" => "s_", "c" => "c_", "b" => "b_",              
     "d_" => "d", "u_" => "u", "s_" => "s", "c_" => "c", "b_" => "b", 
     "e" => "e_", "nue" => "nue_" , "mu" => "mu_", "num" => "num_",
     "tau" => "tau_", "nut" => "nut_",
     "e_" => "e", "nue_" => "nue" , "mu_" => "mu", "num_" => "num",
     "tau_" => "tau", "nut_" => "nut" );

%group = (
                    "c s_ d u_ c s_" => "01",
                    "u u_ u u_ c s_" => "02",
                    "u u_ c c_ c s_" => "03",
                    "u u_ s s_ c s_" => "04",
                    "u u_ b b_ c s_" => "05",
                    "d d_ d d_ c s_" => "06",
                    "d d_ c c_ c s_" => "07",
                    "d d_ s s_ c s_" => "08",
                    "d d_ b b_ c s_" => "09",                  
                    "c c_ c c_ c s_" => "10",
                    "c c_ b b_ c s_" => "11",
                    "s s_ s s_ c s_" => "12",
                    "s s_ b b_ c s_" => "13",
                    "b b_ b b_ c s_" => "14",
                    "u u_ d d_ c s_" => "15",
                    "c c_ s s_ c s_" => "16"
);

@processes_lnu = (
#                    "c c_ s s_ c s_"
#                    "u u_ d d_ c s_"
#                    "b b_ b b_ c s_"
#                    "s s_ b b_ c s_"
#                    "s s_ s s_ c s_"
#                    "c c_ b b_ c s_"
#                    "c c_ c c_ c s_"
#                    "d d_ b b_ c s_"
#                    "d d_ s s_ c s_"
#                    "d d_ c c_ c s_"
#                    "d d_ d d_ c s_"
#                    "u u_ b b_ c s_"
#                    "u u_ s s_ c s_"
#                    "u u_ c c_ c s_"
#                    "u u_ u u_ c s_"
                    "c s_ d u_ c s_"

#                    "c s_ d u_ c s_",
#                    "u u_ u u_ c s_",
#                    "u u_ c c_ c s_",
#                    "u u_ s s_ c s_",
#                    "u u_ b b_ c s_",
#                    "d d_ d d_ c s_",
#                    "d d_ c c_ c s_",
#                    "d d_ s s_ c s_",
#                    "d d_ b b_ c s_",
#                    "c c_ c c_ c s_",
#                    "c c_ b b_ c s_",
#                    "s s_ s s_ c s_",
#                    "s s_ b b_ c s_",
#                    "b b_ b b_ c s_",
#                    "u u_ d d_ c s_",
#                    "c c_ s s_ c s_"
                    );
                    
                    
# FIND PROCESSES

@process_list = ();
@process_idplist = ();
                   
foreach $proc_lnu (@processes_lnu){
@part = split(/ /,$proc_lnu);
$length = $#part;
$grouptag = $group{$proc_lnu};
#print "+++++++++++++++++++++++++++++++++++++++++++\n";
#print "string= $proc_lnu\n";
#print "+++++++++++++++++++++++++++++++++++++++++++\n";
#print "$proc_lnu becomes $part[0] x $part[1] xxx $part[5]\n";
#print "last element of array has label $length\n";

@in_state=();
for(my $i1=0;$i1 < $length;$i1++){
#print "string= $proc_lnu\n";
  @partlocal = @part;
  $first = $antiparticle{splice(@partlocal,$i1,1)};
                                # @partlocal shortened by one element
#print "first = $first\n";
#print "$partlocal['$i1'] is now in position $i1\n";
  for(my $i2=$i1;$i2 < $length;$i2++){
    @partlocal2 = @partlocal;
    $second = $antiparticle{splice(@partlocal2,$i2,1)};
                                         # @partlocal2 now contains
                                         # the final state in split form
#print "second = $second\n";
    $state1 = $first.$second;
    $state2 = $second.$first;
#print "first initial state : $state1\n";
#print "second initial state: $state2\n";
    $found = "false";
    foreach $state (@in_state){
#print "check if found: $state\n";
      if($state eq $state1 || $state eq $state2){
           $found = "true";}
           }
    if($found eq "false"){
       push @in_state,$state1;
       $string = $state1."-";
       $string .= join "",@partlocal2;
       $string .= $grouptag;
#print "reaction:  $string\n";
       push @process_list,$string;
#print "initial partlocal2 @partlocal2\n";
       unshift  @partlocal2, $first,$second;
# Add leptons
       push @partlocal2, "mu","num_";
#print "partlocal2 with initial and leptons @partlocal2\n";
       @process_idp=();
       foreach (@partlocal2) {
       push @process_idp, $pdgnumber{$_};
                             }
       $idp = "  ";
       $idp .=  join " ",@process_idp;
       $process_idplist{$string} = $idp;
#print "process: @partlocal2\n";
#print "idp: $idp\n";
       }
       
     }    # end of loop on $i2
   }      # end of loop on $i1
#print "******************************************\n\n";
#print "group: $proc_lnu\n\n";
#foreach $process (@process_list) {print "$process\n"}
}        # end of loop on group list



# CREATE SUBDIRECTORIES AND INPUT/RUN FILES

foreach $dirname (@process_list){
    $name1 = $dirtreeroot1."/".$dirname;
    $name2 = $Dirtreeroot2."/".$dirname;
#print "name1: $name1\n";
#print "name2: $name2\n\n";
    opendir(DIR1,"$name1") or print "$name1 could not be opened\n";
    @file1 = grep{ $_ ne '.' and $_ ne '..'} readdir DIR1;
#print "@file1\n\n";
     @outfile1=();
     foreach (@file1) { if(/^run\.o/){
#                                      print "$_ --\n";
                                       push @outfile1, $_}}
#print "@outfile1\n\n";
    closedir(DIR1);
    opendir(DIR2,"$name2") or print "$name2 could not be opened\n";
    @file2 = grep{ $_ ne '.' and $_ ne '..'} readdir DIR2;
#print "@file2\n\n";
     @outfile2=();
     foreach (@file2) { if(/^run\.o/){ 
#                                      print "$_ --\n";
                                       push @outfile2, $_}}
#print "@outfile2\n\n";
    closedir(DIR2);

     foreach $file (@outfile1) {
#       print "$file +++++\n";
        open(RESFILE1,"$name1/$file") or print "$file could not be opened\n";
	@list=();
	$finished=0;
	while(<RESFILE1>){
#	print;
	if($_ =~ /Sigma/) {push @list,$_};
	if($_ =~ /FORTRAN STOP/) {$finished=1};
	}
      $outline = pop(@list);
      if($finished == 1){
        print "$name1/$file\n"}
      else{
        print "$name1/$file     UNFINISHED\n"}
      print "$outline";
      close(RESFILE1); 
     }
     foreach $file (@outfile2) {
#       print "$file +++++\n";
        open(RESFILE2,"$name2/$file") or print "$file could not be opened\n";
	@list=();
	$finished=0;
	while(<RESFILE2>){
#	print;
	if($_ =~ /Sigma/) {push @list,$_};
	if($_ =~ /FORTRAN STOP/) {$finished=1};
	}
      $outline = pop(@list);
      if($finished == 1){
        print "$name2/$file\n"}
      else{
        print "$name2/$file     UNFINISHED\n"}
      print "$outline";
      close(RESFILE2); 
     }
     print"\n";
    }


# SUBROUTINES

sub get_option {
	&usage("missing argument for $_[0]") if ($#ARGV==-1) ;
	$result = $ARGV[0];
	shift @ARGV;	 
	return $result;
}


sub usage {
  local($mess) = @_;

  print "compareresultsSGE.pl: $mess\n\n";

  print <<"_EOF_";
usage:  perl compareresultsSGE.pl [-options ...] 

where options include:                                    Defaults -
    -dirtreeroot1  root of  tree #1  (full path)            mh0
    -Dirtreeroot2  root of  tree #2  (full path)            mh1
    -help         prints usage details                      -
long options can be abbreviated up to one letter.

Possible exit codes:
           0 - normal, success           1 - bad arguments (usage)
           2 - interrupted               
_EOF_

 exit 1;
}

# end  
