***********************************************************************
*                        SUBROUTINE COUPLING                          *
*                                                                     *
*                                                                     *
*  Purpose:  It contains DATA (for masses, W and Z widths and         *
*                                                     alfa_couplings) *
*            It computes Higgs and top widths, and couplings between  *
*            particles.                                               *
*                                                                     *
*  Call to Subroutine Bernoulli                                       *
*                                                                     *
***********************************************************************


       subroutine coupling

      implicit real*8 (a-b,d-h,o-z)
      implicit double complex (c)

      include 'common.h'

      DATA rmw/80.40d0/, rmz/91.187d0/, rmt/175.d0/, rmb/4.8d0/,
     &     rmc/0.75d0/,rmtau/1.78d0/,
     &     gamw/0.2042774d+01/, gamz/0.25007d+01/
      DATA gf/1.16639d-05/, alfainv_me/137.0359895d0/,
     &  alfas_w/0.1225d0/, alfas_z/0.123d0/,
     &  alfas_t/0.100d0/, alfas_h/0.100d0/

      DATA pi/3.141592653589793238462643d0/

      DATA czero/(0.d0,0.d0)/, cuno/(1.d0,0.d0)/, cim/(0.d0,1.d0)/


*************** options ***********
      DATA inqcd/1/

      parameter(nterm=10)   ! Bernoulli
***********************************


      if (rmh.ge.0.d0) then
	print*,'rmh=',rmh
      else
        print*, 'NO HIGGS'
      endif

      twopi=2.d0*pi
      fourpi=4.d0*pi

      rmw2=rmw**2
      rmz2=rmz**2
      rmt2=rmt**2
      rmb2=rmb**2
c rmh to be read from input
      rmh2=rmh**2

c running masses and alfas for the Higgs

      qsquared=rmh**2
      rmt_run= pymrun(6,qsquared)
      rmb_run= pymrun(5,qsquared)
      alfas_h= pyalps(qsquared)
      

c gfermi scheme

      s2w=1.d0-rmw2/rmz2
      g2=4.d0*sqrt(2.d0)*gf*rmw2
      alfainv=4.d0*pi/(s2w*g2)

      sw=sqrt(s2w)
      rc2w=1.d0-s2w
      rcw=sqrt(rc2w)
      rcotw=rcw/sw
      rcot2w=rcotw**2

*higgs width

      if (rmh.gt.0.d0) then
        gamhqq=rmh*(3.d0*(rmb_run**2*(1.d0-4.d0*(rmb/rmh)**2)**(1.5)+
     &         rmc**2*(1.d0-4.d0*(rmc/rmh)**2)**(1.5)))
     &         /(8.d0*rmw**2*s2w*alfainv)
        if(inqcd.eq.1)then
          gamhqq=gamhqq*(1.d0+alfas_h/pi*(5.67d0+29.14d0*alfas_h/pi))
        endif
        gamhll=rmh*(rmtau**2*(1.d0-4.d0*(rmtau/rmh)**2)**(1.5))
     &       /(8.d0*rmw**2*s2w*alfainv)

        gamhtt=0.d0
        if(rmh.gt.(2.d0*rmt))then 
          gamhtt=rmh*(3.d0*(rmt_run**2*(1.d0-4.d0*(rmt/rmh)**2)**(1.5)))
     &         /(8.d0*rmw**2*s2w*alfainv)
          if(inqcd.eq.1)then
            gamhtt=gamhtt*(1.d0+alfas_h/pi*(5.67d0+29.14d0*alfas_h/pi))
          endif
        endif

        gamhgg=rmh*((rmh*alfas_h/(3.d0*pi))**2)/
     &       (8.d0*rmw**2*s2w*alfainv)
c     if(inqcd.eq.1)then
c     gamhgg=gamhgg*(1.d0+(95.d0*0.25d0-35.d0/6.d0)*alfas_h/pi)
c     endif

        if(rmh.le.(2.d0*rmw))then
          xh=rmw**2/rmh**2
          gamhww=3.d0*pi**2/(2.d0*alfainv**2*s2w**2*rmw**4) *
     &         rmw**4 * 0.0625d0/pi**3 * rmh *
     &         ( 3.d0*(1.d0-8.d0*xh+20.d0*xh**2)/sqrt(4.d0*xh-1.d0)*
     &         dacos(0.5d0*(3.d0*xh-1.d0)/xh**(1.5)) -
     &         0.5d0*(1.d0-xh)/xh*(2.d0-13.d0*xh+47.d0*xh**2) -
     &         1.5d0*(1.d0-6.d0*xh+4.d0*xh**2)*log(xh))
        elseif(rmh.gt.(2.d0*rmw))then
          xh=rmw**2/rmh**2
          betahi=sqrt(1.d0-4.d0*xh)
          gamhww=rmh/(8.d0*rmw**2*s2w*alfainv)*(0.5d0*rmh**2*
     &         (1.d0-4.d0*xh+12.d0*xh**2)*betahi)
        endif


        if(rmh.le.(2.d0*rmz))then
          xh=rmz**2/rmh**2
          gamhzz=3.d0 * pi**2/(2.d0*alfainv**2*s2w**2*rmw**4) *
     &         rmz**4 * 0.0625d0/pi**3 * rmh *
     &         ( 3.d0*(1.d0-8.d0*xh+20.d0*xh**2)/sqrt(4.d0*xh-1.d0)*
     &         dacos(0.5d0*(3.d0*xh-1.d0)/xh**(1.5)) -
     &         0.5d0*(1.d0-xh)/xh*(2.d0-13.d0*xh+47.d0*xh**2) -
     &         1.5d0*(1.d0-6.d0*xh+4.d0*xh**2)*log(xh)) *
     &         (7.d0/12.d0-10.d0/9.d0*s2w+40.d0/27.d0*s2w**2)
        elseif(rmh.gt.(2.d0*rmz))then
          xh=rmz**2/rmh**2
          betahi=sqrt(1.d0-4.d0*xh)
          gamhzz=rmh/(8.d0*rmw**2*s2w*alfainv)*(0.25d0*rmh**2*
     &         (1.d0-4.d0*xh+12.d0*xh**2)*betahi)
        endif
        
        gamh=gamhqq+gamhtt+gamhll+gamhgg+gamhww+gamhzz

        print*,'gamh=',gamh

      endif

* top width
        
c gamt taken from sixphact

        CALL Bernoulli(nterm)
        gamt=sqrt((rmt**2-rmb**2-rmw**2)**2-4.
     &               d0*rmb**2*rmw**2)
        gamt=gamt*(rmt**2+rmb**2-2.
     &    d0*rmw**2+(rmt**2-rmb**2)**2/rmw**2)
        gamt=gamt*rmw**2*Gf/(8.d0*pi*sqrt(2.d0)*rmt**3)
        IF(inqcd.EQ.1)THEN
          ry=(rmw/rmt)**2
          rf=2.d0*pi**2/3.d0-2.5d0+2.d0*log(ry)*log(1.d0-ry)+
     &       4.d0*rli2(ry,nterm)-2.d0*ry+((5.d0+4.d0*ry)*log(1.d0-ry)+
     &       2.d0*ry*log(ry)/(1.d0-ry)-4.d0*ry**3*(1.d0-ry+log(ry))/
     &       ((1.d0-ry)**2))/(1.d0+2.d0*ry)
          gamt=gamt*(1.d0-2.d0*alfas_t*rf/(3.d0*pi))
        ENDIF !inqcd


      cgmw=cim*gamw*rmw
      cgmz=cim*gamz*rmz
      cgmh=cim*gamh*rmh
      cgmt=cim*gamt*rmt

      cmw2=rmw2-cgmw
      cmz2=rmz2-cgmz
      cmh2=rmh2-cgmh
      cmt2=rmt2-cgmt

c masses 

      do i=1,16
        imass(i)=0
        rmass(i)=0.d0
        rmass2(i)=0.d0
        cmass2(i)=czero
      enddo
* set masses different from zero for the amplitudes
      imass(5)=1
*Pythia wants input INITIAL particle masses =0 (check if unavoidable)
*  For this reaso we put here ALL b masses =0
c      rmass(5)=rmb
c      rmass2(5)=rmb2
c      cmass2(5)=rmb2
      rmass(5)=0.d0
      rmass2(5)=0.d0
      cmass2(5)=0.d0
*
      imass(6)=1
      rmass(6)=rmt
      rmass2(6)=rmt2
      cmass2(6)=cmt2
* set masses of the antiparticles
      do i=1,16
        imass(-i)=imass(i)
        rmass(-i)=rmass(i)
        rmass2(-i)=rmass2(i)
        cmass2(-i)=cmass2(i)
      enddo

* couplings

      if (rmh.ge.0.d0) then
        rhbb=-rmb_run/(2.d0*rmw*sw)
        rhtt=-rmt_run/(2.d0*rmw*sw)
      else
        rhbb=0.d0
        rhtt=0.d0
      endif

* photon-electron left and right
      fel=-1.d0
      fer=-1.d0
* Zeta-electron left and right
      zer=(+1.d0*s2w)/rcw/sw
      zel=(-.5d0+1.d0*s2w)/rcw/sw
* Zeta-neutrino left and right
      zvr=0.d0
      zvl=.5d0/rcw/sw
* Zeta-quarkup left and right
      fqul=2.d0/3.d0
      fqur=fqul
      zqur=(-2.d0/3.d0*s2w)/rcw/sw
      zqul=(.5d0-2.d0/3.d0*s2w)/rcw/sw
* Zeta-quarkdown left and right
      fqdl=-1.d0/3.d0
      fqdr=fqdl
      zqdr=(1.d0/3.d0*s2w)/rcw/sw
      zqdl=(-.5d0+1.d0/3.d0*s2w)/rcw/sw
* W left (right=0)
      wcl=1.d0/sw/sqrt(2.d0)


* 1=d, 2=u, 3=s, 4=c, 5=b, 6=t
* 11=e-, 12=v_e, 13=mu-, 14=v_mu, 15=tau-, 16=v_tau
* all antiparticles have the same number but opposite sign
* moreover: 21=gluon, 22=gamma, 23=Z0, 24=W+, 25=h

* which fermions are leptons

      do i=1,6
        ilept(i)=0
      enddo  
      do i=11,16
        ilept(i)=1
      enddo  
      do i=1,16
        ilept(-i)=ilept(i)
      enddo  

* which fermions are neutrinos

      do i=1,16
        ineutri(i)=0
      enddo
      ineutri(12)=1
      ineutri(14)=1
      ineutri(16)=1
      do i=1,16
        ineutri(-i)=ineutri(i)
      enddo  


* which fermions are of the up type

      do i=1,16
        if (mod(i,2).eq.1) then
          iup(i)=0
        else
          iup(i)=1
        endif
        iup(-i)=iup(i)
      enddo


      do i=1,6
        if (iup(i).eq.0) then
          fcr(i)=fqdr
          fcl(i)=fqdl
          zcr(i)=zqdr
          zcl(i)=zqdl
        else
          fcr(i)=fqur
          fcl(i)=fqul
          zcr(i)=zqur
          zcl(i)=zqul
        endif
        fcr(-i)=fcr(i)
        fcl(-i)=fcl(i)
        zcr(-i)=zcr(i)
        zcl(-i)=zcl(i)
      enddo

      do i=11,16
        if (iup(i).eq.0) then
          fcr(i)=fer
          fcl(i)=fel
          zcr(i)=zer
          zcl(i)=zel
        else
          fcr(i)=0.d0
          fcl(i)=0.d0
          zcr(i)=zvr
          zcl(i)=zvl
        endif
        fcr(-i)=fcr(i)
        fcl(-i)=fcl(i)
        zcr(-i)=zcr(i)
        zcl(-i)=zcl(i)
      enddo

      return
      end



