* CUT FUNCTION: RETURNS 1 IF ALL CUTS ARE PASSED, 0 IF NOT 

      INTEGER FUNCTION IPASSCUTS(p,
     &  e_min_lep,pt_min_lep,eta_max_lep,ptmiss_min,
     &  e_min_j,pt_min_j,eta_max_j,
     &  eta_def_jf_min,eta_def_jb_max,eta_def_jc_max,
     &  pt_min_jcjc,rm_min_jj,rm_min_jlep,
     &  rm_min_jcjc,rm_max_jcjc,
     &  rm_min_jfjb,eta_min_jfjb,
     &  d_ar_jj,d_ar_jlep,
     &  thetamin_jj,thetamin_jlep,
     &  i_e_min_lep,i_pt_min_lep,i_eta_max_lep,i_ptmiss_min,
     &  i_e_min_j,i_pt_min_j,i_eta_max_j,i_eta_jf_jb_jc,
     &  i_pt_min_jcjc,i_rm_min_jj,i_rm_min_jlep,
     &  i_rm_min_jcjc,i_rm_max_jcjc,
     &  i_rm_min_jfjb,i_eta_min_jfjb,
     &  i_d_ar_jj,i_d_ar_jlep,
     &  i_thetamin_jj,i_thetamin_jlep,
     &  i_usercuts, i_usercutsos)

      IMPLICIT REAL*8 (a-b,d-h,o-z)
      include 'common.h'
      INCLUDE 'common_subproc.h'

      DIMENSION p(0:3,8)
     

*      LOCAL VARIABLES
      DIMENSION p0(0:3,6),eta0(6),pt0(6),rmsq(6,6),pmod(6),
     &          delphi(6,6),deltheta(6,6),paux(2)

      DIMENSION idp0(6),invord0(8)


      ipasscuts=0
           
* EXTRACT FINAL STATE PARTICLES AND RECORD THEIR LOCATION IN p0
      i0=1
      DO i=1,8
        IF(idp_inout(i).GT.0)THEN
          idp0(i0)=idp(i)
          DO j=0,3
            p0(j,i0)=p(j,i)
          ENDDO
          invord0(i)=i0
          i0=i0+1
        ENDIF
      ENDDO

C   PDG        parton label (6, 5, 4, 3, 2, 1, 0, -1, ......,-5,-6)
C                       for (t, b, c, s, u, d, g, d~, ....., b~,t~)
C   PDG        parton label (11, 12,  13,  14,  15,   16)
C                       for (e-, ve, mu-, vmu, tau, vtau)

* COMPUTE PT'S AND RAPIDITIES
      DO i=1,6
        part_ptsq=p0(1,i)*p0(1,i)+p0(2,i)*p0(2,i)
        pt0(i)=sqrt(part_ptsq)
        part_3psq=part_ptsq+p0(3,i)*p0(3,i)
        pmod(i)=sqrt(part_3psq)
* elmod
        app=p0(3,i)/pmod(i)
        IF(app.GT.1.d0.OR.app.LT.-1.d0) RETURN
        eta0(i)=-log(tan(0.5d0*acos(app)))
* elmodend
      ENDDO

* CHARGED LEPTONS
      DO i=1,6
        IF((abs(idp0(i)).EQ.11).OR.(abs(idp0(i)).EQ.13).OR.
     &     (abs(idp0(i)).EQ.15))THEN
* ENERGY
          IF(i_e_min_lep.EQ.1) THEN
            IF(p0(0,i).LT.e_min_lep) RETURN
          ENDIF
* PT
          IF(i_pt_min_lep.EQ.1) THEN
            IF(pt0(i).LT.pt_min_lep) RETURN
          ENDIF
* RAPIDITY
          IF(i_eta_max_lep.EQ.1) THEN
            IF(abs(eta0(i)).GT.eta_max_lep) RETURN
          ENDIF
        ENDIF
      ENDDO

* NEUTRINOS: WE ASSUME THERE IS AT MOST ONE NEUTRINO
      DO i=1,6
        IF(abs(idp0(i)).EQ.12 .OR.
     &     abs(idp0(i)).EQ.14 .OR.
     &     abs(idp0(i)).EQ.16)THEN
* PT
          IF(i_ptmiss_min.EQ.1) THEN
            IF(pt0(i).LT.ptmiss_min) RETURN
          ENDIF
        ENDIF
      ENDDO

* JETS
      eta_max_j0=-1.d3
      eta_min_j0=1.d3
      DO i=1,6
        IF(abs(idp0(i)).LE.5)THEN
* ENERGY
          IF(i_e_min_j.EQ.1) THEN
            IF(p0(0,i).LT.e_min_j) RETURN
          ENDIF
* PT
          IF(i_pt_min_j.EQ.1) THEN
            IF(pt0(i).LT.pt_min_j) RETURN
          ENDIF
* RAPIDITY
          IF(i_eta_max_j.EQ.1) THEN
            IF(abs(eta0(i)).GT.eta_max_j) RETURN
          ENDIF
* FIND MOST FORWARD AND MOST BACKWARD JETS
          IF(eta0(i).LT.eta_min_j0)THEN
            eta_min_j0=eta0(i)
            i_eta_min_j0=i
          ENDIF
          IF(eta0(i).GT.eta_max_j0)THEN
            eta_max_j0=eta0(i)
            i_eta_max_j0=i
          ENDIF
        ENDIF
      ENDDO
      
      IF(i_eta_jf_jb_jc.EQ.1) THEN
* ASK FOR THE MOST BACKWARD JET TO BE BACKWARD
        IF(eta_min_j0.GT.eta_def_jb_max) RETURN
* ASK FOR THE MOST FORWARD JET TO BE FORWARD
        IF(eta_max_j0.LT.eta_def_jf_min) RETURN
* ASK FOR OTHER TWO JETS TO BE CENTRAL
        DO i=1,6 
          IF(abs(idp0(i)).LE.5 .AND. 
     &       i.NE.i_eta_min_j0 .AND. i.NE.i_eta_max_j0)THEN
            IF(abs(eta0(i)).GT.eta_def_jc_max) RETURN
          ENDIF
        ENDDO
      ENDIF   !  (i_eta_jf_jb_jc.EQ.1)

* ASK FOR MINIMUM RAPIDITY SEPARATION BETWEEN MOST FORWARD AND MOST BACKWARD 
* JET
      IF(i_eta_min_jfjb.EQ.1) THEN
        IF(eta_max_j0-eta_min_j0.LT.eta_min_jfjb) RETURN
      ENDIF
          

* ASK FOR MINIMUM PT OF TWO CENTRAL JETS
      IF(i_pt_min_jcjc.EQ.1)THEN
        paux(1)=0.d0
        paux(2)=0.d0
        DO i=1,6 
          IF(abs(idp0(i)).LE.5 .AND. 
     &       i.NE.i_eta_min_j0 .AND. i.NE.i_eta_max_j0)THEN
            paux(1)=paux(1)+p0(1,i)
            paux(2)=paux(2)+p0(2,i)
          ENDIF
        ENDDO
        part_ptsq=paux(1)*paux(1)+paux(2)*paux(2)
        IF(part_ptsq.LT.pt_min_jcjc*pt_min_jcjc) RETURN
      ENDIF
      
* COMPUTE INVARIANT MASSES
      DO i=1,6
        DO j=i+1,6
          rmsq(i,j)=(p0(0,i)+p0(0,j))**2
          DO i0=1,3
            rmsq(i,j)=rmsq(i,j)-(p0(i0,i)+p0(i0,j))**2
          ENDDO
          rmsq(j,i)=rmsq(i,j)
        ENDDO
      ENDDO

* ASK FOR MINIMUM INVARIANT MASS OF FORWARD AND BACKWARD JETS
       IF(i_rm_min_jfjb.EQ.1) THEN
         IF(rmsq(i_eta_min_j0,i_eta_max_j0).LT.
     &                                rm_min_jfjb*rm_min_jfjb)
     &                          RETURN
       ENDIF
       
* ASK FOR MINIMUM MASS SEPARATION BETWEEN JETS
      IF(i_rm_min_jj.EQ.1) THEN
        DO i=1,6
          IF(abs(idp0(i)).LE.5)THEN
            DO j=i+1,6
              IF(abs(idp0(j)).LE.5)THEN
                IF(rmsq(i,j).LT.rm_min_jj*rm_min_jj) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
      
* ASK FOR MINIMUM MASS SEPARATION BETWEEN CENTRAL JETS
      IF(i_rm_min_jcjc.EQ.1) THEN
        DO i=1,6
          IF(abs(idp0(i)).LE.5 .AND. 
     &       i.NE.i_eta_min_j0 .AND. i.NE.i_eta_max_j0)THEN
            DO j=i+1,6
              IF(abs(idp0(j)).LE.5 .AND. 
     &         j.NE.i_eta_min_j0 .AND. j.NE.i_eta_max_j0)THEN
                IF(rmsq(i,j).LT.rm_min_jcjc*rm_min_jcjc) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
      
* ASK FOR MAXIMUM MASS SEPARATION BETWEEN CENTRAL JETS
      IF(i_rm_max_jcjc.EQ.1) THEN
        DO i=1,6
          IF(abs(idp0(i)).LE.5 .AND. 
     &       i.NE.i_eta_min_j0 .AND. i.NE.i_eta_max_j0)THEN
            DO j=i+1,6
              IF(abs(idp0(j)).LE.5 .AND. 
     &         j.NE.i_eta_min_j0 .AND. j.NE.i_eta_max_j0)THEN
                IF(rmsq(i,j).GT.rm_max_jcjc*rm_max_jcjc) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF

* ASK FOR MINIMUM MASS SEPARATION BETWEEN CHARGED LEPTON AND JETS
      IF(i_rm_min_jlep.EQ.1) THEN
        DO i=1,6
          IF((abs(idp0(i)).EQ.11).OR.(abs(idp0(i)).EQ.13).OR.
     &       (abs(idp0(i)).EQ.15))THEN
            DO j=1,6
              IF(abs(idp0(j)).LE.5)THEN
                IF(rmsq(i,j).LT.rm_min_jlep*rm_min_jlep) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
         

*COMPUTE SEPARATION IN ANGLE AND IN AZIMUTHAL ANGLE PHI
      DO i=1,6
        DO j=i+1,6
          delphi(i,j)=(p0(1,i)*p0(1,j)+p0(2,i)*p0(2,j))/pt0(i)/pt0(j)
          deltheta(i,j)=(p0(1,i)*p0(1,j)+p0(2,i)*p0(2,j)
     &                          +p0(3,i)*p0(3,j))/pmod(i)/pmod(j)
* elmod
          IF(delphi(i,j).GT.1.d0.OR.delphi(i,j).LT.-1.d0) RETURN
          IF(deltheta(i,j).GT.1.d0.OR.deltheta(i,j).LT.-1.d0) RETURN
* elmodend
          delphi(i,j)=acos(delphi(i,j))
          delphi(j,i)=delphi(i,j)
          deltheta(i,j)=acos(deltheta(i,j))
          deltheta(j,i)=deltheta(i,j)
        ENDDO
      ENDDO

* ASK FOR MINIMUM DELTA_R SEPARATION BETWEEN JETS
      IF(i_d_ar_jj.EQ.1) THEN
        DO i=1,6
          IF(abs(idp0(i)).LE.5)THEN
            DO j=i+1,6
              IF(abs(idp0(j)).LE.5)THEN
                delr0=delphi(i,j)*delphi(i,j)
     &                          +(eta0(i)-eta0(j))**2
                IF(delr0.LT.d_ar_jj*d_ar_jj) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
         
* ASK FOR MINIMUM DELTA_R SEPARATION BETWEEN JETS AND CHARGED LEPTON
      IF(i_d_ar_jlep.EQ.1) THEN
        DO i=1,6
          IF((abs(idp0(i)).EQ.11).OR.(abs(idp0(i)).EQ.13).OR.
     &       (abs(idp0(i)).EQ.15))THEN
            DO j=1,6
              IF(abs(idp0(j)).LE.5)THEN
                delr0=delphi(i,j)*delphi(i,j)
     &                          +(eta0(i)-eta0(j))**2
                IF(delr0.LT.d_ar_jlep*d_ar_jlep) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF

* ASK FOR MINIMUM ANGULAR SEPARATION BETWEEN JETS
      IF(i_thetamin_jj.EQ.1) THEN
        DO i=1,6
          IF(abs(idp0(i)).LE.5)THEN
            DO j=i+1,6
              IF(abs(idp0(j)).LE.5)THEN
                IF(deltheta(i,j).LT.thetamin_jj) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
         
* ASK FOR MINIMUM ANGULAR SEPARATION BETWEEN JETS AND CHARGED LEPTON
      IF(i_thetamin_jlep.EQ.1) THEN
        DO i=1,6
          IF((abs(idp0(i)).EQ.11).OR.(abs(idp0(i)).EQ.13).OR.
     &       (abs(idp0(i)).EQ.15))THEN
            DO j=1,6
              IF(abs(idp0(j)).LE.5)THEN
                IF(deltheta(i,j).LT.thetamin_jlep) RETURN
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
                  
* USER SUPPLIED CUTS
      IF(i_usercuts.EQ.1) THEN
        IF(iuserfunc(p0,idp0,eta0,pt0,pmod,rmsq,delphi,deltheta,
     &        i_eta_max_j0,eta_max_j0,i_eta_min_j0,eta_min_j0,
     &        invord0)
     &             .EQ.0) RETURN
      ENDIF

* USER SUPPLIED CUTS in oneshot
      IF(i_usercutsos.EQ.1) THEN
        IF(iuserfuncos(p0,idp0,eta0,pt0,pmod,rmsq,delphi,deltheta,
     &        i_eta_max_j0,eta_max_j0,i_eta_min_j0,eta_min_j0,
     &        invord0)
     &             .EQ.0) RETURN
      ENDIF
      
* IF WE GET HERE ALL CUTS HAVE BEEN PASSED
      ipasscuts=1
      RETURN

      END


* SAMPLE USER CUTS ROUTINE

      INTEGER FUNCTION IUSERFUNC(p0,idp0,eta0,pt0,pmod,rmsq,delphi,
     &        deltheta,i_eta_max_j0,eta_max_j0,i_eta_min_j0,eta_min_j0,
     &        invord0)

      IMPLICIT REAL*8 (a-b,d-h,o-z)
      include 'common.h'
      DIMENSION
     &   p0(0:3,6),eta0(6),pt0(6),pmod(6),rmsq(6,6),
     &  delphi(6,6),deltheta(6,6)
      DIMENSION
     &   idp0(6),invord0(8)
* LOCAL VARIABLES

      iuserfunc=0 
       
* ASK FOR MINIMUM TRANSVERSE MOMENTUM OF FOUR CENTRAL PARTICLES
      pperpmin=100.d0
      px=0.d0
      py=0.d0
      DO i=1,6
        IF(i.NE.i_eta_max_j0 .AND. i.NE.i_eta_min_j0) THEN
          px=px+p0(1,i)
          py=py+p0(2,i)
        ENDIF
      ENDDO
      IF(sqrt(px*px+py*py).LT.pperpmin) RETURN

* IF WE GET HERE ALL USER CUTS HAVE BEEN PASSED
      iuserfunc=1
      RETURN

      END
      
      
* Oneshot second SAMPLE USER CUTS ROUTINE

      INTEGER FUNCTION IUSERFUNCOS(p0,idp0,eta0,pt0,pmod,rmsq,delphi,
     &        deltheta,i_eta_max_j0,eta_max_j0,i_eta_min_j0,eta_min_j0,
     &        invord0)

      IMPLICIT REAL*8 (a-b,d-h,o-z)
      include 'common.h'
      DIMENSION
     &   p0(0:3,6),eta0(6),pt0(6),pmod(6),rmsq(6,6),
     &  delphi(6,6),deltheta(6,6)
      DIMENSION
     &   idp0(6),invord0(8)
* LOCAL VARIABLES

      iuserfuncos=0 
       
* ASK FOR MINIMUM TRANSVERSE MOMENTUM OF FOUR CENTRAL PARTICLES
      pperpmin=100.d0
      px=0.d0
      py=0.d0
      DO i=1,6
        IF(i.NE.i_eta_max_j0 .AND. i.NE.i_eta_min_j0) THEN
          px=px+p0(1,i)
          py=py+p0(2,i)
        ENDIF
      ENDDO
      IF(sqrt(px*px+py*py).LT.pperpmin) RETURN

* IF WE GET HERE ALL USER CUTS HAVE BEEN PASSED
      iuserfuncos=1
      RETURN

      END
      
