************************************************************************
*                        SUBROUTINE INTEGRATION                        *
*                                                                      *
*                                                                      *
* Purpose:  integration run based on a modified multichannel approach. *
*         It determines the alfa(i) during thermalization.             *
*                                                                      *
*  Call to subroutine VEGAS                                            *
*  Fill phavegas0i.dat if iflat.eq.1 ( see WRITE(24,*) )               *
*                                                                      *
************************************************************************


      SUBROUTINE integration


      IMPLICIT REAL*8 (a-b,d-h,o-z)


      INCLUDE 'common.h'
      INCLUDE 'common_subproc.h'
      PARAMETER (ndmx=50,mxdim=15)
      COMMON/iinteg_readinput/ncall_therm,itmx_therm,ncall,itmx
      COMMON/rinteg_readinput/acc_therm,acc
      COMMON/i_norm/i_normalize

      DIMENSION avgi(mxnphs),sd(mxnphs),y(ndmx,mxdim,mxnphs),
     &          ndo(mxnphs),si(mxnphs),swgt(mxnphs),schi(mxnphs)

*** variables for subroutine Vegas
      COMMON /pharand/ idum
      COMMON/abchia/rcalls
*6
      COMMON /phavestop/ivegasstop
      DIMENSION ytmp(ndmx,mxdim)
*6end

*** variables for phavegas/i/.dat, with i =sequential phase_space index 
      CHARACTER*80 phavegas
      character*1 charint1
      character*2 charint2

*** flag to use (1) or not (0) normalization in alfas determination 
      DATA inormal/1/

      EXTERNAL fxn

*** normalization : determines rnormal which is used in fxn
      IF (inormal.EQ.1) then

      PRINT*,' '
      PRINT*,'------------------------------------------------------'
      PRINT*,' '
      PRINT*,'NORMALIZATION'


* i_normalize is a flag that tells fxn not to compute amplitudes
        i_normalize=1

        itmx_norm=3
        ncall_norm=200000
        acc_norm=0.003d0
        DO j=1,mxnphs
          rnormal(j)=1.d0
        ENDDO

        i=0
        DO j=1,mxnphs
          IF(ialfa(j).NE.0)THEN
            i=i+1
            init_norm=0
            iphs_ind=j
            print*,' '
            print*,'iphs_ind=',iphs_ind
            CALL vegas(region,ndim,fxn,init_norm,ncall_norm,itmx_norm,
     &             nprn,rnormal(iphs_ind),sd(i),rchi2a,acc_norm,
     &             y(1,1,i),it,ndo(i),si(i),swgt(i),schi(i))
          ENDIF
        ENDDO !j

        i_normalize=0

      ELSE

        DO j=1,mxnphs
          rnormal(j)=1.d0
        ENDDO

      ENDIF


*** initialization alfa(i)
      DO j=1,mxnphs
        IF(ialfa(j).NE.0)THEN
          alfa(j)=1.d0/n_kphs
c          print*,'iphs_ind and alfa=',j,alfa(j)
        ENDIF
      ENDDO !j


      PRINT*,' '
      PRINT*,'------------------------------------------------------'
      PRINT*,' '
      PRINT*,'ALFA(i) DETERMINATION'
      PRINT*,' '

*** THERMALIZATION
*6
c      DO nit=1,itmx_therm

* ivegasstoptherm is the product of the ivegasstop of the various 
*   channels, so that all channel have eventually to stop at the same iteration

      ivegasstoptherm=0
      nit=1


      DO WHILE (nit.LE.itmx_therm.AND.ivegasstoptherm.EQ.0)
*6end

*7
      avgi_tot=0.d0
      sd_tot=0.d0
*7end

        i=0
*6
        ivegasstoptherm=1        
*6end
        DO j=1,mxnphs
          IF(ialfa(j).NE.0)THEN
            i=i+1
            iphs_ind=j
            print*, '   '
            print*,'iphs_ind=',iphs_ind

            IF(nit.EQ.1)THEN
              init_therm=0
            ELSE
              init_therm=2
            ENDIF
            it=nit
*6
            ivegasstop=0         ! flag which becomes =1 when vegas reach
                                !      the requested accuracy acc.    
*6end
            
            CALL vegas(region,ndim,fxn,init_therm,ncall_therm,nit,
     &                 nprn,avgi(i),sd(i),rchi2a,acc_therm,y(1,1,i),
     &                 it,ndo(i),si(i),swgt(i),schi(i))

*6
            ivegasstoptherm=ivegasstoptherm*ivegasstop
*6end

c results of various channels are put together only for the last 
c   iteration
*7
c            IF (nit.EQ.itmx_therm.OR.ivegasstoptherm.NE.0) THEN
c              avgi_tot=0.d0
c              sd_tot=0.d0
c            ENDIF
*7end
            IF (nit.EQ.itmx_therm.OR.ivegasstoptherm.NE.0) THEN
              avgi_tot=avgi_tot+avgi(i)
              sd_tot=sd_tot+sd(i)**2
            ENDIF
c
          ENDIF
        ENDDO !j

        app=0.d0
        i=0
        DO j=1,mxnphs
          IF(ialfa(j).ne.0)then
            i=i+1
            app=app+avgi(i)
          ENDIF
        ENDDO !j

        i=0
        DO j=1,mxnphs
          IF(ialfa(j).ne.0) then
            i=i+1
            iphs_ind=j
            alfa(iphs_ind)=avgi(i)/app
            print*,'iphs_ind and alfa=',iphs_ind,alfa(j)
          ENDIF
        ENDDO !j
*6
        nit=nit+1
*6end
      ENDDO !nit
      sd_tot=sqrt(sd_tot)
      PRINT*,' '

      PRINT 200
  200 FORMAT('------------------------------------------------------')
      PRINT*,' '
      PRINT 201,avgi_tot,sd_tot
  201 FORMAT(' Sigma = ',d13.7,' +/-',d10.3,' (pb)')

      do j=1,mxnphs
        if(ialfa(j).ne.0)then
          print*,'alfa=',alfa(j)
        endif
      enddo

*** integration

      PRINT*,' '
      PRINT*,'------------------------------------------------------'
      PRINT*,' '
      PRINT*,'INTEGRATION'
      PRINT*,' '

      avgi_tot=0.d0
      sd_tot=0.d0

      i=0
      DO j=1,mxnphs
        IF(ialfa(j).NE.0)THEN
          i=i+1
          init=1
          iphs_ind=j
          print*,'iphs_ind=',iphs_ind

          IF(iflat.EQ.0)THEN
            CALL vegas(region,ndim,fxn,init,ncall,itmx,nprn,avgi(i),
     &               sd(i),rchi2a,acc,y(1,1,i),it,ndo(i),si(i),
     &               swgt(i),schi(i))
          ELSE IF(iflat.EQ.1)THEN
*6
c$$$            acc=0.d0
c$$$
c$$$* initialization of the variables used if IFLAT=1
c$$$            novermax=0          ! number of points exceding the maximum
c$$$            nevent=0            ! number of generated momenta
c$$$            rmaxfxn=0.d0   ! maximum value of fxn*wgt after last but one
c$$$                                 ! Vegas iteration
c$$$            rmaxfxn_2=0.d0      ! maximum value of fxn*wgt after last
c$$$                                 ! Vegas iteration
c$$$
c$$$            if (i.le.9) then
c$$$              write (charint1,'(i1)') i
c$$$              charint2='0'//charint1
c$$$            elseif (i.le.99) then
c$$$              write (charint2,'(i2)') i
c$$$            endif
c$$$            phavegas='phavegas'//charint2//'.dat'
c$$$
c$$$            DO nit=1,itmx
c$$$
c$$$              IF(nit.GE.2)init=2
c$$$
c$$$              CALL vegas(region,ndim,fxn,init,ncall,nit,nprn,avgi(i),
c$$$     &             sd(i),rchi2a,acc,y(1,1,i),it,ndo(i),si(i),
c$$$     &             swgt(i),schi(i))
c$$$
c$$$              IF(istorvegas.EQ.1.AND.nit.EQ.(itmx-1))THEN
c$$$                OPEN(unit=24,file=phavegas,status='new')
c$$$*sandro 25/07
c$$$                do k=1,8
c$$$                  WRITE(24,*)iproc(k)
c$$$                enddo  
c$$$*sandro 25/07 end
c$$$               WRITE(24,*) igroup,n_kphs,iphs_ind,nidenticalinitial
c$$$                do jj=1,mxnphs
c$$$                  WRITE(24,*)ialfa(jj),rnormal(jj),alfa(jj)
c$$$                enddo
c$$$                do k=1,8
c$$$                  WRITE(24,*)idp(k),idp_inout(k)
c$$$                enddo
c$$$                do jj=1,mxnphs
c$$$                  do k=1,8
c$$$                    WRITE(24,*)iorder(k,jj)
c$$$                  enddo
c$$$                enddo
c$$$                WRITE(24,*)avgi(i),sd(i),rchi2a
c$$$                DO n=1,ndmx
c$$$                  DO m=1,mxdim
c$$$                    WRITE(24,*)y(n,m,i)
c$$$                  ENDDO         !m
c$$$                ENDDO           !n
c$$$                WRITE(24,*)ndo(i),si(i),swgt(i),schi(i)
c$$$              ENDIF
c$$$              IF(istorvegas.EQ.1.AND.nit.EQ.itmx)THEN
c$$$                rmaxfxn=rmaxfxn_2
c$$$                WRITE(24,*)rmaxfxn,rcalls
c$$$                WRITE(24,*)avgi(i)
c$$$                WRITE(24,*)ncall,itmx
c$$$                CLOSE(24)
c$$$              ENDIF
c$$$           ENDDO !nit



* initialization of the variables used if IFLAT=1

            ivegasstop=0         ! flag which becomes =1 when vegas reach
                                !      the requested accuracy acc.    
            nevent=0            ! number of generated events
            novermax=0          ! number of points exceeding the maximum
            rmaxfxn=0.d0        ! maximum value of fxn*wgt after last but one
                                ! Vegas iteration 
            rmaxfxnnew=0        ! maximum value of fxn*wgt after last
                                 ! Vegas iteration

            nit=1

            DO WHILE (nit.LE.itmx.AND.ivegasstop.EQ.0)

              IF(nit.GE.2)init=2

              CALL vegas(region,ndim,fxn,init,ncall,nit,nprn,avgi(i),
     &             sd(i),rchi2a,acc,y(1,1,i),it,ndo(i),si(i),
     &             swgt(i),schi(i))

* ivegasstop in phavegas can exit with value 1 only after the first iteration
              IF (ivegasstop.EQ.0.AND.nit.NE.itmx) THEN
                rmaxfxn=rmaxfxnnew
                rmaxfxnnew=0
                novermax=0
                nevent=0
              ENDIF

              IF(istorvegas.EQ.1)THEN
                avgitmp=avgi(i)
                sdtmp=sd(i)
                rchi2atmp=rchi2a
                DO n=1,ndmx
                  DO m=1,mxdim
                    ytmp(n,m)=y(n,m,i)
                  ENDDO         !m
                ENDDO           !n
                ndotmp=ndo(i)
                sitmp=si(i)
                swgttmp=swgt(i)
                schitmp=schi(i)
              ENDIF

              nit=nit+1

            ENDDO  !nit and ivegasstop

            IF(istorvegas.EQ.1)THEN
              if (i.le.9) then
                write (charint1,'(i1)') i
                charint2='0'//charint1
              elseif (i.le.99) then
                write (charint2,'(i2)') i
              endif
              phavegas='phavegas'//charint2//'.dat'

              OPEN(unit=24,file=phavegas,status='new')
              
              do k=1,8
                WRITE(24,*)iproc(k)
              enddo  
              WRITE(24,*) igroup,n_kphs,iphs_ind,nidenticalinitial
              do jj=1,mxnphs
                WRITE(24,*)ialfa(jj),rnormal(jj),alfa(jj)
              enddo
              do k=1,8
                WRITE(24,*)idp(k),idp_inout(k)
              enddo
              do jj=1,mxnphs
                do k=1,8
                  WRITE(24,*)iorder(k,jj)
                enddo
              enddo
              WRITE(24,*)avgitmp,sdtmp,rchi2atmp
              DO n=1,ndmx
                DO m=1,mxdim
                  WRITE(24,*)ytmp(n,m)
                ENDDO           !m
              ENDDO             !n
              WRITE(24,*)ndotmp,sitmp,swgttmp,schitmp

              WRITE(24,*)rmaxfxnnew,rcalls
              WRITE(24,*)avgi(i)
              WRITE(24,*)ncall,itmx
              CLOSE(24)
            ENDIF

*6end
          ENDIF       !iflat

          avgi_tot=avgi_tot+avgi(i)
          sd_tot=sd_tot+sd(i)**2

          PRINT 202
 202    FORMAT('------------------------------------------------------')
          PRINT*,' '
          PRINT 203,avgi(i),sd(i)
 203      FORMAT(' Sigma = ',d13.7,' +/-',d10.3,' (pb)')

          IF(iflat.EQ.1)THEN
            PRINT*,'Informations about flat events generation:'
            PRINT*,'         ----------------------           '

           PRINT 204,rmaxfxn
 204       FORMAT(' Maximum after next-to-last VEGAS iteration = ',d9.3)
*6
c            PRINT 205,rmaxfxn_2
            PRINT 205,rmaxfxnnew
*6end
 205        FORMAT(' Maximum after last VEGAS iteration = ',d9.3)
            PRINT 206,nevent
 206        FORMAT(' Flat events number = ',i9)
            PRINT 207,novermax
 207        FORMAT(' number of function values over maximum = ',i9)
          ENDIF
        
        ENDIF ! (ialfa(j).NE.0)

      ENDDO !j

      sd_tot=sqrt(sd_tot)

      PRINT 208,avgi_tot,sd_tot
 208  FORMAT(' Sigma = ',d13.7,' +/-',d10.3,' (pb)')

      RETURN
      END
