#!/usr/bin/perl
#
# Script to create directories, input files and SGE-run files for all
# processes in PHASE
# USES:
# array process_lnu which contains a string with the set of particles for each
# group, all outgoing, separated by spaces.
# Particles are          d  u  s  c  b  mu  num
# Antiparticles are      d_ u_ s_ c_ b_ mu_ num_
# The leptons DO NOT APPEAR in directory names, the group number DOES. 
#
# Help is available entering:
# perl setupdirSGE.pl -h
#


# DEFAULT DIRECTORIES AND FILES
$dirtreeroot_def = "mh0";
$basedir_def = "./";
$input_template_def = "template.st0";
$inputfilename = "r.in";
$runfilename = "run";
$SGEfilename = "SGEfile";
$executefile_def = "phase.exe";
$foreach_script = "SGEall";

#INITIALIZATION TO GET RID OF WARNING MESSAGES
$basedir="";
$dirtreeroot="";
$executefile="";
$input_template="";

# PARSE COMMAND OPTIONS

while ( $_ = $ARGV[0]) {
	shift;
	last if (/^--$/);
	if    (/^-b/)		{ $basedir = &get_option("-basedir");	} 
	elsif (/^-d/)		{ $dirtreeroot = &get_option("-dirtreeroot");	}
        elsif (/^-e/)           { $executefile = &get_option("-executable");   }
        elsif (/^-t/)           { $input_template = &get_option("-template");   }
	elsif (/^-h/)		{ &usage("help")	} 
	elsif (/^[a-zA-Z]*/)    { 
	  print "can't specify more then one target!" if ($dirtreeroot ne "");
	}
	else 		        { print "unknown argument";          }
	
}

if ($basedir eq ""){ $basedir=$basedir_def;}
if ($dirtreeroot eq ""){ $dirtreeroot=$dirtreeroot_def;}
if ($executefile eq ""){ $executefile=$executefile_def;}
if ($input_template eq ""){ $input_template=$input_template_def;}

# BASIC DEFINITIONS

%pdgnumber = (  
     "d" => 1, "u" => 2, "s" => 3, "c" => 4, "b" => 5,
     "d_" => -1, "u_" => -2, "s_" => -3, "c_" => -4, "b_" => -5, 
     "e" => 11, "nue" => 12 , "mu" => 13, "num" => 14, "tau" => 15, "nut" => 17,
     "e_" => -11, "nue_" => -12 , "mu_" => -13, "num_" => -14,
     "tau_" => -15, "nut_" => -17 
     );
                 
                 
%antiparticle = ( 
     "d" => "d_", "u" => "u_", "s" => "s_", "c" => "c_", "b" => "b_",              
     "d_" => "d", "u_" => "u", "s_" => "s", "c_" => "c", "b_" => "b", 
     "e" => "e_", "nue" => "nue_" , "mu" => "mu_", "num" => "num_",
     "tau" => "tau_", "nut" => "nut_",
     "e_" => "e", "nue_" => "nue" , "mu_" => "mu", "num_" => "num",
     "tau_" => "tau", "nut_" => "nut" );

%group = (
                    "c s_ d u_ c s_" => "01",
                    "u u_ u u_ c s_" => "02",
                    "u u_ c c_ c s_" => "03",
                    "u u_ s s_ c s_" => "04",
                    "u u_ b b_ c s_" => "05",
                    "d d_ d d_ c s_" => "06",
                    "d d_ c c_ c s_" => "07",
                    "d d_ s s_ c s_" => "08",
                    "d d_ b b_ c s_" => "09",                  
                    "c c_ c c_ c s_" => "10",
                    "c c_ b b_ c s_" => "11",
                    "s s_ s s_ c s_" => "12",
                    "s s_ b b_ c s_" => "13",
                    "b b_ b b_ c s_" => "14",
                    "u u_ d d_ c s_" => "15",
                    "c c_ s s_ c s_" => "16"
);

@processes_lnu = (

# here one can choose just one group

#                    "c s_ d u_ c s_"
#                    "u u_ u u_ c s_"
#                    "u u_ c c_ c s_"
#                    "u u_ s s_ c s_"
#                    "u u_ b b_ c s_"
#                    "d d_ d d_ c s_"
#                    "d d_ c c_ c s_"
#                    "d d_ s s_ c s_"
#                    "d d_ b b_ c s_"
#                    "c c_ c c_ c s_"
#                    "c c_ b b_ c s_"
#                    "s s_ s s_ c s_"
#                    "s s_ b b_ c s_"
#                    "b b_ b b_ c s_"
#                    "u u_ d d_ c s_"
#                    "c c_ s s_ c s_"


#here one chooses all processes. Processes of the group 10, 12 and 16 come
# first as they are longer (more than 1000 graphs each).
# For this reason it is better to use for submitting SGEfile instead of  SGEall

                    "c c_ s s_ c s_",
                    "s s_ s s_ c s_",
                    "c c_ c c_ c s_",
                    "u u_ d d_ c s_",
                    "b b_ b b_ c s_",
                    "s s_ b b_ c s_",
                    "c c_ b b_ c s_",
                    "d d_ b b_ c s_",
                    "d d_ s s_ c s_",
                    "d d_ c c_ c s_",
                    "d d_ d d_ c s_",
                    "u u_ b b_ c s_",
                    "u u_ s s_ c s_",
                    "u u_ c c_ c s_",
                    "u u_ u u_ c s_",
                    "c s_ d u_ c s_"
                    );
                    
                    
# FIND PROCESSES

@process_list = ();
@process_idplist = ();
                   
foreach $proc_lnu (@processes_lnu){
@part = split(/ /,$proc_lnu);
$length = $#part;
$grouptag = $group{$proc_lnu};
#print "+++++++++++++++++++++++++++++++++++++++++++\n";
#print "string= $proc_lnu\n";
#print "+++++++++++++++++++++++++++++++++++++++++++\n";
#print "$proc_lnu becomes $part[0] x $part[1] xxx $part[5]\n";
#print "last element of array has label $length\n";

@in_state=();
for(my $i1=0;$i1 < $length;$i1++){
#print "string= $proc_lnu\n";
  @partlocal = @part;
  $first = $antiparticle{splice(@partlocal,$i1,1)};
                                # @partlocal shortened by one element
#print "first = $first\n";
#print "$partlocal['$i1'] is now in position $i1\n";
  for(my $i2=$i1;$i2 < $length;$i2++){
    @partlocal2 = @partlocal;
    $second = $antiparticle{splice(@partlocal2,$i2,1)};
                                         # @partlocal2 now contains
                                         # the final state in split form
#print "second = $second\n";
    $state1 = $first.$second;
    $state2 = $second.$first;
#print "first initial state : $state1\n";
#print "second initial state: $state2\n";
    $found = "false";
    foreach $state (@in_state){
#print "check if found: $state\n";
      if($state eq $state1 || $state eq $state2){
           $found = "true";}
           }
    if($found eq "false"){
       push @in_state,$state1;
       $string = $state1."-";
       $string .= join "",@partlocal2;
       $string .= $grouptag;
#print "reaction:  $string\n";
       push @process_list,$string;
#print "initial partlocal2 @partlocal2\n";
       unshift  @partlocal2, $first,$second;
# Add leptons
       push @partlocal2, "mu","num_";
#print "partlocal2 with initial and leptons @partlocal2\n";
       @process_idp=();
       foreach (@partlocal2) {
       push @process_idp, $pdgnumber{$_};
                             }
       $idp = "  ";
       $idp .=  join " ",@process_idp;
       $process_idplist{$string} = $idp;
#print "process: @partlocal2\n";
#print "idp: $idp\n";
       }
       
     }    # end of loop on $i2
   }      # end of loop on $i1
#print "******************************************\n\n";
#print "group: $proc_lnu\n\n";
#foreach $process (@process_list) {print "$process\n"}
}        # end of loop on group list


# CREATE ROOT DIRECTORY OF THE TREE (If necessary)

system("mkdir $dirtreeroot") unless -d $dirtreeroot;

# CREATE SGE SUBMISSION FILE

$submitfile = $dirtreeroot."/".$SGEfilename;
open(SGEFILE,"> $submitfile");

print SGEFILE "\#!/bin/bash\n\n";

# CREATE SUBDIRECTORIES AND INPUT/RUN FILES

foreach $dirname (@process_list){
# Add basedir and dirtreeroot
#    $name = $basedir."/".$dirtreeroot."/".$dirname;
    $name = $dirtreeroot."/".$dirname;
#print "attemping creation of $name\n";
    system("mkdir $name");
# Write process idp's in real input file
    open(WRITEFILE,"> $name/$inputfilename");
    print WRITEFILE "****** IF (IONESH.EQ.0) THEN\n";
    print WRITEFILE "*  CALL iread('iproc',iproc,8)    ! process\n";
#    $idp_string = shift @process_idp;
    $idp_string = $process_idplist{$dirname};
#print "idpstring: $idp_string\n";
    print WRITEFILE "iproc  $idp_string\n";
    print WRITEFILE "****** ENDIF\n\n";
# copy input template into real input file
    $remember = $/;                       # Note current record separator
    $/ = undef;                           # Ignore newline characters
    open(FILE,"$input_template") || die "Can't open file: $!\n";
                                        # open file if possible
    $_ = <FILE>;                          # Load entire file into matchspace
    close(FILE);
    print WRITEFILE $_;
    $/ = $remember;                       # Recognise separator again
    close(WRITEFILE);

# create run file
    open(WRITEFILE,"> $name/$runfilename");
    print WRITEFILE "\#!/bin/csh\n\n";
    print WRITEFILE "cd $dirtreeroot/$dirname\n";
    print WRITEFILE "echo \$HOSTNAME\n";
    print WRITEFILE "date\n";
    print WRITEFILE "unbuffer $basedir/$executefile\n";
    print WRITEFILE "date\n";
    close(WRITEFILE);

# add submission lines into SGE  file
    print SGEFILE "subcheck  $dirtreeroot/$dirname    $runfilename   slow\n";

# create symbolic link to cteq5m1.tbl
    system " ln -s  $basedir/cteq5m1.tbl $name/cteq5m1.tbl";
    system " ln -s  $basedir/cteq5l.tbl $name/cteq5l.tbl"; 
    }

# close SGE submission file
close(SGEFILE);

# SET PERMISSIONS
    system " chmod +x $submitfile";



##Create standard script which submits the whole lot
#$foreachfile = $dirtreeroot."/".$foreach_script;
#
#open(SGEFILE,"> $foreachfile");
#print SGEFILE "\#!/bin/csh\n\n";
#print SGEFILE 
#"foreach i ( */ )\n
#  echo \$i\n
#  subcheck  $dirtreeroot/\$i    $runfilename slow\n
#end\n 
#exit\n";

#    system " chmod +x $foreachfile";





# SUBROUTINES

sub get_option {
	&usage("missing argument for $_[0]") if ($#ARGV==-1) ;
	$result = $ARGV[0];
	shift @ARGV;	 
	return $result;
}


sub usage {
  local($mess) = @_;

  print "setupdirSGE.pl: $mess\n\n";

  print <<"_EOF_";
usage:  perl setupdirSGE.pl [-options ...] 

where options include:                                    Defaults -
    -basedir      directory which contains the exe file
                  and CTEQ pdf files   (full pathname)     ./
    -dirtreeroot  root of new tree (full pathname)         mh0
    -template     input template file                      template.st0
    -executable   executable file                          phase.exe
    -help         prints usage details                      -
long options can be abbreviated up to one letter.

Possible exit codes:
           0 - normal, success           1 - bad arguments (usage)
           2 - interrupted               
_EOF_

 exit 1;
}

# end  
