
      SUBROUTINE proc

      IMPLICIT REAL*8 (a-b,d-h,o-z)
      implicit double complex (c)

C   Cteq convention:
C   Iparton  is the parton label (5, 4, 3, 2, 1, 0, -1, ......, -5)
C                            for (b, c, s, d, u, g, u_bar, ..., b_bar),
C
C   PDG  convention:
C   1=d, 2=u, 3=s, 4=c, 5=b, 6=t
C   11=e-, 12=v_e, 13=mu-, 14=v_mu, 15=tau-, 16=v_tau
C   all antiparticles have the same number but opposite sign
C   moreover: 21=gluon, 22=gamma, 23=Z0, 24=W+, 25=h


* Phase space list and numbering:

* t(00) W+ W-                                ialfa(1)  1_1_4   
* t(0W) Z W-  and t(0W) Z W+                 ialfa(2)  1_1_4
* t(W0) Z W-  and t(W0) Z W+                 ialfa(3)  1_1_4
* t(WW) W- W- and t(WW) W+ W+                ialfa(4)  1_1_4
* t(WW) W+ W-                                ialfa(5)  1_1_4
* W- -> W- W+ W-                             ialfa(6)  2_4
* W- -> W- W+ W-                             ialfa(7)  2_4
* W+ -> W+ W+ W-                             ialfa(8)  2_4
* W+ -> W+ W+ W-                             ialfa(9)  2_4
* Z  -> Z  W+ W-                             ialfa(10) 2_4
* W- -> Z  Z  W- and W+ -> Z  Z  W+          ialfa(11) 2_4
* t(W0) t                                    ialfa(12) 1_1_31
* t(0W) t                                    ialfa(13) 1_1_31
* t(W ) Z  t                                 ialfa(14) 1_2_3
* t(0 ) W- t                                 ialfa(15) 1_2_3
* t(0 ) W+ t                                 ialfa(16) 1_2_3
* W- -> t  Z  and  W+ -> t  Z                ialfa(17) 3_3
* Z  -> t  t                                 ialfa(18) 3_3


C INPUT (integer vector)
* iproc (e.g.  1 1 -> 2 2 3 -4 13 -14  for   d d -> u u s c~ mu- vm~)


C OUTPUT(igroup,n_kphs,ialfa,idp,idp_inout,iorder,{input_phasespace})

* igroup=group number
* n_kphs=total number of phase spaces
* ialfa(i)=0/1 (off/on phase space)
* idp(i)= particle identity according to the amplitude order
* idp_inout(j,i)=-1/+1  (incoming/outgoing particles)
* iorder(j,i) gives the momenta order as passed to the amplitude 
* i.e. iorder(j_phsp,iphs_ind)=amplitude_index 
* {input_phasespace} = set of phase_space input 
* e.g. rm1(i),gam1(i) etc.
 

      INCLUDE 'common.h'
      INCLUDE 'common_subproc.h'
      COMMON/phaones/ionesh
* the following variables represent the input of the various phase spaces
* rmN=invariant mass of the N-fork
* gamN=width of the N-fork
* spmNlo=lower bound of the resonance in the N-fork
* spmNhi=upper bound of the resonance in the N-fork
* in the multi-mapping strategy, 
* [spmNlo,spmNhi] defines the range where the mapping on the resonance is on
* rmtN=boson mass in the "N" t-channel mapping
* expN=value of the exponent used in the "N" t-channel mapping
* sptN=separaction between mapped and flat regions in the "N" t-channel 
* see the specific phase space for details.

      COMMON/phspinput_proc/
     &     rm1(mxnphs),gam1(mxnphs),spm1lo(mxnphs),spm1hi(mxnphs),
     &  rm11a(mxnphs),gam11a(mxnphs),spm11alo(mxnphs),spm11ahi(mxnphs),
     &  rm11b(mxnphs),gam11b(mxnphs),spm11blo(mxnphs),spm11bhi(mxnphs),
     &  rm12a(mxnphs),gam12a(mxnphs),spm12alo(mxnphs),spm12ahi(mxnphs),
     &  rm12b(mxnphs),gam12b(mxnphs),spm12blo(mxnphs),spm12bhi(mxnphs),
     &  rm2(mxnphs),gam2(mxnphs),spm2lo(mxnphs),spm2hi(mxnphs),
     &  rm2a(mxnphs),gam2a(mxnphs),spm2alo(mxnphs),spm2ahi(mxnphs),
     &  rm2b(mxnphs),gam2b(mxnphs),spm2blo(mxnphs),spm2bhi(mxnphs),
     &  rm21a(mxnphs),gam21a(mxnphs),spm21alo(mxnphs),spm21ahi(mxnphs),
     &  rm21b(mxnphs),gam21b(mxnphs),spm21blo(mxnphs),spm21bhi(mxnphs),
     &  rm111(mxnphs),gam111(mxnphs),spm111lo(mxnphs),spm111hi(mxnphs),
     &  rm11(mxnphs),gam11(mxnphs),spm11lo(mxnphs),spm11hi(mxnphs),
     &  rmta(mxnphs),expa(mxnphs),spta,rmtb(mxnphs),expb(mxnphs),sptb



      CHARACTER *40 phsp(mxnphs,mxnphs)

      DIMENSION Q(0:8)
      DIMENSION Le(0:8),iFa(0:8),idhep(0:8),idout(8),inout(8,mxnphs)
      DIMENSION kmax(mxnphs),l2max(mxnphs,0:mxnphs)
      DIMENSION iord(mxnphs,mxnphs,8)
      DIMENSION ii(-3:mxnphs),jj(-3:mxnphs),kk(-3:mxnphs),
     &          ii1(-3:mxnphs),jj1(-3:mxnphs),kk1(-3:mxnphs),
     &          ii2(-3:mxnphs),jj2(-3:mxnphs),kk2(-3:mxnphs)
      DIMENSION idb(6)
      DIMENSION n(-16:16)
      DIMENSION iwp(mxnphs,mxnphs,mxnphs),jwp(mxnphs,mxnphs,mxnphs),
     &     iwm(mxnphs,mxnphs,mxnphs),
     &  jwm(mxnphs,mxnphs,mxnphs),iz(mxnphs,mxnphs,mxnphs),jz(mxnphs,
     &     mxnphs,mxnphs),
     &  ib(mxnphs,mxnphs,mxnphs),jb(mxnphs,mxnphs,mxnphs),it0w(mxnphs,
     &     mxnphs),jt0w(mxnphs,mxnphs),
     &  itw0(mxnphs,mxnphs),jtw0(mxnphs,mxnphs),itww(mxnphs,mxnphs),
     &     jtww(mxnphs,mxnphs),
     &  it00(mxnphs,mxnphs),jt00(mxnphs,mxnphs)
      
* these variables are used in the phase space. Their value is specified
* once for all in the DATA, and it is not computed anymore
      DATA expa/mxnphs*0.8d0/, expb/mxnphs*0.8d0/ 

*** define phase space input
* Coefficient of the Higgs width in spmXlo and spmXhi: linear 
* interpolation.
* Here we define the number of Higgs widths to be taken around the Higgs
* mass, for defining the phase space region where the mapping on the Higgs
* resonance must be switched on.  
* variables {xhiggswidth, spmNlo, spmNhi where N=h,w,z,t} are defined
* just once, independently on the process.
  
      IF(rmh.LT.150.d0)THEN
        xhiggswidth=30.d0+(rmh-120.d0)/(150.d0-120.d0)*(20.d0-30.d0)
      ELSEIF(rmh.LT.250.d0)THEN
        xhiggswidth=20.d0+(rmh-150.d0)/(250.d0-150.d0)*(3.d0-20.d0)
      ELSEIF(rmh.LT.1.d3)THEN
        xhiggswidth=3.d0+(rmh-250.d0)/(1.d3-250.d0)*(1.d0-3.d0)
      ENDIF
      IF(rmh.LT.1.d3.AND.rmh.GE.0.d0)THEN
        spmhlo=rmh-xhiggswidth*gamh
        spmhhi=rmh+xhiggswidth*gamh
      ELSE
* IF rmh > 1.d3 GeV or rmh < 0 the Higgs resonance is not mapped
* When both a Z and a Higgs resonance are possible (n5 > 0) only the first
* is activated
        spmhlo=0.d0
        spmhhi=0.d0
      ENDIF
      spmhlo=max(spmhlo,0.d0)
      spmhhi=min(spmhhi,ecoll)
      spmwlo=rmw-5.d0*gamw
      spmwhi=rmw+5.d0*gamw
      spmzlo=rmz-5.d0*gamz
      spmzhi=rmz+5.d0*gamz
      spmtlo=rmt-5.d0*gamt
      spmthi=rmt+5.d0*gamt

* continue defining phase space inputs
* this part needs to know the input iproc.
* phase space {ia=2,3,8,9,11,14,16} need iproc to know how many b's are
* in the process. The number of b's determines whether or not the 
* mapping on the Higgs resonance must be switched on in the phase space.

      DO ia=1,18
        IF(ia.EQ.1)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rmta(ia)=0.d0
          rmtb(ia)=0.d0
        ELSE IF(ia.EQ.2)THEN
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)n5=n5+1             
          ENDDO !j
          IF(ABS(iproc(1)).EQ.5)n5=n5-1
          IF(ABS(iproc(2)).EQ.5)n5=n5-1

          rm1(ia)=0.d0
          gam1(ia)=0.d0
          spm1lo(ia)=0.d0
          spm1hi(ia)=0.d0
          IF(n5.EQ.0 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm11a(ia)=0.d0
            gam11a(ia)=0.d0
            spm11alo(ia)=0.d0
            spm11ahi(ia)=0.d0
            rm11b(ia)=rmz
            gam11b(ia)=gamz
            spm11blo(ia)=spmzlo
            spm11bhi(ia)=spmzhi
          ELSE
            rm11a(ia)=rmz
            gam11a(ia)=gamz
            spm11alo(ia)=spmzlo
            spm11ahi(ia)=spmzhi
            rm11b(ia)=rmh
            gam11b(ia)=gamh
            spm11blo(ia)=spmhlo
            spm11bhi(ia)=spmhhi
          ENDIF
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rmta(ia)=0.d0
          rmtb(ia)=rmw
        ELSE IF(ia.EQ.3)THEN
          rm1(ia)=0.d0
          gam1(ia)=0.d0
          spm1lo(ia)=0.d0
          spm1hi(ia)=0.d0
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)n5=n5+1             
          ENDDO !j
          IF(ABS(iproc(1)).EQ.5)n5=n5-1
          IF(ABS(iproc(2)).EQ.5)n5=n5-1
          IF(n5.EQ.0 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm11a(ia)=0.d0
            gam11a(ia)=0.d0
            spm11alo(ia)=0.d0
            spm11ahi(ia)=0.d0
            rm11b(ia)=rmz
            gam11b(ia)=gamz
            spm11blo(ia)=spmzlo
            spm11bhi(ia)=spmzhi
          ELSE
            rm11a(ia)=rmz
            gam11a(ia)=gamz
            spm11alo(ia)=spmzlo
            spm11ahi(ia)=spmzhi
            rm11b(ia)=rmh
            gam11b(ia)=gamh
            spm11blo(ia)=spmhlo
            spm11bhi(ia)=spmhhi
          ENDIF
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rmta(ia)=rmw
          rmtb(ia)=0.d0
        ELSE IF(ia.EQ.4)THEN
          rm1(ia)=0.d0
          gam1(ia)=0.d0
          spm1lo(ia)=0.d0
          spm1hi(ia)=0.d0
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rmta(ia)=rmw
          rmtb(ia)=rmw
        ELSE IF(ia.EQ.5)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rmta(ia)=rmw
          rmtb(ia)=rmw
        ELSE IF(ia.EQ.6)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
        ELSE IF(ia.EQ.7)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
        ELSE IF(ia.EQ.8.OR.ia.EQ.9)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
        ELSE IF(ia.EQ.10)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm12a(ia)=0.d0
          gam12a(ia)=0.d0
          spm12alo(ia)=0.d0
          spm12ahi(ia)=0.d0
          rm12b(ia)=rmw
          gam12b(ia)=gamw
          spm12blo(ia)=spmwlo
          spm12bhi(ia)=spmwhi
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)n5=n5+1
          ENDDO !j
          IF(n5.EQ.0 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm2a(ia)=0.d0
            gam2a(ia)=0.d0
            spm2alo(ia)=0.d0
            spm2ahi(ia)=0.d0
            rm2b(ia)=rmz
            gam2b(ia)=gamz
            spm2blo(ia)=spmzlo
            spm2bhi(ia)=spmzhi
          ELSE
            rm2a(ia)=rmz
            gam2a(ia)=gamz
            spm2alo(ia)=spmzlo
            spm2ahi(ia)=spmzhi
            rm2b(ia)=rmh
            gam2b(ia)=gamh
            spm2blo(ia)=spmhlo
            spm2bhi(ia)=spmhhi
          ENDIF
        ELSE IF(ia.EQ.11)THEN
          rm1(ia)=rmh
          gam1(ia)=gamh
          spm1lo(ia)=spmhlo
          spm1hi(ia)=spmhhi
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)THEN
              n5=n5+1
            ENDIF
          ENDDO !j
          IF(n5.NE.4 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm11a(ia)=0.d0
            gam11a(ia)=0.d0
            spm11alo(ia)=0.d0
            spm11ahi(ia)=0.d0
            rm11b(ia)=rmz
            gam11b(ia)=gamz
            spm11blo(ia)=spmzlo
            spm11bhi(ia)=spmzhi
          ELSE IF(n5.EQ.4)THEN
            rm11a(ia)=rmz
            gam11a(ia)=gamz
            spm11alo(ia)=spmzlo
            spm11ahi(ia)=spmzhi
            rm11b(ia)=rmh
            gam11b(ia)=gamh
            spm11blo(ia)=spmhlo
            spm11bhi(ia)=spmhhi
          ENDIF
          IF(n5.EQ.0 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm12a(ia)=0.d0
            gam12a(ia)=0.d0
            spm12alo(ia)=0.d0
            spm12ahi(ia)=0.d0
            rm12b(ia)=rmz
            gam12b(ia)=gamz
            spm12blo(ia)=spmzlo
            spm12bhi(ia)=spmzhi
          ELSE
            rm12a(ia)=rmz
            gam12a(ia)=gamz
            spm12alo(ia)=spmzlo
            spm12ahi(ia)=spmzhi
            rm12b(ia)=rmh
            gam12b(ia)=gamh
            spm12blo(ia)=spmhlo
            spm12bhi(ia)=spmhhi
          ENDIF
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
        ELSE IF(ia.EQ.12)THEN
          rm1(ia)=0.d0
          gam1(ia)=0.d0
          spm1lo(ia)=0.d0
          spm1hi(ia)=0.d0
          rm11(ia)=rmt
          gam11(ia)=gamt
          spm11lo(ia)=spmtlo
          spm11hi(ia)=spmthi
          rm111(ia)=rmw
          gam111(ia)=gamw
          spm111lo(ia)=spmwlo
          spm111hi(ia)=spmwhi
          rmta(ia)=rmw
          rmtb(ia)=0.d0
        ELSE IF(ia.EQ.13)THEN
          rm1(ia)=0.d0
          gam1(ia)=0.d0
          spm1lo(ia)=0.d0
          spm1hi(ia)=0.d0
          rm11(ia)=rmt
          gam11(ia)=gamt
          spm11lo(ia)=spmtlo
          spm11hi(ia)=spmthi
          rm111(ia)=rmw
          gam111(ia)=gamw
          spm111lo(ia)=spmwlo
          spm111hi(ia)=spmwhi
          rmta(ia)=0.d0
          rmtb(ia)=rmw
        ELSE IF(ia.EQ.14)THEN
          rm1(ia)=rmt
          gam1(ia)=gamt
          spm1lo(ia)=spmtlo
          spm1hi(ia)=spmthi
          rm11(ia)=rmw
          gam11(ia)=gamw
          spm11lo(ia)=spmwlo
          spm11hi(ia)=spmwhi
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)n5=n5+1
          ENDDO !j
          IF(ABS(iproc(1)).EQ.5)n5=n5-1
          IF(ABS(iproc(2)).EQ.5)n5=n5-1
          IF(n5.EQ.0 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm2a(ia)=0.d0
            gam2a(ia)=0.d0
            spm2alo(ia)=0.d0
            spm2ahi(ia)=0.d0
            rm2b(ia)=rmz
            gam2b(ia)=gamz
            spm2blo(ia)=spmzlo
            spm2bhi(ia)=spmzhi
          ELSE
            rm2a(ia)=rmz
            gam2a(ia)=gamz
            spm2alo(ia)=spmzlo
            spm2ahi(ia)=spmzhi
            rm2b(ia)=rmh
            gam2b(ia)=gamh
            spm2blo(ia)=spmhlo
            spm2bhi(ia)=spmhhi
          ENDIF
          rmta(ia)=rmw
        ELSE IF(ia.EQ.15)THEN
          rm1(ia)=rmt
          gam1(ia)=gamt
          spm1lo(ia)=spmtlo
          spm1hi(ia)=spmthi
          rm11(ia)=rmw
          gam11(ia)=gamw
          spm11lo(ia)=spmwlo
          spm11hi(ia)=spmwhi
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
          rmta(ia)=0.d0
        ELSE IF(ia.EQ.16)THEN
          rm1(ia)=rmt
          gam1(ia)=gamt
          spm1lo(ia)=spmtlo
          spm1hi(ia)=spmthi
          rm11(ia)=rmw
          gam11(ia)=gamw
          spm11lo(ia)=spmwlo
          spm11hi(ia)=spmwhi
          rm2a(ia)=0.d0
          gam2a(ia)=0.d0
          spm2alo(ia)=0.d0
          spm2ahi(ia)=0.d0
          rm2b(ia)=rmw
          gam2b(ia)=gamw
          spm2blo(ia)=spmwlo
          spm2bhi(ia)=spmwhi
          rmta(ia)=0.d0
        ELSE IF(ia.EQ.17)THEN
          rm1(ia)=rmt
          gam1(ia)=gamt
          spm1lo(ia)=spmtlo
          spm1hi(ia)=spmthi
          rm11(ia)=rmw
          gam11(ia)=gamw
          spm11lo(ia)=spmwlo
          spm11hi(ia)=spmwhi
          rm2(ia)=0.d0
          gam2(ia)=0.d0
          spm2lo(ia)=0.d0
          spm2hi(ia)=0.d0
          n5=0
          DO j=3,8
            IF(ABS(iproc(j)).EQ.5)THEN
              n5=n5+1
            ENDIF
          ENDDO !j
          IF(n5.NE.4 .OR. rmh.GT.1.d3.OR.rmh.LT.0.d0)THEN
            rm21a(ia)=0.d0
            gam21a(ia)=0.d0
            spm21alo(ia)=0.d0
            spm21ahi(ia)=0.d0
            rm21b(ia)=rmz
            gam21b(ia)=gamz
            spm21blo(ia)=spmzlo
            spm21bhi(ia)=spmzhi
          ELSE
            rm21a(ia)=rmz
            gam21a(ia)=gamz
            spm21alo(ia)=spmzlo
            spm21ahi(ia)=spmzhi
            rm21b(ia)=rmh
            gam21b(ia)=gamh
            spm21blo(ia)=spmhlo
            spm21bhi(ia)=spmhhi
          ENDIF
        ELSE IF(ia.EQ.18)THEN
          rm1(ia)=rmt
          gam1(ia)=gamt
          spm1lo(ia)=spmtlo
          spm1hi(ia)=spmthi
          rm11a(ia)=0.d0
          gam11a(ia)=0.d0
          spm11alo(ia)=0.d0
          spm11ahi(ia)=0.d0
          rm11b(ia)=rmw
          gam11b(ia)=gamw
          spm11blo(ia)=spmwlo
          spm11bhi(ia)=spmwhi
          rm2(ia)=rmt
          gam2(ia)=gamt
          spm2lo(ia)=spmtlo
          spm2hi(ia)=spmthi
          rm21a(ia)=0.d0
          gam21a(ia)=0.d0
          spm21alo(ia)=0.d0
          spm21ahi(ia)=0.d0
          rm21b(ia)=rmw
          gam21b(ia)=gamw
          spm21blo(ia)=spmwlo
          spm21bhi(ia)=spmwhi
        ENDIF !ia
      ENDDO !ia
* end of phase space input

      IF(ionesh.EQ.0)THEN

* particle identity
      do i=1,8
        idhep(i)=iproc(i)
      enddo
      
* 'outgoing process' identity number: the sign of the incoming particles
* is reversed in order to consider all outgoing particles. 
      do i=1,2
        idout(i)=-idhep(i)
      enddo
      do i=3,8
        idout(i)=idhep(i)
      enddo

* charge (Q), lepton number (Le) and family (iFa) determination:
* these variables are used to reconstruct the forks in the process.
      do i=1,8
      if(abs(idhep(i)).ge.1.and.abs(idhep(i)).le.5)then
        Le(i)=0
        if(idhep(i).eq.1.or.idhep(i).eq.3.or.idhep(i).eq.5)then
          Q(i)=-1.d0/3.d0
        else if(idhep(i).eq.-1.or.idhep(i).eq.-3.or.
     &          idhep(i).eq.-5)then
          Q(i)=1.d0/3.d0
        endif
        if(idhep(i).eq.2.or.idhep(i).eq.4)then
          Q(i)=2.d0/3.d0
        else if(idhep(i).eq.-2.or.idhep(i).eq.-4)then
          Q(i)=-2.d0/3.d0
        endif
        if(abs(idhep(i)).eq.1.or.abs(idhep(i)).eq.2)iFa(i)=1
        if(abs(idhep(i)).eq.3.or.abs(idhep(i)).eq.4)iFa(i)=2
        if(abs(idhep(i)).eq.5)iFa(i)=3
      else
        Le(i)=1
        if(idhep(i).eq.11.or.idhep(i).eq.13.or.idhep(i).eq.15)then
          Q(i)=-1.d0
        else if(idhep(i).eq.-11.or.idhep(i).eq.-13.or.
     &          idhep(i).eq.-15)then
          Q(i)=1.d0
        else if(abs(idhep(i)).eq.12.or.abs(idhep(i)).eq.14.or.
     &          abs(idhep(i)).eq.16)then
          Q(i)=0.d0
        endif
        if(abs(idhep(i)).eq.11.or.abs(idhep(i)).eq.12)iFa(i)=1
        if(abs(idhep(i)).eq.13.or.abs(idhep(i)).eq.14)iFa(i)=2
        if(abs(idhep(i)).eq.15.or.abs(idhep(i)).eq.16)iFa(i)=3
      endif
      enddo !i

* counting the number of fermions of each kind, to be used in the group 
* determination
      do i=-16,16
        n(i)=0
      enddo !i

      do k=-16,16
        if((abs(k).lt.11.and.abs(k).gt.5).or.k.eq.0)goto 100
        do i=1,8
          if(idout(i).eq.k)n(k)=n(k)+1
        enddo
 100  enddo




* group(1:16) determination:

      DO i=0,2,2
        DO j=-1,1,2
          IF(n((-1-i)*j).EQ.2.AND.n((2+i)*j).EQ.2.AND.
     &       n((-4+i)*j).EQ.1.AND.n((3-i)*j).EQ.1.AND.
     &       ((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.
     &       (n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.
     &       (n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=1
          ELSE IF(n((2+i)*j).EQ.2.AND.n((-2-i)*j).EQ.2.AND.
     &            n((4-i)*j).EQ.1.AND.n((-3+i)*j).EQ.1.AND.
     &            ((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.
     &            (n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.
     &            (n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=2
          ELSE IF(n((2+i)*j).EQ.1.AND.n((-2-i)*j).EQ.1.AND.
     &            n((4-i)*j).EQ.2.AND.n((-4+i)*j).EQ.1.AND.
     &            n((-3+i)*j).EQ.1.AND.((n(13*j).EQ.1.AND.
     &            n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.
     &            n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.
     &            n(-16*j).EQ.1)))THEN
            igroup=3
          ELSE IF(n((2+i)*j).EQ.1.AND.n((-2-i)*j).EQ.1.AND.
     &            n((4-i)*j).EQ.1.AND.n((-3+i)*j).EQ.2.AND.
     &            n((3-i)*j).EQ.1.AND.((n(13*j).EQ.1.AND.
     &            n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.
     &            n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.
     &            n(-16*j).EQ.1)))THEN
            igroup=4
          ELSE IF(n((2+i)*j).EQ.1.AND.n((-2-i)*j).EQ.1.AND.
     &            n((4-i)*j).EQ.1.AND.n((-3+i)*j).EQ.1.AND.
     &            n(5).EQ.1.AND.n(-5).EQ.1.AND.((n(13*j).EQ.1.AND.
     &            n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.n(-12*j).EQ.1).
     &            OR.(n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=5
          ELSE IF(n((1+i)*j).EQ.2.AND.n((-1-i)*j).EQ.2.AND.
     &            n((4-i)*j).EQ.1.AND.n((-3+i)*j).EQ.1.AND.
     &            ((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.(n(11*j).EQ.1.
     &            AND.n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.
     &            n(-16*j).EQ.1)))THEN
            igroup=6
          ELSE IF(n((1+i)*j).EQ.1.AND.n((-1-i)*j).EQ.1.AND.n((4-i)*j).
     &            EQ.2.AND.n((-4+i)*j).EQ.1.AND.n((-3+i)*j).EQ.1.AND.
     &            ((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.(n(11*j).EQ.1.
     &            AND.n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.
     &            n(-16*j).EQ.1)))THEN
            igroup=7
          ELSE IF(n((1+i)*j).EQ.1.AND.n((-1-i)*j).EQ.1.AND.n((3-i)*j).
     &            EQ.1.AND.n((-3+i)*j).EQ.2.AND.n((4-i)*j).EQ.1.AND.
     &            ((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.(n(11*j).EQ.1.
     &            AND.n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.n(-16*j).
     &            EQ.1)))THEN
            igroup=8
          ELSE IF(n((1+i)*j).EQ.1.AND.n((-1-i)*j).EQ.1.AND.n((4-i)*j).
     &            EQ.1.AND.n((-3+i)*j).EQ.1.AND.n(5).EQ.1.AND.n(-5).
     &            EQ.1.AND.((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.
     &            (n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.(n(15*j).EQ.1.
     &            AND.n(-16*j).EQ.1)))THEN
            igroup=9
          ELSE IF(n((4-i)*j).EQ.3.AND.n((-4+i)*j).EQ.2.AND.n((-3+i)*j).
     &            EQ.1.AND.((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.
     &            (n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.
     &            (n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=10
          ELSE IF(n((4-i)*j).EQ.2.AND.n((-4+i)*j).EQ.1.AND.n((-3+i)*j).
     &            EQ.1.AND.n(5).EQ.1.AND.n(-5).EQ.1.AND.((n(13*j).EQ.1.
     &            AND.n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.n(-12*j).
     &            EQ.1).OR.(n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=11
          ELSE IF(n((3-i)*j).EQ.2.AND.n((-3+i)*j).EQ.3.AND.n((4-i)*j).
     &            EQ.1.AND.((n(13*j).EQ.1.AND.n(-14*j).EQ.1).OR.
     &            (n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.
     &            (n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=12
          ELSE IF(n((3-i)*j).EQ.1.AND.n((-3+i)*j).EQ.2.AND.n((4-i)*j).
     &            EQ.1.AND.n(5).EQ.1.AND.n(-5).EQ.1.AND.((n(13*j).
     &            EQ.1.AND.n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.
     &            n(-12*j).EQ.1).OR.(n(15*j).EQ.1.AND.n(-16*j).EQ.1)))
     &            THEN
            igroup=13
          ELSE IF(n((4-i)*j).EQ.1.AND.n((-3+i)*j).EQ.1.AND.n(5).EQ.2.
     &            AND.n(-5).EQ.2.AND.((n(13*j).EQ.1.AND.n(-14*j).EQ.1)
     &            .OR.(n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.(n(15*j).
     &            EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=14
          ELSE IF(n((2+i)*j).EQ.1.AND.n((-2-i)*j).EQ.1.AND.n((1+i)*j).
     &            EQ.1.AND.n((-1-i)*j).EQ.1.AND.n((4-i)*j).EQ.1.AND.
     &            n((-3+i)*j).EQ.1.AND.((n(13*j).EQ.1.AND.n(-14*j).EQ.
     &            1).OR.(n(11*j).EQ.1.AND.n(-12*j).EQ.1).OR.(n(15*j).
     &            EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=15
          ELSE IF(n((4-i)*j).EQ.2.AND.n((-4+i)*j).EQ.1.AND.n((3-i)*j).
     &            EQ.1.AND.n((-3+i)*j).EQ.2.AND.((n(13*j).EQ.1.AND.
     &            n(-14*j).EQ.1).OR.(n(11*j).EQ.1.AND.n(-12*j).EQ.1).
     &            OR.(n(15*j).EQ.1.AND.n(-16*j).EQ.1)))THEN
            igroup=16
          ENDIF
        ENDDO
      ENDDO
      
      print*,'group=',igroup

      if(igroup.eq.1)then
        print*,'number of diagrams: 202'
        print*,'kind: 4W'
      else if(igroup.eq.2.or.igroup.eq.3.or.igroup.eq.4.or.
     &  igroup.eq.6.or.igroup.eq.7.or.igroup.eq.8)then
        print*,'number of diagrams: 422'
        print*,'kind: 2Z2W'
      else if(igroup.eq.5.or.igroup.eq.9)then
        print*,'number of diagrams: 233'
        print*,'kind: 2Z2W'
      else if(igroup.eq.10.or.igroup.eq.12)then
        print*,'number of diagrams: 1266'
        print*,'kind: 2Z2W'
      else if(igroup.eq.11.or.igroup.eq.13)then
        print*,'number of diagrams: 466'
        print*,'kind: 2Z2W'
      else if(igroup.eq.14)then
        print*,'number of diagrams: 610'
        print*,'kind: 2Z2W'
      else if(igroup.eq.15)then
        print*,'number of diagrams: 312'
        print*,'kind: Mixed'
      else if(igroup.eq.16)then
        print*,'number of diagrams: 1046'
        print*,'kind: Mixed'
      else
        print*,'error'
        stop
      endif

* initializing output variables:

      do i=1,8
        do j=1,mxnphs
          inout(i,j)=1
          iorder(i,j)=0
        enddo
      enddo

      do j=1,mxnphs
        ialfa(j)=0
      enddo

* Phase space determination:
* scanning the vector iproc, particles are sequentially recombined in pairs
* in order to see if they can form a boson.
* This procedure is done in three steps. First one considers the two initial 
* particles. If they can form a boson, we have an s-channel phase space.
* As second step, we consider initial and final particles. Starting from 
* the incoming particles, we scan all outgoing particles to see if one or
* two bosons in t-channel can be formed. Finally, we scan just the outgoing
* particles to extract the remaining possible forks.

* Phase space determination: 
* as soon as a fork is found, we fill the string phsp(nphsp,k*)(1:6) which
* defines the particular phase space. As an example, if we find that the
* two initial particles form a W+ boson, we start building our phase space
* string as phsp(nphsp,k)(1:6)='W+ ->. The same holds for W- and Z bosons.

      
* incoming pair: first step - the two initial particles are scanned to see
* if they can form a boson in s-channel
      DO k=1,mxnphs
        nphsp=0
        IF(iFa(1).EQ.iFa(2).AND.Le(1).EQ.Le(2))THEN
          IF((Q(1)+Q(2)).EQ.1.d0)THEN
            nphsp=nphsp+1
            phsp(nphsp,k)(1:6)='W+ -> '
            isch=1
          ELSE IF((Q(1)+Q(2)).EQ.-1.d0)THEN
            nphsp=nphsp+1
            phsp(nphsp,k)(1:6)='W- -> '
            isch=1
          ELSE IF((Q(1)+Q(2)).EQ.0.d0)THEN
            nphsp=nphsp+1
            phsp(nphsp,k)(1:6)='Z  -> '
            isch=1
          ENDIF
        ENDIF
      ENDDO !k
      nphsp_s=nphsp

* incoming-outgoing pairs: starting from one incoming particle, all final
* state particles are scanned in order to extract t-channel forks if they
* exist. 
* itNM=first final state particle to form a t-channel fork
* jtNM=second final state particle to form a t-channel fork
* with N=0,W and 0=gamma

* The phase space is given by the string phsp(nphsp,k1)(1:6)='t(NM) ' 
* where N,M can be 0(=gamma), W(=W+, W-) and blanc.

      do i=1,mxnphs
        do j=1,8
          iord(i,1,j)=0
        enddo !j
      enddo !i

      do k1=1,mxnphs
      nphsp=nphsp_s
      DO j=3,8
        IF(iFa(1).EQ.iFa(j).AND.Le(1).EQ.Le(j).AND.Q(1).EQ.Q(j))THEN
          k=j
          DO j1=3,8
            IF(j1.NE.k.AND.iFa(2).EQ.iFa(j1).AND.Le(2).EQ.Le(j1).AND.
     &         Q(2).EQ.Q(j1))THEN
              nphsp=nphsp+1
              phsp(nphsp,k1)(1:6)='t(00) '
              itch=1
              iord(nphsp,k1,3)=j
              iord(nphsp,k1,4)=j1
              it00(nphsp,k1)=j
              jt00(nphsp,k1)=j1
              IF(iFa(1).EQ.3.OR.iFa(2).EQ.3)THEN
                IF(idhep(1).EQ.-idhep(2))THEN
                  nphsp=nphsp+1
                  phsp(nphsp,k1)(1:6)='t(0 ) '
                  iord(nphsp,k1,3)=0
                  iord(nphsp,k1,4)=j1
                  it00(nphsp,k1)=j1
                  jt00(nphsp,k1)=j
                  nphsp=nphsp+1
                  phsp(nphsp,k1)(1:6)='t(0 ) '
                  iord(nphsp,k1,3)=j
                  iord(nphsp,k1,4)=0
                  it00(nphsp,k1)=j
                  jt00(nphsp,k1)=j1
                ELSE
                  nphsp=nphsp+1
                  phsp(nphsp,k1)(1:6)='t(0 ) '
                  IF(iFa(1).EQ.3)THEN
                    iord(nphsp,k1,3)=0
                    iord(nphsp,k1,4)=j1
                    it00(nphsp,k1)=j1
                    jt00(nphsp,k1)=j
                  ELSE
                    iord(nphsp,k1,3)=j
                    iord(nphsp,k1,4)=0
                    it00(nphsp,k1)=j
                    jt00(nphsp,k1)=j1
                  ENDIF
                ENDIF
              ENDIF
            ELSE IF(j1.NE.k.AND.iFa(2).EQ.iFa(j1).AND.
     &              Le(2).EQ.Le(j1).AND.abs(Q(2)-Q(j1)).eq.1)THEN
              nphsp=nphsp+1
              phsp(nphsp,k1)(1:6)='t(0W) '
              itch=1
              iord(nphsp,k1,3)=j
              iord(nphsp,k1,4)=j1
              it0W(nphsp,k1)=j
              jt0W(nphsp,k1)=j1
              IF(iFa(1).EQ.3)THEN
                nphsp=nphsp+1
                phsp(nphsp,k1)(1:6)='t( W) '
                iord(nphsp,k1,3)=0
                iord(nphsp,k1,4)=j1
                itW0(nphsp,k1)=j1
                jtW0(nphsp,k1)=j
              ENDIF
            ENDIF
          ENDDO
        ENDIF
        IF(iFa(1).EQ.iFa(j).AND.Le(1).EQ.Le(j).AND.
     &     ABS(Q(1)-Q(j)).EQ.1)THEN
          k=j
          DO j1=3,8
            IF(j1.NE.k.AND.iFa(2).EQ.iFa(j1).AND.Le(2).EQ.Le(j1).AND.
     &        Q(2).EQ.Q(j1))THEN
              nphsp=nphsp+1
              phsp(nphsp,k1)(1:6)='t(W0) '
              itch=1
              iord(nphsp,k1,3)=j
              iord(nphsp,k1,4)=j1
              itW0(nphsp,k1)=j
              jtW0(nphsp,k1)=j1
              IF(iFa(2).EQ.3)THEN
                nphsp=nphsp+1
                phsp(nphsp,k1)(1:6)='t(W ) '
                iord(nphsp,k1,3)=j
                iord(nphsp,k1,4)=0
                itW0(nphsp,k1)=j
                jtW0(nphsp,k1)=j1
              ENDIF
            ELSE IF(j1.NE.k.AND.iFa(2).EQ.iFa(j1).AND.
     &              Le(2).EQ.Le(j1).AND.ABS(Q(2)-Q(j1)).EQ.1)THEN
              nphsp=nphsp+1
              phsp(nphsp,k1)(1:6)='t(WW) '
              itch=1
              iord(nphsp,k1,3)=j
              iord(nphsp,k1,4)=j1
              itWW(nphsp,k1)=j
              jtWW(nphsp,k1)=j1
            ENDIF
          ENDDO !j1
        ENDIF
      ENDDO !j
      enddo !k1

* outgoing pairs:
* purely final state particles are scanned in order to reconstruct possible 
* forks.
* iN and jN variables are the identity of particles forming the "N" boson.
* N={wm,wp,z}

* The phase space is identified by the string phsp(np,k)(l1:l2)='N '
* where N can be W+, W-, Z, t.
* If more forks are found the string is filled in during the sequential scan
* with the symbols identifying the various forks.

      DO np=1,nphsp
        DO i=0,mxnphs
          ii1(i)=0
          jj1(i)=0
          ii2(i)=0
          jj2(i)=0
        ENDDO !i
        k=0      
        nwp=0
        nwm=0
        nz=0
        kmax(np)=0
        DO i=3,7
          DO j=i+1,8
            IF(nphsp.EQ.0)THEN
              l1=1
              l2=3
            ELSE
              l1=7
              l2=9
            ENDIF
            IF((i.NE.ii1(k).OR.j.NE.jj1(k)).AND.
     &         (i.NE.jj1(k).OR.j.NE.ii1(k)).AND.
     &         (i.NE.ii2(k).OR.j.NE.jj2(k)).AND.
     &         (i.NE.jj2(k).OR.j.NE.ii2(k)).AND.
     &         ((i.NE.iord(np,k+1,3).AND.j.NE.iord(np,k+1,4)).AND.
     &         (i.NE.iord(np,k+1,4).AND.j.NE.iord(np,k+1,3))).AND.
     &         iFa(i).EQ.iFa(j).AND.Le(i).EQ.Le(j))THEN
              IF(k.GT.1.AND.(i.EQ.ii1(k-1).AND.j.EQ.jj1(k-1)).OR.
     &           (i.EQ.jj1(k-1).AND.j.EQ.ii1(k-1)).OR.
     &           (i.EQ.ii2(k-1).AND.j.EQ.jj2(k-1)).OR.
     &           (i.EQ.jj2(k-1).AND.j.EQ.ii2(k-1)))GOTO 300
              IF(k.GT.2.AND.(i.EQ.ii1(k-2).AND.j.EQ.jj1(k-2)).OR.
     &           (i.EQ.jj1(k-2).AND.j.EQ.ii1(k-2)).OR.
     &           (i.EQ.ii2(k-2).AND.j.EQ.jj2(k-2)).OR.
     &           (i.EQ.jj2(k-2).AND.j.EQ.ii2(k-2)))GOTO 300
              IF(k.GT.3.AND.(i.EQ.ii1(k-3).AND.j.EQ.jj1(k-3)).OR.
     &           (i.EQ.jj1(k-3).AND.j.EQ.ii1(k-3)).OR.
     &           (i.EQ.ii2(k-3).AND.j.EQ.jj2(k-3)).OR.
     &           (i.EQ.jj2(k-3).AND.j.EQ.ii2(k-3)))GOTO 300
              IF((Q(i)+Q(j)).EQ.1.d0)THEN
                k=k+1
                phsp(np,k)(l1:l2)='W+ '
                nwp=nwp+1
                iwp(np,k,nwp)=i
                jwp(np,k,nwp)=j
                l1=l1+3
                l2=l2+3
              ELSE IF((Q(i)+Q(j)).EQ.-1.d0)THEN
                k=k+1
                phsp(np,k)(l1:l2)='W- '
                nwm=nwm+1
                iwm(np,k,nwm)=i
                jwm(np,k,nwm)=j
                l1=l1+3
                l2=l2+3
              ELSE IF((Q(i)+Q(j)).EQ.0.d0)THEN
                k=k+1
                phsp(np,k)(l1:l2)='Z  '
                nz=nz+1
                iz(np,k,nz)=i
                jz(np,k,nz)=j
                l1=l1+3
                l2=l2+3
              ENDIF
              DO i1=3,7
                DO j1=i1+1,8
                  IF(i1.NE.i.AND.j1.NE.j.AND.i1.NE.j.AND.j1.NE.i.AND.
     &               ((i1.NE.iord(np,k+1,3).AND.j1.NE.iord(np,k+1,4)).
     &               AND.(i1.NE.iord(np,k+1,4).AND.
     &               j1.NE.iord(np,k+1,3))).AND.
     &               (i1.NE.ii1(k).OR.j1.NE.jj1(k)).AND.
     &               (i1.NE.jj1(k).OR.j1.NE.ii1(k)).AND.
     &               (i1.NE.ii2(k).OR.j1.NE.jj2(k)).AND.
     &               (i1.NE.jj2(k).OR.j1.NE.ii2(k)))THEN
                    IF(iFa(i1).EQ.iFa(j1).AND.Le(i1).EQ.Le(j1))THEN
                      IF(k.EQ.kmax(np))k=k+1
                      ii1(k)=i1
                      jj1(k)=j1
                      IF((Q(i1)+Q(j1)).EQ.1.d0)THEN
                        phsp(np,k)(l1:l2)='W+ '
                        nwp=nwp+1
                        iwp(np,k,nwp)=i1
                        jwp(np,k,nwp)=j1
                        IF(isch.EQ.1.AND.np.EQ.1)THEN
                          l1=l1+3
                          l2=l2+3
                        ENDIF
                        ii1(k)=i1
                        jj1(k)=j1
                      ELSE IF((Q(i1)+Q(j1)).EQ.-1.d0)THEN
                        phsp(np,k)(l1:l2)='W- '
                        nwm=nwm+1
                        iwm(np,k,nwm)=i1
                        jwm(np,k,nwm)=j1
                        IF(isch.EQ.1.AND.np.EQ.1)THEN
                          l1=l1+3
                          l2=l2+3
                        ENDIF
                        ii1(k)=i1
                        jj1(k)=j1
                      ELSE IF((Q(i1)+Q(j1)).EQ.0.d0)THEN
                        phsp(np,k)(l1:l2)='Z  '
                        nz=nz+1
                        iz(np,k,nz)=i1
                        jz(np,k,nz)=j1
                        IF(isch.EQ.1.AND.np.EQ.1)THEN
                          l1=l1+3
                          l2=l2+3
                        ENDIF
                        ii1(k)=i1
                        jj1(k)=j1
                      ENDIF
                    ENDIF !iFa(i1) e iFa(j1)
                    IF(np.EQ.1.AND.isch.EQ.1)then
                      DO i2=3,7
                        DO j2=i2+1,8
                          IF(i2.NE.i1.AND.j2.NE.j1.AND.i2.NE.j1.
     &                       AND.j2.NE.i1.AND.i2.NE.i.AND.j2.NE.j.
     &                       AND.i2.NE.j.AND.j2.NE.i)THEN
                            IF(iFa(i2).EQ.iFa(j2).AND.
     &                         Le(i2).EQ.Le(j2))THEN
                              IF(k.EQ.kmax(np))k=k+1
                              ii2(k)=i2
                              jj2(k)=j2
                              IF((Q(i2)+Q(j2)).EQ.1.d0)THEN
                                phsp(np,k)(l1:l2)='W+ '
                                nwp=nwp+1
                                iwp(np,k,nwp)=i2
                                jwp(np,k,nwp)=j2
                                ii2(k)=i2
                                jj2(k)=j2
                              ELSE IF((Q(i2)+Q(j2)).EQ.-1.d0)THEN
                                phsp(np,k)(l1:l2)='W- '
                                nwm=nwm+1
                                iwm(np,k,nwm)=i2
                                jwm(np,k,nwm)=j2
                                ii2(k)=i2
                                jj2(k)=j2
                              ELSE IF((Q(i2)+Q(j2)).EQ.0.d0)THEN
                                phsp(np,k)(l1:l2)='Z  '
                                nz=nz+1
                                iz(np,k,nz)=i2
                                jz(np,k,nz)=j2
                                ii2(k)=i2
                                jj2(k)=j2
                              ENDIF
                            ENDIF
                          ENDIF
                        ENDDO
                      ENDDO
                    ENDIF
                  ENDIF !i1 e j1
                ENDDO
              ENDDO
 300        ENDIF
            kmax(np)=k
            l2max(np,k)=max(l2,l2max(np,k))
            nwm=0
            nwp=0
            nz=0
          ENDDO
        ENDDO
      ENDDO
* outgoing pairs with b's: same thing as above for processes with b's.
* This reconstruction accounts for the top fork.

      IF(ABS(idhep(3)).EQ.5.OR.ABS(idhep(4)).EQ.5.OR.
     &   ABS(idhep(5)).EQ.5.OR.ABS(idhep(6)).EQ.5.OR.
     &   ABS(idhep(7)).EQ.5.OR.ABS(idhep(8)).EQ.5)THEN
        DO np=1,nphsp
          DO i=0,mxnphs
            ii(i)=0
            jj(i)=0
            kk(i)=0
            ii1(i)=0
            jj1(i)=0
            kk1(i)=0
            ii2(i)=0
            jj2(i)=0
            kk2(i)=0
          ENDDO !i
          k=kmax(np)
          nwp=0
          nwm=0
          nz=0
          nb=0
          DO i=3,7
            DO j=i+1,8
              IF(nphsp.EQ.0)THEN
                l1=1
                l2=3
              ELSE
                l1=7
                l2=9
              ENDIF
              IF((i.NE.ii1(k).OR.j.NE.jj1(k)).AND.
     &           (i.NE.jj1(k).OR.j.NE.ii1(k)).AND.
     &           (i.NE.ii2(k).OR.j.NE.jj2(k)).AND.
     &           (i.NE.jj2(k).OR.j.NE.ii2(k)).AND.
     &           ((i.NE.iord(np,k+1,3).AND.j.NE.iord(np,k+1,4)).AND.
     &           (i.NE.iord(np,k+1,4).AND.j.NE.iord(np,k+1,3))).AND.
     &           iFa(i).EQ.iFa(j).AND.Le(i).EQ.Le(j))THEN
                IF(k.GT.1.AND.(i.EQ.ii1(k-1).AND.j.EQ.jj1(k-1)).OR.
     &             (i.EQ.jj1(k-1).AND.j.EQ.ii1(k-1)).OR.
     &             (i.EQ.ii2(k-1).AND.j.EQ.jj2(k-1)).OR.
     &             (i.EQ.jj2(k-1).AND.j.EQ.ii2(k-1)))GOTO 500
                IF(k.GT.2.AND.(i.EQ.ii1(k-2).AND.j.EQ.jj1(k-2)).OR.
     &             (i.EQ.jj1(k-2).AND.j.EQ.ii1(k-2)).OR.
     &             (i.EQ.ii2(k-2).AND.j.EQ.jj2(k-2)).OR.
     &             (i.EQ.jj2(k-2).AND.j.EQ.ii2(k-2)))GOTO 500
                IF(k.GT.3.AND.(i.EQ.ii1(k-3).AND.j.EQ.jj1(k-3)).OR.
     &             (i.EQ.jj1(k-3).AND.j.EQ.ii1(k-3)).OR.
     &             (i.EQ.ii2(k-3).AND.j.EQ.jj2(k-3)).OR.
     &             (i.EQ.jj2(k-3).AND.j.EQ.ii2(k-3)))GOTO 500
                IF((Q(i)+Q(j)).EQ.1.d0)THEN
                  k=k+1
                  jjb=0
                  DO iib=3,8
                    IF(iib.NE.i.AND.iib.NE.j.AND.iib.NE.iord(np,k+1,3)
     &                 .AND.iib.NE.iord(np,k+1,4))THEN
                      jjb=jjb+1
                      idb(jjb)=iib
                    ENDIF
                  ENDDO !iib
                  IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.5).OR.
     &               ((jjb-1).GE.1.AND.idhep(idb(jjb-1)).EQ.5).OR.
     &               ((jjb-2).GE.1.AND.idhep(idb(jjb-2)).EQ.5).OR.
     &               ((jjb-3).GE.1.AND.idhep(idb(jjb-3)).EQ.5))THEN
                    nb=nb+1
                    DO iib=1,jjb
                      IF(idhep(idb(iib)).EQ.5)THEN
                        ib(np,k,nb)=idb(iib)
                        GOTO 101
                      ENDIF
                    ENDDO !iib
 101                phsp(np,k)(l1:l2)='t  '
                    ii(k)=i
                    jj(k)=j
                    kk(k)=ib(np,k,nb)
                  ELSE
                    phsp(np,k)(l1:l2)='W+ '
                    ii(k)=i
                    jj(k)=j                                        
                  ENDIF
                  nwp=nwp+1
                  iwp(np,k,nwp)=i
                  jwp(np,k,nwp)=j
                ELSE IF((Q(i)+Q(j)).EQ.-1.d0)THEN
                  k=k+1
                  jjb=0
                  DO iib=3,8
                    IF(iib.NE.i.AND.iib.NE.j.AND.iib.NE.iord(np,k+1,3)
     &                 .AND.iib.NE.iord(np,k+1,4))THEN
                      jjb=jjb+1
                      idb(jjb)=iib
                    ENDIF
                  ENDDO !iib
                  IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.-5).OR.
     &               ((jjb-1).GE.1.AND.idhep(idb(jjb-1)).EQ.-5).OR.
     &               ((jjb-2).GE.1.AND.idhep(idb(jjb-2)).EQ.-5).OR.
     &               ((jjb-3).GE.1.AND.idhep(idb(jjb-3)).EQ.-5))THEN
                    nb=nb+1
                    DO iib=1,jjb
                      IF(idhep(idb(iib)).EQ.-5)THEN
                        ib(np,k,nb)=idb(iib)
                        GOTO 102
                      ENDIF
                    ENDDO !iib
 102                phsp(np,k)(l1:l2)='t  '
                    ii(k)=i
                    jj(k)=j
                    kk(k)=ib(np,k,nb)
                  ELSE
                    phsp(np,k)(l1:l2)='W- '
                    ii(k)=i
                    jj(k)=j                                        
                  ENDIF
                  nwm=nwm+1
                  iwm(np,k,nwm)=i
                  jwm(np,k,nwm)=j
                ELSE IF((Q(i)+Q(j)).EQ.0.d0)THEN
                  IF(ABS(idhep(i)).NE.5.AND.ABS(idhep(j)).NE.5)THEN
                    k=k+1
                    phsp(np,k)(l1:l2)='Z  '
                    nz=nz+1
                    iz(np,k,nz)=i
                    jz(np,k,nz)=j
                    ii(k)=i
                    jj(k)=j
                  ELSE
                    GOTO 500
                  ENDIF
                ELSE
                  GOTO 500
                ENDIF
                jjb=0
                DO iib=3,8
                  IF(iib.NE.ii(k).AND.iib.NE.jj(k).AND.iib.NE.
     &               kk(k).AND.iib.NE.iord(np,k+1,3)
     &               .AND.iib.NE.iord(np,k+1,4))THEN
                    jjb=jjb+1
                    idb(jjb)=iib
                  ENDIF
                ENDDO !iib
                IF(jjb.EQ.1.AND.abs(idhep(idb(1))).EQ.5)THEN
                  IF(nb.EQ.1)THEN
                    jb(np,k,nb)=idb(1)
                  ELSE IF(nb.EQ.2)THEN
                    jb(np,k,nb-1)=idb(1)
                  ENDIF
                ENDIF !jjb
                DO i1=3,7
                  DO j1=i1+1,8
                    IF(i1.NE.i.AND.j1.NE.j.AND.i1.NE.j.AND.j1.NE.i.AND.
     &                 ((i1.NE.iord(np,k+1,3).AND.j1.NE.iord(np,k+1,4))
     &                 .AND.(i1.NE.iord(np,k+1,4).AND.
     &                 j1.NE.iord(np,k+1,3))).AND.
     &                 (i1.NE.kk(k).AND.j1.NE.kk(k)).AND.
     &                 (i1.NE.ii1(k).OR.j1.NE.jj1(k)).AND.
     &                 (i1.NE.jj1(k).OR.j1.NE.ii1(k)).AND.
     &                 (i1.NE.ii2(k).OR.j1.NE.jj2(k)).AND.
     &                 (i1.NE.jj2(k).OR.j1.NE.ii2(k)))THEN
                      IF(iFa(i1).EQ.iFa(j1).AND.Le(i1).EQ.Le(j1))THEN
                        IF((Q(i1)+Q(j1)).EQ.1.d0)THEN
                          IF(k.EQ.kmax(np))k=k+1
                          l1=l1+3
                          l2=l2+3
                          jjb=0
                          DO iib=3,8
                            IF(iib.NE.ii(k).AND.iib.NE.jj(k).AND.
     &                         iib.NE.kk(k).AND.iib.NE.i1.AND.iib.NE.j1
     &                         .AND.iib.NE.iord(np,k+1,3)
     &                         .AND.iib.NE.iord(np,k+1,4))THEN
                              jjb=jjb+1
                              idb(jjb)=iib
                            ENDIF
                          ENDDO !iib
                          IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.5).OR.
     &                       ((jjb-1).GE.1.AND.idhep(idb(jjb-1)).EQ.5)
     &                       .OR.((jjb-2).GE.1.AND.idhep(idb(jjb-2)).EQ.
     &                       5).OR.((jjb-3).GE.1.AND.idhep(idb(jjb-3))
     &                       .EQ.5))THEN
                            DO iib=1,jjb
                              IF(idhep(idb(iib)).EQ.5)THEN
                                IF(ib(np,k,nb).EQ.0)THEN
                                  nb=nb+1
                                  ib(np,k,nb)=idb(iib)
                                  kk1(k)=ib(np,k,nb)
                                ELSE
                                  jb(np,k,nb)=idb(iib)
                                  kk1(k)=jb(np,k,nb)
                                ENDIF
                                GOTO 103
                              ENDIF
                            ENDDO !iib
 103                        phsp(np,k)(l1:l2)='t  '
                          ELSE
                            phsp(np,k)(l1:l2)='W+ '
                          ENDIF
                          nwp=nwp+1
                          iwp(np,k,nwp)=i1
                          jwp(np,k,nwp)=j1
                          ii1(k)=i1
                          jj1(k)=j1
                        ELSE IF((Q(i1)+Q(j1)).EQ.-1.d0)THEN
                           l1=l1+3
                           l2=l2+3
                          jjb=0
                          DO iib=3,8
                            IF(iib.NE.ii(k).AND.iib.NE.jj(k).AND.iib.NE.
     &                         kk(k).AND.iib.NE.i1.AND.iib.NE.j1.AND.
     &                         iib.NE.iord(np,k+1,3)
     &                         .AND.iib.NE.iord(np,k+1,4))THEN
                              jjb=jjb+1
                              idb(jjb)=iib
                            ENDIF
                          ENDDO !iib
                          IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.-5).OR.
     &                       ((jjb-1).GE.1.AND.idhep(idb(jjb-1)).EQ.-5)
     &                       .OR.((jjb-2).GE.1.AND.idhep(idb(jjb-2)).EQ.
     &                       -5).OR.((jjb-3).GE.1.AND.idhep(idb(jjb-3))
     &                       .EQ.-5))THEN
                            DO iib=1,jjb
                              IF(idhep(idb(iib)).EQ.-5)THEN
                                IF(ib(np,k,nb).EQ.0)THEN
                                  nb=nb+1
                                  ib(np,k,nb)=idb(iib)
                                  kk1(k)=ib(np,k,nb)
                                ELSE
                                  jb(np,k,nb)=idb(iib)
                                  kk1(k)=jb(np,k,nb)
                                ENDIF
                                GOTO 104
                              ENDIF
                            ENDDO !iib
 104                        phsp(np,k)(l1:l2)='t  '
                          ELSE
                            phsp(np,k)(l1:l2)='W- '
                          ENDIF
                          nwm=nwm+1
                          iwm(np,k,nwm)=i1
                          jwm(np,k,nwm)=j1
                          ii1(k)=i1
                          jj1(k)=j1
                        ELSE IF((Q(i1)+Q(j1)).EQ.0.d0)THEN
                          IF(ABS(idhep(i1)).NE.5)THEN
                            l1=l1+3
                            l2=l2+3
                            phsp(np,k)(l1:l2)='Z  '
                            nz=nz+1
                            iz(np,k,nz)=i1
                            jz(np,k,nz)=j1
                            ii1(k)=i1
                            jj1(k)=j1
                          ELSE
                            IF(nb.EQ.1)THEN
                              nb=nb+1
                              l1=l1+3
                              l2=l2+3
                              phsp(np,k)(l1:l2)='Z  '
                              nz=nz+1
                              iz(np,k,nz)=i1
                              jz(np,k,nz)=j1
                              ii1(k)=i1
                              jj1(k)=j1
                            ELSE
                              GOTO 400
                            ENDIF
                          ENDIF
                        ELSE
                          GOTO 400
                        ENDIF
                        jjb=0
                        DO iib=3,8
                          IF(iib.NE.ii(k).AND.iib.NE.jj(k).AND.iib.NE.
     &                       kk(k).AND.iib.NE.ii1(k).AND.iib.NE.jj1(k)
     &                       .AND.iib.NE.kk1(k).AND.iib.NE.
     &                       iord(np,k+1,3)
     &                       .AND.iib.NE.iord(np,k+1,4))THEN
                            jjb=jjb+1
                            idb(jjb)=iib
                          ENDIF
                        ENDDO !iib
                        IF(jjb.EQ.1.AND.abs(idhep(idb(1))).EQ.5)THEN
                          IF(nb.EQ.1)THEN
                            jb(np,k,nb)=idb(1)
                          ELSE IF(nb.EQ.2)THEN
                            jb(np,k,nb-1)=idb(1)
                          ENDIF
                        ENDIF !jjb
                          
                        DO i2=3,7
                          DO j2=i2+1,8
                            IF(i2.NE.ii1(k).AND.j2.NE.jj1(k).AND.
     &                         i2.NE.jj1(k).AND.j2.NE.ii1(k).AND.
     &                         i2.NE.ii(k).AND.j2.NE.jj(k).AND.
     &                         i2.NE.jj(k).AND.j2.NE.ii(k).AND.
     &                         i2.NE.kk(k).AND.i2.NE.kk1(k).AND.
     &                         j2.NE.kk(k).AND.j2.NE.kk1(k).AND.
     &                         i2.NE.iord(np,k,3).AND.i2.NE.
     &                         iord(np,k,4).AND.j2.NE.iord(np,k,3).AND.
     &                         j2.NE.iord(np,k,4))THEN
                              IF(iFa(i2).EQ.iFa(j2).AND.
     &                           Le(i2).EQ.Le(j2))THEN
                                IF(k.EQ.kmax(np))k=k+1
                                IF((Q(i2)+Q(j2)).EQ.1.d0)THEN
                                  l1=l1+3
                                  l2=l2+3
                                  jjb=0
                                  DO iib=3,8
                                    IF(iib.NE.i2.AND.iib.NE.j2.AND.
     &                                 iib.NE.ii(k).AND.iib.NE.jj(k)
     &                                 .AND.iib.NE.kk(k).AND.iib.NE.
     &                                 ii1(k).AND.iib.NE.jj1(k).AND.
     &                                 iib.NE.kk1(k).AND.
     &                                 iib.NE.iord(np,k+1,3).AND.
     &                                 iib.NE.iord(np,k+1,4))THEN
                                      jjb=jjb+1
                                      idb(jjb)=iib
                                    ENDIF
                                  ENDDO !iib
                                  IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.5)
     &                               .OR.((jjb-1).GE.1.AND.
     &                               idhep(idb(jjb-1)).EQ.5).OR.
     &                               ((jjb-2).GE.1.AND.idhep(idb(jjb-2))
     &                               .EQ.5).OR.((jjb-3).GE.1.AND.
     &                               idhep(idb(jjb-3)).EQ.5))THEN
                                    DO iib=1,jjb
                                      IF(idhep(idb(iib)).EQ.5)THEN
                                        IF(ib(np,k,nb).EQ.0)THEN
                                          nb=nb+1
                                          ib(np,k,nb)=idb(iib)
                                          kk2(k)=ib(np,k,nb)
                                        ELSE
                                          jb(np,k,nb)=idb(iib)
                                          kk2(k)=jb(np,k,nb)
                                        ENDIF
                                        GOTO 105
                                      ENDIF
                                    ENDDO !iib
 105                                phsp(np,k)(l1:l2)='t  '
                                  ELSE
                                    phsp(np,k)(l1:l2)='W+ '
                                  ENDIF
                                  nwp=nwp+1
                                  iwp(np,k,nwp)=i2
                                  jwp(np,k,nwp)=j2
                                  ii2(k)=i2
                                  jj2(k)=j2
                                ELSE IF((Q(i2)+Q(j2)).EQ.-1.d0)THEN
                                  l1=l1+3
                                  l2=l2+3
                                  jjb=0
                                  DO iib=3,8
                                    IF(iib.NE.i2.AND.iib.NE.j2.AND.
     &                                 iib.NE.ii(k).AND.iib.NE.jj(k)
     &                                 .AND.iib.NE.kk(k).AND.iib.NE.
     &                                 ii1(k).AND.iib.NE.jj1(k).AND.
     &                                 iib.NE.kk1(k).AND.
     &                                 iib.NE.iord(np,k+1,3).AND.
     &                                 iib.NE.iord(np,k+1,4))THEN
                                      jjb=jjb+1
                                      idb(jjb)=iib
                                    ENDIF
                                  ENDDO !iib
                                  IF((jjb.GE.1.AND.idhep(idb(jjb)).EQ.5)
     &                               .OR.((jjb-1).GE.1.AND.
     &                               idhep(idb(jjb-1)).EQ.5).OR.
     &                               ((jjb-2).GE.1.AND.idhep(idb(jjb-2))
     &                               .EQ.5).OR.((jjb-3).GE.1.AND.
     &                               idhep(idb(jjb-3)).EQ.5))THEN
                                    DO iib=1,jjb
                                      IF(idhep(idb(iib)).EQ.5)THEN
                                        IF(ib(np,k,nb).EQ.0)THEN
                                          nb=nb+1
                                          ib(np,k,nb)=idb(iib)
                                          kk2(k)=ib(np,k,nb)
                                        ELSE
                                          jb(np,k,nb)=idb(iib)
                                          kk2(k)=jb(np,k,nb)
                                        ENDIF
                                        GOTO 106
                                      ENDIF
                                    ENDDO !iib
 106                                phsp(np,k)(l1:l2)='t  '
                                  ELSE
                                    phsp(np,k)(l1:l2)='W- '
                                  ENDIF
                                  nwm=nwm+1
                                  iwm(np,k,nwm)=i2
                                  jwm(np,k,nwm)=j2
                                  ii2(k)=i2
                                  jj2(k)=j2
                                ELSE IF((Q(i2)+Q(j2)).EQ.0.d0)THEN
                                  IF(ABS(idhep(i2)).NE.5.AND.
     &                               ABS(idhep(j2)).NE.5)THEN
                                    phsp(np,k)(l1:l2)='Z  '
                                    nz=nz+1
                                    iz(np,k,nz)=i2
                                    jz(np,k,nz)=j2
                                  ELSE
                                    IF(nb.EQ.1)THEN
                                      phsp(np,k)(l1:l2)='Z  '
                                    ENDIF
                                  ENDIF
                                  ii2(k)=i2
                                  jj2(k)=j2
                                ENDIF
                              ENDIF
                            ENDIF
                          ENDDO
                        ENDDO
                      ENDIF !iFa and iLe
                    ENDIF !i1 e j1
 400              ENDDO !j1
                ENDDO !i1
              ENDIF
              kmax(np)=k
              l2max(np,k)=max(l2,l2max(np,k))
              nwm=0
              nwp=0
              nz=0
 500        ENDDO !j
          ENDDO !i
        ENDDO
      ENDIF ! on b's


* Phase space input and momentum ordering
* once the phase space is definined, the vectors iorder, idp, and inout
* and ialfa are filled in for each ia=iphs_ind.
* iorder(i,iphs_ind)=j means that the p(i) coming out from the phase spase 
* is named p(j) in the amplitude. In other words, iorder(i,iphs_ind) says
* that p(i) must be in the j-th position in the amplitude order (which is
* the reference order)
* idp= identity of the particles
* inout=1/-1 for outgoing/incoming particles
* ialfa(iphs_ind)=1 if the phase space is on, otherwise is 0. 

      DO i=1,nphsp
        DO j=1,kmax(i)
          IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(00) W+ W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(00) W- W+ ').AND.
     &       ialfa(1).EQ.0)THEN
            ialfa(1)=1
            ia=1
* momentum exchange : phase space -> amplitude order
            IF(idhep(it00(i,1)).GT.0)then
              IF((Le(iwp(i,j,1)).EQ.0.AND.iFa(it00(i,1)).EQ.
     &           iFa(iwp(i,j,1)).AND.ABS(idhep(jt00(i,1))).NE.5.
     &           AND.igroup.NE.15.AND.igroup.NE.16).
     &           OR.(igroup.EQ.15.AND.Q(it00(i,1)).LT.0.d0).OR.
     &           (igroup.EQ.16.AND.Q(it00(i,1)).LT.0.d0).OR.
     &           (idhep(it00(i,1)).EQ.5.AND.
     &           ABS(idhep(jt00(i,1))).NE.5))THEN
                iorder(7,1)=3
                iorder(1,1)=4            
                idp(3)=idhep(it00(i,1))
                idp(4)=-idhep(1)
                inout(4,1)=-1
                IF(idhep(jt00(i,1)).GT.0)THEN
                  iorder(8,1)=1
                  iorder(2,1)=2
                  idp(1)=idhep(jt00(i,1))
                  idp(2)=-idhep(2)
                  inout(2,1)=-1
                ELSE
                  iorder(8,1)=2
                  iorder(2,1)=1
                  idp(2)=idhep(jt00(i,1))
                  idp(1)=-idhep(2)
                  inout(1,1)=-1
                ENDIF
              ELSE
                iorder(7,1)=1
                iorder(1,1)=2            
                idp(1)=idhep(it00(i,1))
                idp(2)=-idhep(1)
                inout(2,1)=-1
                IF(idhep(jt00(i,1)).GT.0)THEN
                  iorder(8,1)=3
                  iorder(2,1)=4
                  idp(3)=idhep(jt00(i,1))
                  idp(4)=-idhep(2)
                  inout(4,1)=-1
                ELSE
                  iorder(8,1)=4
                  iorder(2,1)=3
                  idp(4)=idhep(jt00(i,1))
                  idp(3)=-idhep(2)
                  inout(3,1)=-1
                ENDIF
              ENDIF
            ELSE IF(idhep(it00(i,1)).LT.0)THEN
              IF((Le(iwp(i,j,1)).EQ.0.AND.iFa(it00(i,1)).EQ.
     &           iFa(iwp(i,j,1)).AND.ABS(idhep(jt00(i,1))).NE.5.
     &           AND.igroup.NE.15.AND.igroup.NE.16).
     &           OR.(igroup.EQ.15.AND.Q(it00(i,1)).GT.0.d0).OR.
     &           (igroup.EQ.16.AND.Q(it00(i,1)).GT.0.d0).OR.
     &           (idhep(it00(i,1)).EQ.-5.AND.
     &           ABS(idhep(jt00(i,1))).NE.5))THEN
                iorder(7,1)=4
                iorder(1,1)=3
                idp(4)=idhep(it00(i,1))
                idp(3)=-idhep(1)
                inout(3,1)=-1
	        IF(idhep(jt00(i,1)).GT.0)THEN
                  iorder(8,1)=1
                  iorder(2,1)=2
                  idp(1)=idhep(jt00(i,1))
                  idp(2)=-idhep(2)
                  inout(2,1)=-1
	        ELSE
                  iorder(8,1)=2
                  iorder(2,1)=1
                  idp(2)=idhep(jt00(i,1))
                  idp(1)=-idhep(2)
                  inout(1,1)=-1
                ENDIF
              ELSE
                iorder(7,1)=2
                iorder(1,1)=1
                idp(2)=idhep(it00(i,1))
                idp(1)=-idhep(1)
                inout(1,1)=-1
                IF(idhep(jt00(i,1)).GT.0)THEN
                  iorder(8,1)=3
                  iorder(2,1)=4
                  idp(3)=idhep(jt00(i,1))
                  idp(4)=-idhep(2)
                  inout(4,1)=-1
                ELSE
                  iorder(8,1)=4
                  iorder(2,1)=3
                  idp(4)=idhep(jt00(i,1))
                  idp(3)=-idhep(2)
                  inout(3,1)=-1
                ENDIF
              ENDIF
	    ENDIF
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,1)=5
              iorder(4,1)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,1)=5
              iorder(4,1)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(5,1)=7
              iorder(6,1)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(5,1)=7
              iorder(6,1)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(0W) Z  W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(0W) W- Z '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(0W) Z  W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(0W) W+ Z ').AND.
     &       ialfa(2).EQ.0)THEN
            ialfa(2)=1
            ia=2
            IF(idhep(it0W(i,1)).GT.0)THEN
              IF((iFa(1).EQ.iFa(2).AND.ABS(idhep(iz(i,j,1))).NE.5.
     &           AND.igroup.NE.15.AND.igroup.NE.16).
     &           OR.(igroup.EQ.15.AND.Q(it0W(i,1)).LT.0.d0).OR.
     &           (igroup.EQ.16.AND.Q(it0W(i,1)).LT.0.d0).OR.
     &           idhep(it0W(i,1)).EQ.5)THEN
                iorder(7,2)=3
                iorder(1,2)=4            
                idp(3)=idhep(it0W(i,1))
                idp(4)=-idhep(1)
                inout(4,2)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,2)=1
                  iorder(4,2)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE
                  iorder(3,2)=1
                  iorder(4,2)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(7,2)=1
                iorder(1,2)=2            
                idp(1)=idhep(it0W(i,1))
                idp(2)=-idhep(1)
                inout(2,2)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,2)=3
                  iorder(4,2)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE
                  iorder(3,2)=3
                  iorder(4,2)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ELSE IF(idhep(it0W(i,1)).LT.0)THEN
              IF((iFa(1).EQ.iFa(2).AND.ABS(idhep(iz(i,j,1))).NE.5.
     &           AND.igroup.NE.15.AND.igroup.NE.16).
     &           OR.(igroup.EQ.15.AND.Q(it0W(i,1)).GT.0).OR.
     &           (igroup.EQ.16.AND.Q(it0W(i,1)).GT.0).OR.
     &           idhep(it0W(i,1)).EQ.-5)THEN
                iorder(7,2)=4
                iorder(1,2)=3            
                idp(4)=idhep(it0W(i,1))
                idp(3)=-idhep(1)
                inout(3,2)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,2)=1
                  iorder(4,2)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE
                  iorder(3,2)=1
                  iorder(4,2)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(7,2)=2
                iorder(1,2)=1            
                idp(2)=idhep(it0W(i,1))
                idp(1)=-idhep(1)
                inout(1,2)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,2)=3
                  iorder(4,2)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE
                  iorder(3,2)=3
                  iorder(4,2)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ENDIF
            IF(idhep(jt0W(i,1)).GT.0)THEN
              IF(Q(jt0W(i,1)).GT.0)THEN
                iorder(8,2)=5
                iorder(2,2)=6
                idp(5)=idhep(jt0W(i,1))
                idp(6)=-idhep(2)
                inout(6,2)=-1
              ELSE
                iorder(8,2)=7
                iorder(2,2)=8
                idp(7)=idhep(jt0W(i,1))
                idp(8)=-idhep(2)
                inout(8,2)=-1
              ENDIF
            ELSE IF(idhep(jt0W(i,1)).LT.0)THEN
              IF(Q(jt0W(i,1)).GT.0)THEN
                iorder(8,2)=6
                iorder(2,2)=5
                idp(6)=idhep(jt0W(i,1))
                idp(5)=-idhep(2)
                inout(5,2)=-1
              ELSE
                iorder(8,2)=8
                iorder(2,2)=7
                idp(8)=idhep(jt0W(i,1))
                idp(7)=-idhep(2)
                inout(7,2)=-1
              ENDIF
            ENDIF
            IF(idhep(iwm(i,1,1)).GT.0)THEN
              iorder(5,2)=7
              iorder(6,2)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,1,1)).LT.0)THEN
              iorder(5,2)=7
              iorder(6,2)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(W0) Z  W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(W0) W- Z '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(W0) Z  W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(W0) W+ Z ').AND.
     &       ialfa(3).EQ.0)THEN
            ialfa(3)=1
            ia=3
            IF(idhep(itW0(i,1)).GT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(7,3)=5
                iorder(1,3)=6            
                idp(5)=idhep(itW0(i,1))
                idp(6)=-idhep(1)
                inout(6,3)=-1
              ELSE
                iorder(7,3)=7
                iorder(1,3)=8            
                idp(7)=idhep(itW0(i,1))
                idp(8)=-idhep(1)
                inout(8,3)=-1
              ENDIF
            ELSE IF(idhep(itW0(i,1)).LT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(7,3)=6
                iorder(1,3)=5
                idp(6)=idhep(itW0(i,1))
                idp(5)=-idhep(1)
                inout(5,3)=-1
              ELSE
                iorder(7,3)=8
                iorder(1,3)=7            
                idp(8)=idhep(itW0(i,1))
                idp(7)=-idhep(1)
                inout(7,3)=-1
              ENDIF
            ENDIF
            IF(idhep(jtW0(i,1)).GT.0)THEN
              IF((idhep(iz(i,j,1)).NE.5.AND.iFa(1).EQ.iFa(2).
     &           AND.igroup.NE.15.AND.igroup.NE.16).OR.
     &           idhep(jtW0(i,1)).EQ.5.OR.
     &           ((igroup.EQ.15.OR.igroup.EQ.16).AND.
     &           Q(jtW0(i,1)).LT.0))THEN
                iorder(8,3)=3
                iorder(2,3)=4
                idp(3)=idhep(jtW0(i,1))
                idp(4)=-idhep(2)
                inout(4,3)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,3)=1
                  iorder(4,3)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(3,3)=1
                  iorder(4,3)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(8,3)=1
                iorder(2,3)=2
                idp(1)=idhep(jtW0(i,1))
                idp(2)=-idhep(2)
                inout(2,3)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,3)=3
                  iorder(4,3)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(3,3)=3
                  iorder(4,3)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ELSE IF(idhep(jtW0(i,1)).LT.0)THEN
               IF((idhep(iz(i,j,1)).NE.5.AND.iFa(1).EQ.iFa(2).AND.
     &           igroup.NE.15.AND.igroup.NE.16).OR.
     &           idhep(jtW0(i,1)).EQ.-5.OR.
     &           ((igroup.EQ.15.OR.igroup.EQ.16).AND.
     &           Q(jtW0(i,1)).GT.0))THEN
                iorder(8,3)=4
                iorder(2,3)=3
                idp(4)=idhep(jtW0(i,1))
                idp(3)=-idhep(2)
                inout(3,3)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,3)=1
                  iorder(4,3)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(3,3)=1
                  iorder(4,3)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(8,3)=2
                iorder(2,3)=1
                idp(2)=idhep(jtW0(i,1))
                idp(1)=-idhep(2)
                inout(1,3)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,3)=3
                  iorder(4,3)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(3,3)=3
                  iorder(4,3)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(5,3)=7
              iorder(6,3)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(5,3)=7
              iorder(6,3)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF            
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(5,3)=5
              iorder(6,3)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(5,3)=5
              iorder(6,3)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
            ENDIF            
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(WW) W- W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(WW) W+ W+ ').AND.
     &       ialfa(4).EQ.0)THEN
            ialfa(4)=1
            ia=4
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(itWW(i,1)).GT.0)THEN
                IF(Q(itWW(i,1)).GT.0.d0)THEN
                  iorder(7,4)=5
                  iorder(1,4)=6            
                  idp(5)=idhep(itWW(i,1))
                  idp(6)=-idhep(1)
                  inout(6,4)=-1
                ELSE
                  iorder(7,4)=3
                  iorder(1,4)=4
                  idp(3)=idhep(itWW(i,1))
                  idp(4)=-idhep(1)
                  inout(4,4)=-1
                ENDIF
              ELSE IF(idhep(itWW(i,1)).LT.0)THEN
                IF(Q(itWW(i,1)).GT.0.d0)THEN
                  iorder(7,4)=6
                  iorder(1,4)=5
                  idp(6)=idhep(itWW(i,1))
                  idp(5)=-idhep(1)
                  inout(5,4)=-1
                ELSE
                  iorder(7,4)=4
                  iorder(1,4)=3
                  idp(4)=idhep(itWW(i,1))
                  idp(3)=-idhep(1)
                  inout(3,4)=-1
                ENDIF
              ENDIF
              IF(idhep(jtWW(i,1)).GT.0)THEN
                IF(Q(jtWW(i,1)).GT.0.d0)THEN
                  iorder(8,4)=1
                  iorder(2,4)=2
                  idp(1)=idhep(jtWW(i,1))
                  idp(2)=-idhep(2)
                  inout(2,4)=-1
                ELSE
                  iorder(8,4)=7
                  iorder(2,4)=8
                  idp(7)=idhep(jtWW(i,1))
                  idp(8)=-idhep(2)
                  inout(8,4)=-1
                ENDIF
              ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                IF(Q(jtWW(i,1)).GT.0.d0)THEN
                  iorder(8,4)=2
                  iorder(2,4)=1
                  idp(2)=idhep(jtWW(i,1))
                  idp(1)=-idhep(2)
                  inout(1,4)=-1
                ELSE
                  iorder(8,4)=8
                  iorder(2,4)=7
                  idp(8)=idhep(jtWW(i,1))
                  idp(7)=-idhep(2)
                  inout(7,4)=-1
                ENDIF
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(3,4)=3
                  iorder(4,4)=4
                  idp(3)=idhep(iwm(i,j,1))
                  idp(4)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,4)=7
                    iorder(6,4)=8
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE
                    iorder(5,4)=7
                    iorder(6,4)=8
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,4)=7
                  iorder(6,4)=8
                  idp(7)=idhep(iwm(i,j,1))
                  idp(8)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(3,4)=3
                    iorder(4,4)=4
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE
                    iorder(3,4)=3
                    iorder(4,4)=4
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(3,4)=3
                  iorder(4,4)=4
                  idp(4)=idhep(iwm(i,j,1))
                  idp(3)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,4)=7
                    iorder(6,4)=8
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE
                    iorder(5,4)=7
                    iorder(6,4)=8
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,4)=7
                  iorder(6,4)=8
                  idp(8)=idhep(iwm(i,j,1))
                  idp(7)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(3,4)=3
                    iorder(4,4)=4
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE
                    iorder(3,4)=3
                    iorder(4,4)=4
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(3,4)=1
                  iorder(4,4)=2
                  idp(1)=idhep(iwp(i,j,1))
                  idp(2)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(5,4)=5
                    iorder(6,4)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE
                    iorder(5,4)=5
                    iorder(6,4)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,4)=5
                  iorder(6,4)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,4)=1
                    iorder(4,4)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE
                    iorder(3,4)=1
                    iorder(4,4)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(3,4)=1
                  iorder(4,4)=2
                  idp(2)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(5,4)=5
                    iorder(6,4)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE
                    iorder(5,4)=5
                    iorder(6,4)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,4)=5
                  iorder(6,4)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,4)=1
                    iorder(4,4)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE
                    iorder(3,4)=1
                    iorder(4,4)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
            ELSE  !mixed
              IF(iwm(i,j,1).NE.0.AND.iwm(i,j,2).NE.0)THEN
                IF(idhep(itWW(i,1)).GT.0.AND.Q(itWW(i,1)).GT.0.d0)THEN
                  IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,1)).AND.
     &               Le(itWW(i,1)).EQ.Le(iwm(i,j,1)))THEN
                    iorder(7,4)=1
                    iorder(1,4)=4            
                    idp(1)=idhep(itWW(i,1))
                    idp(4)=-idhep(1)
                    inout(4,4)=-1
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,2)).GT.0)THEN
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(7)=idhep(iwm(i,j,2))
                      idp(8)=idhep(jwm(i,j,2))
                    ELSE
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(8)=idhep(iwm(i,j,2))
                      idp(7)=idhep(jwm(i,j,2))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=5
                      iorder(2,4)=6            
                      idp(5)=idhep(jtww(i,1))
                      idp(6)=-idhep(2)
                      inout(6,4)=-1
                    ELSE
                      iorder(8,4)=6
                      iorder(2,4)=5            
                      idp(6)=idhep(jtww(i,1))
                      idp(5)=-idhep(2)
                      inout(5,4)=-1
                    ENDIF
                  ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,2)).AND.
     &                    Le(itWW(i,1)).EQ.Le(iwm(i,j,2)))THEN
                    iorder(7,4)=1
                    iorder(1,4)=4            
                    idp(1)=idhep(itWW(i,1))
                    idp(4)=-idhep(1)
                    inout(4,4)=-1
                    IF(idhep(iwm(i,j,2)).GT.0)THEN
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(3)=idhep(iwm(i,j,2))
                      idp(2)=idhep(jwm(i,j,2))
                    ELSE
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(2)=idhep(iwm(i,j,2))
                      idp(3)=idhep(jwm(i,j,2))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(7)=idhep(iwm(i,j,1))
                      idp(8)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(8)=idhep(iwm(i,j,1))
                      idp(7)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=5
                      iorder(2,4)=6            
                      idp(5)=idhep(jtWW(i,1))
                      idp(6)=-idhep(2)
                      inout(6,4)=-1
                    ELSE
                      iorder(8,4)=6
                      iorder(2,4)=5            
                      idp(6)=idhep(jtWW(i,1))
                      idp(5)=-idhep(2)
                      inout(5,4)=-1
                    ENDIF
                  ELSE
                    iorder(7,4)=5
                    iorder(1,4)=6
                    idp(5)=idhep(itWW(i,1))
                    idp(6)=-idhep(1)
                    inout(6,4)=-1
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=1
                      iorder(2,4)=4
                      idp(1)=idhep(jtWW(i,1))
                      idp(4)=-idhep(2)
                      inout(4,4)=-1
                    ELSE
                      iorder(8,4)=4
                      iorder(2,4)=1
                      idp(4)=idhep(jtWW(i,1))
                      idp(1)=-idhep(2)
                      inout(1,4)=-1
                    ENDIF
                    IF(iFa(iwm(i,j,1)).EQ.iFa(jtWW(i,1)).AND.
     &                 Le(iwm(i,j,1)).EQ.Le(jtWW(i,1)))THEN
                      IF(idhep(iwm(i,j,1)).GT.0)THEN
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(3)=idhep(iwm(i,j,1))
                        idp(2)=idhep(jwm(i,j,1))
                      ELSE
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(2)=idhep(iwm(i,j,1))
                        idp(3)=idhep(jwm(i,j,1))
                      ENDIF
                      IF(idhep(iwm(i,j,2)).GT.0)THEN
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(7)=idhep(iwm(i,j,2))
                        idp(8)=idhep(jwm(i,j,2))
                      ELSE
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(8)=idhep(iwm(i,j,2))
                        idp(7)=idhep(jwm(i,j,2))
                      ENDIF                      
                    ELSE IF(iFa(iwm(i,j,2)).EQ.iFa(jtWW(i,2)).AND.
     &                      Le(iwm(i,j,2)).EQ.Le(jtWW(i,2)))THEN
                      IF(idhep(iwm(i,j,2)).GT.0)THEN
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(3)=idhep(iwm(i,j,2))
                        idp(2)=idhep(jwm(i,j,2))
                      ELSE
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(2)=idhep(iwm(i,j,2))
                        idp(3)=idhep(jwm(i,j,2))
                      ENDIF
                      IF(idhep(iwm(i,j,1)).GT.0)THEN
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(7)=idhep(iwm(i,j,1))
                        idp(8)=idhep(jwm(i,j,1))
                      ELSE
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(8)=idhep(iwm(i,j,1))
                        idp(7)=idhep(jwm(i,j,1))
                      ENDIF                      
                    ENDIF
                  ENDIF
                ELSE IF(idhep(itWW(i,1)).LT.0.AND.
     &                  Q(itWW(i,1)).GT.0.d0)THEN
                  IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,1)).AND.
     &               Le(itWW(i,1)).EQ.Le(iwm(i,j,1)))THEN
                    iorder(7,4)=4
                    iorder(1,4)=1            
                    idp(4)=idhep(itWW(i,1))
                    idp(1)=-idhep(1)
                    inout(1,4)=-1
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,2)).GT.0)THEN
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(7)=idhep(iwm(i,j,2))
                      idp(8)=idhep(jwm(i,j,2))
                    ELSE
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(8)=idhep(iwm(i,j,2))
                      idp(7)=idhep(jwm(i,j,2))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=5
                      iorder(2,4)=6            
                      idp(5)=idhep(jtww(i,1))
                      idp(6)=-idhep(2)
                      inout(6,4)=-1
                    ELSE
                      iorder(8,4)=6
                      iorder(2,4)=5            
                      idp(6)=idhep(jtww(i,1))
                      idp(5)=-idhep(2)
                      inout(5,4)=-1
                    ENDIF
                  ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,2)).AND.
     &                    Le(itWW(i,1)).EQ.Le(iwm(i,j,2)))THEN
                    iorder(7,4)=4
                    iorder(1,4)=1            
                    idp(4)=idhep(itWW(i,1))
                    idp(1)=-idhep(1)
                    inout(1,4)=-1
                    IF(idhep(iwm(i,j,2)).GT.0)THEN
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(3)=idhep(iwm(i,j,2))
                      idp(2)=idhep(jwm(i,j,2))
                    ELSE
                      iorder(3,4)=3
                      iorder(4,4)=2            
                      idp(2)=idhep(iwm(i,j,2))
                      idp(3)=idhep(jwm(i,j,2))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(7)=idhep(iwm(i,j,1))
                      idp(8)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,4)=7
                      iorder(6,4)=8            
                      idp(8)=idhep(iwm(i,j,1))
                      idp(7)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=5
                      iorder(2,4)=6            
                      idp(5)=idhep(jtWW(i,1))
                      idp(6)=-idhep(2)
                      inout(6,4)=-1
                    ELSE
                      iorder(8,4)=6
                      iorder(2,4)=5            
                      idp(6)=idhep(jtWW(i,1))
                      idp(5)=-idhep(2)
                      inout(5,4)=-1
                    ENDIF
                  ELSE
                    iorder(7,4)=6
                    iorder(1,4)=5
                    idp(6)=idhep(itWW(i,1))
                    idp(5)=-idhep(1)
                    inout(5,4)=-1
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=1
                      iorder(2,4)=4
                      idp(1)=idhep(jtWW(i,1))
                      idp(4)=-idhep(2)
                      inout(4,4)=-1
                    ELSE
                      iorder(8,4)=4
                      iorder(2,4)=1
                      idp(4)=idhep(jtWW(i,1))
                      idp(1)=-idhep(2)
                      inout(1,4)=-1
                    ENDIF
                    IF(iFa(iwm(i,j,1)).EQ.iFa(jtWW(i,1)).AND.
     &                 Le(iwm(i,j,1)).EQ.Le(jtWW(i,1)))THEN
                      IF(idhep(iwm(i,j,1)).GT.0)THEN
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(3)=idhep(iwm(i,j,1))
                        idp(2)=idhep(jwm(i,j,1))
                      ELSE
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(2)=idhep(iwm(i,j,1))
                        idp(3)=idhep(jwm(i,j,1))
                      ENDIF
                      IF(idhep(iwm(i,j,2)).GT.0)THEN
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(7)=idhep(iwm(i,j,2))
                        idp(8)=idhep(jwm(i,j,2))
                      ELSE
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(8)=idhep(iwm(i,j,2))
                        idp(7)=idhep(jwm(i,j,2))
                      ENDIF                      
                    ELSE IF(iFa(iwm(i,j,2)).EQ.iFa(jtWW(i,2)).AND.
     &                      Le(iwm(i,j,2)).EQ.Le(jtWW(i,2)))THEN
                      IF(idhep(iwm(i,j,2)).GT.0)THEN
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(3)=idhep(iwm(i,j,2))
                        idp(2)=idhep(jwm(i,j,2))
                      ELSE
                        iorder(3,4)=3
                        iorder(4,4)=2
                        idp(2)=idhep(iwm(i,j,2))
                        idp(3)=idhep(jwm(i,j,2))
                      ENDIF
                      IF(idhep(iwm(i,j,1)).GT.0)THEN
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(7)=idhep(iwm(i,j,1))
                        idp(8)=idhep(jwm(i,j,1))
                      ELSE
                        iorder(5,4)=7
                        iorder(6,4)=8
                        idp(8)=idhep(iwm(i,j,1))
                        idp(7)=idhep(jwm(i,j,1))
                      ENDIF                      
                    ENDIF
                  ENDIF
                ENDIF ! T(WW)W-W-  
              ELSE IF(iwp(i,j,1).NE.0.AND.iwp(i,j,2).NE.0)THEN
                IF(idhep(itWW(i,1)).GT.0.AND.Q(itWW(i,1)).LT.0.d0)THEN
                  IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,1)).AND.
     &               Le(itWW(i,1)).EQ.Le(iwp(i,j,1)))THEN
                    iorder(7,4)=3
                    iorder(1,4)=2            
                    idp(3)=idhep(itWW(i,1))
                    idp(2)=-idhep(1)
                    inout(2,4)=-1
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwp(i,j,2)).GT.0)THEN
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(5)=idhep(iwp(i,j,2))
                      idp(6)=idhep(jwp(i,j,2))
                    ELSE
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(6)=idhep(iwp(i,j,2))
                      idp(5)=idhep(jwp(i,j,2))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=7
                      iorder(2,4)=8            
                      idp(7)=idhep(jtww(i,1))
                      idp(8)=-idhep(2)
                      inout(8,4)=-1
                    ELSE
                      iorder(8,4)=8
                      iorder(2,4)=7            
                      idp(8)=idhep(jtww(i,1))
                      idp(7)=-idhep(2)
                      inout(7,4)=-1
                    ENDIF
                  ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,2)).AND.
     &                    Le(itWW(i,1)).EQ.Le(iwp(i,j,2)))THEN
                    iorder(7,4)=3
                    iorder(1,4)=2            
                    idp(3)=idhep(itWW(i,1))
                    idp(2)=-idhep(1)
                    inout(2,4)=-1
                    IF(idhep(iwp(i,j,2)).GT.0)THEN
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(1)=idhep(iwp(i,j,2))
                      idp(4)=idhep(jwp(i,j,2))
                    ELSE
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(4)=idhep(iwp(i,j,2))
                      idp(1)=idhep(jwp(i,j,2))
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(5)=idhep(iwp(i,j,1))
                      idp(6)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(6)=idhep(iwp(i,j,1))
                      idp(5)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=7
                      iorder(2,4)=8            
                      idp(7)=idhep(jtww(i,1))
                      idp(8)=-idhep(2)
                      inout(8,4)=-1
                    ELSE
                      iorder(8,4)=8
                      iorder(2,4)=7            
                      idp(8)=idhep(jtww(i,1))
                      idp(7)=-idhep(2)
                      inout(7,4)=-1
                    ENDIF
                  ELSE
                    iorder(7,4)=7
                    iorder(1,4)=8
                    idp(7)=idhep(itWW(i,1))
                    idp(8)=-idhep(1)
                    inout(8,4)=-1
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=3
                      iorder(2,4)=2
                      idp(3)=idhep(jtWW(i,1))
                      idp(2)=-idhep(2)
                      inout(2,4)=-1
                    ELSE
                      iorder(8,4)=2
                      iorder(2,4)=3
                      idp(2)=idhep(jtWW(i,1))
                      idp(3)=-idhep(2)
                      inout(3,4)=-1
                    ENDIF
                    IF(iFa(iwp(i,j,1)).EQ.iFa(jtWW(i,1)).AND.
     &                 Le(iwp(i,j,1)).EQ.Le(jtWW(i,1)))THEN
                      IF(idhep(iwp(i,j,1)).GT.0)THEN
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(1)=idhep(iwp(i,j,1))
                        idp(4)=idhep(jwp(i,j,1))
                      ELSE
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(4)=idhep(iwp(i,j,1))
                        idp(1)=idhep(jwp(i,j,1))
                      ENDIF
                      IF(idhep(iwp(i,j,2)).GT.0)THEN
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(5)=idhep(iwp(i,j,2))
                        idp(6)=idhep(jwp(i,j,2))
                      ELSE
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(6)=idhep(iwp(i,j,2))
                        idp(5)=idhep(jwp(i,j,2))
                      ENDIF                      
                    ELSE IF(iFa(iwp(i,j,2)).EQ.iFa(jtWW(i,2)).AND.
     &                      Le(iwp(i,j,2)).EQ.Le(jtWW(i,2)))THEN
                      IF(idhep(iwp(i,j,2)).GT.0)THEN
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(1)=idhep(iwp(i,j,2))
                        idp(4)=idhep(jwp(i,j,2))
                      ELSE
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(4)=idhep(iwp(i,j,2))
                        idp(1)=idhep(jwp(i,j,2))
                      ENDIF
                      IF(idhep(iwp(i,j,1)).GT.0)THEN
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(5)=idhep(iwm(i,j,1))
                        idp(6)=idhep(jwm(i,j,1))
                      ELSE
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(6)=idhep(iwp(i,j,1))
                        idp(5)=idhep(jwp(i,j,1))
                      ENDIF                      
                    ENDIF
                  ENDIF
                ELSE IF(idhep(itWW(i,1)).LT.0.AND.
     &                  Q(itWW(i,1)).LT.0.d0)THEN
                  IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,1)).AND.
     &               Le(itWW(i,1)).EQ.Le(iwp(i,j,1)))THEN
                    iorder(7,4)=2
                    iorder(1,4)=3            
                    idp(2)=idhep(itWW(i,1))
                    idp(3)=-idhep(1)
                    inout(3,4)=-1
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwp(i,j,2)).GT.0)THEN
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(5)=idhep(iwp(i,j,2))
                      idp(6)=idhep(jwp(i,j,2))
                    ELSE
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(6)=idhep(iwp(i,j,2))
                      idp(5)=idhep(jwp(i,j,2))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=3
                      iorder(2,4)=2            
                      idp(3)=idhep(jtww(i,1))
                      idp(2)=-idhep(2)
                      inout(2,4)=-1
                    ELSE
                      iorder(8,4)=2
                      iorder(2,4)=3            
                      idp(2)=idhep(jtww(i,1))
                      idp(3)=-idhep(2)
                      inout(3,4)=-1
                    ENDIF
                  ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,2)).AND.
     &                    Le(itWW(i,1)).EQ.Le(iwp(i,j,2)))THEN
                    iorder(7,4)=2
                    iorder(1,4)=3            
                    idp(2)=idhep(itWW(i,1))
                    idp(3)=-idhep(1)
                    inout(3,4)=-1
                    IF(idhep(iwp(i,j,2)).GT.0)THEN
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(1)=idhep(iwp(i,j,2))
                      idp(4)=idhep(jwp(i,j,2))
                    ELSE
                      iorder(3,4)=1
                      iorder(4,4)=4            
                      idp(4)=idhep(iwp(i,j,2))
                      idp(1)=idhep(jwp(i,j,2))
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(5)=idhep(iwp(i,j,1))
                      idp(6)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(5,4)=5
                      iorder(6,4)=6            
                      idp(6)=idhep(iwp(i,j,1))
                      idp(5)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(jtww(i,1)).GT.0)THEN
                      iorder(8,4)=3
                      iorder(2,4)=2            
                      idp(3)=idhep(jtww(i,1))
                      idp(2)=-idhep(2)
                      inout(2,4)=-1
                    ELSE
                      iorder(8,4)=2
                      iorder(2,4)=3            
                      idp(2)=idhep(jtww(i,1))
                      idp(3)=-idhep(2)
                      inout(3,4)=-1
                    ENDIF
                  ELSE
                    iorder(7,4)=6
                    iorder(1,4)=5
                    idp(6)=idhep(itWW(i,1))
                    idp(5)=-idhep(1)
                    inout(5,4)=-1
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,4)=3
                      iorder(2,4)=2
                      idp(3)=idhep(jtWW(i,1))
                      idp(2)=-idhep(2)
                      inout(2,4)=-1
                    ELSE
                      iorder(8,4)=2
                      iorder(2,4)=3
                      idp(2)=idhep(jtWW(i,1))
                      idp(3)=-idhep(2)
                      inout(3,4)=-1
                    ENDIF
                    IF(iFa(iwp(i,j,1)).EQ.iFa(jtWW(i,1)).AND.
     &                 Le(iwp(i,j,1)).EQ.Le(jtWW(i,1)))THEN
                      IF(idhep(iwp(i,j,1)).GT.0)THEN
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(1)=idhep(iwp(i,j,1))
                        idp(4)=idhep(jwp(i,j,1))
                      ELSE
                        iorder(3,4)=4
                        iorder(4,4)=1
                        idp(4)=idhep(iwp(i,j,1))
                        idp(1)=idhep(jwp(i,j,1))
                      ENDIF
                      IF(idhep(iwp(i,j,2)).GT.0)THEN
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(5)=idhep(iwp(i,j,2))
                        idp(6)=idhep(jwp(i,j,2))
                      ELSE
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(6)=idhep(iwp(i,j,2))
                        idp(5)=idhep(jwp(i,j,2))
                      ENDIF                      
                    ELSE IF(iFa(iwp(i,j,2)).EQ.iFa(jtWW(i,2)).AND.
     &                      Le(iwp(i,j,2)).EQ.Le(jtWW(i,2)))THEN
                      IF(idhep(iwp(i,j,2)).GT.0)THEN
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(1)=idhep(iwp(i,j,2))
                        idp(4)=idhep(jwp(i,j,2))
                      ELSE
                        iorder(3,4)=1
                        iorder(4,4)=4
                        idp(4)=idhep(iwp(i,j,2))
                        idp(1)=idhep(jwp(i,j,2))
                      ENDIF
                      IF(idhep(iwp(i,j,1)).GT.0)THEN
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(5)=idhep(iwp(i,j,1))
                        idp(6)=idhep(jwp(i,j,1))
                      ELSE
                        iorder(5,4)=5
                        iorder(6,4)=6
                        idp(6)=idhep(iwp(i,j,1))
                        idp(5)=idhep(jwp(i,j,1))
                      ENDIF                      
                    ENDIF
                  ENDIF
                ENDIF  
              ENDIF ! T(WW)W+W+
            ENDIF !igroup=15,16
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(WW) W+ W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'t(WW) W- W+ ').AND.
     &       ialfa(5).EQ.0)THEN
            ialfa(5)=1
            ia=5
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(itWW(i,1)).GT.0)THEN
                IF(Q(itWW(i,1)).GT.0.d0)THEN
                  iorder(7,5)=1
                  iorder(1,5)=2            
                  idp(1)=idhep(itWW(i,1))
                  idp(2)=-idhep(1)
                  inout(2,5)=-1
                ELSE
                  iorder(7,5)=3
                  iorder(1,5)=4            
                  idp(3)=idhep(itWW(i,1))
                  idp(4)=-idhep(1)
                  inout(4,5)=-1
                ENDIF
              ELSE IF(idhep(itWW(i,1)).LT.0)THEN
                IF(Q(itWW(i,1)).GT.0.d0)THEN
                  iorder(7,5)=2
                  iorder(1,5)=1
                  idp(2)=idhep(itWW(i,1))
                  idp(1)=-idhep(1)
                  inout(1,5)=-1
                ELSE
                  iorder(7,5)=4
                  iorder(1,5)=3
                  idp(4)=idhep(itWW(i,1))
                  idp(3)=-idhep(1)
                  inout(3,5)=-1
                ENDIF
              ENDIF
              IF(idhep(jtWW(i,1)).GT.0)THEN
                IF(Q(jtWW(i,1)).GT.0.d0)THEN
                  iorder(8,5)=1
                  iorder(2,5)=2
                  idp(1)=idhep(jtWW(i,1))
                  idp(2)=-idhep(2)
                  inout(2,5)=-1
                ELSE
                  iorder(8,5)=3
                  iorder(2,5)=4
                  idp(3)=idhep(jtWW(i,1))
                  idp(4)=-idhep(2)
                  inout(4,5)=-1
                ENDIF
              ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                IF(Q(jtWW(i,1)).GT.0.d0)THEN
                  iorder(8,5)=2
                  iorder(2,5)=1
                  idp(2)=idhep(jtWW(i,1))
                  idp(1)=-idhep(2)
                  inout(1,5)=-1
                ELSE
                  iorder(8,5)=4
                  iorder(2,5)=3
                  idp(4)=idhep(jtWW(i,1))
                  idp(3)=-idhep(2)
                  inout(3,5)=-1
                ENDIF
              ENDIF
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                iorder(3,5)=5
                iorder(4,5)=6
                idp(5)=idhep(iwp(i,j,1))
                idp(6)=idhep(jwp(i,j,1))
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                iorder(3,5)=5
                iorder(4,5)=6
                idp(6)=idhep(iwp(i,j,1))
                idp(5)=idhep(jwp(i,j,1))
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(5,5)=7
                iorder(6,5)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(5,5)=7
                iorder(6,5)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
              ENDIF
            ELSE    !mixed
              IF(idhep(itWW(i,1)).GT.0.AND.Q(itWW(i,1)).GT.0.d0)THEN
                IF(iFa(itWW(i,1)).EQ.iFa(jtWW(i,1)))THEN
                  iorder(7,5)=1
                  iorder(1,5)=4            
                  idp(1)=idhep(itWW(i,1))
                  idp(4)=-idhep(1)
                  inout(4,5)=-1
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=3
                    iorder(2,5)=2
                    idp(3)=idhep(jtWW(i,1))
                    idp(2)=-idhep(2)
                    inout(2,5)=-1
                  ELSE
                    iorder(8,5)=2
                    iorder(2,5)=3
                    idp(2)=idhep(jtWW(i,1))
                    idp(3)=-idhep(2)
                    inout(3,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,1)))THEN
                  iorder(7,5)=1
                  iorder(1,5)=4            
                  idp(1)=idhep(itWW(i,1))
                  idp(4)=-idhep(1)
                  inout(4,5)=-1
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=3
                    iorder(6,5)=2
                    idp(3)=idhep(iwm(i,j,1))
                    idp(2)=-idhep(2)
                    inout(2,5)=-1
                  ELSE
                    iorder(6,5)=2
                    iorder(5,5)=3
                    idp(2)=idhep(iwm(i,j,1))
                    idp(3)=-idhep(2)
                    inout(3,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=7
                    iorder(2,5)=8
                    idp(7)=idhep(jtWW(i,1))
                    idp(8)=idhep(jtWW(i,1))
                    inout(8,5)=-1
                  ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                    iorder(8,5)=8
                    iorder(2,5)=7
                    idp(8)=idhep(jtWW(i,1))
                    idp(7)=-idhep(2)
                    inout(7,5)=-1
                  ENDIF
                ELSE
                  iorder(7,5)=5
                  iorder(1,5)=6            
                  idp(5)=idhep(itWW(i,1))
                  idp(6)=-idhep(1)
                  inout(6,5)=-1
                  IF(iFa(jtWW(i,1)).EQ.iFa(iwp(i,j,1)))THEN
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=3
                      iorder(2,5)=2
                      idp(3)=idhep(jtWW(i,1))
                      idp(2)=-idhep(2)
                      inout(2,5)=-1
                    ELSE
                      iorder(8,5)=2
                      iorder(2,5)=3
                      idp(2)=idhep(jtWW(i,1))
                      idp(3)=-idhep(2)
                      inout(3,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=7
                      iorder(6,5)=8
                      idp(7)=idhep(iwm(i,j,1))
                      idp(8)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=7
                      iorder(6,5)=8
                      idp(8)=idhep(iwm(i,j,1))
                      idp(7)=idhep(jwm(i,j,1))
                    ENDIF
                  ELSE
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=7
                      iorder(2,5)=8
                      idp(7)=idhep(jtWW(i,1))
                      idp(8)=-idhep(2)
                      inout(8,5)=-1
                    ELSE
                      iorder(8,5)=8
                      iorder(2,5)=7
                      idp(8)=idhep(jtWW(i,1))
                      idp(7)=-idhep(2)
                      inout(7,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                  ENDIF
                ENDIF
              ELSE IF(idhep(itWW(i,1)).LT.0.AND.Q(itWW(i,1)).GT.0.d0)
     &                THEN
                IF(iFa(itWW(i,1)).EQ.iFa(jtWW(i,1)))THEN
                  iorder(7,5)=4
                  iorder(1,5)=1            
                  idp(4)=idhep(itWW(i,1))
                  idp(1)=-idhep(1)
                  inout(1,5)=-1
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=3
                    iorder(2,5)=2
                    idp(3)=idhep(jtWW(i,1))
                    idp(2)=-idhep(2)
                    inout(2,5)=-1
                  ELSE
                    iorder(8,5)=2
                    iorder(2,5)=3
                    idp(2)=idhep(jtWW(i,1))
                    idp(3)=-idhep(2)
                    inout(3,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwm(i,j,1)))THEN
                  iorder(7,5)=4
                  iorder(1,5)=1            
                  idp(4)=idhep(itWW(i,1))
                  idp(1)=-idhep(1)
                  inout(1,5)=-1
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=3
                    iorder(6,5)=2
                    idp(3)=idhep(iwm(i,j,1))
                    idp(2)=-idhep(2)
                    inout(2,5)=-1
                  ELSE
                    iorder(6,5)=2
                    iorder(5,5)=3
                    idp(2)=idhep(iwm(i,j,1))
                    idp(3)=-idhep(2)
                    inout(3,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=7
                    iorder(2,5)=8
                    idp(7)=idhep(jtWW(i,1))
                    idp(8)=idhep(jtWW(i,1))
                    inout(8,5)=-1
                  ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                    iorder(8,5)=8
                    iorder(2,5)=7
                    idp(8)=idhep(jtWW(i,1))
                    idp(7)=-idhep(2)
                    inout(7,5)=-1
                  ENDIF
                ELSE
                  iorder(7,5)=6
                  iorder(1,5)=5            
                  idp(6)=idhep(itWW(i,1))
                  idp(5)=-idhep(1)
                  inout(5,5)=-1
                  IF(iFa(jtWW(i,1)).EQ.iFa(iwp(i,j,1)))THEN
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=3
                      iorder(2,5)=2
                      idp(3)=idhep(jtWW(i,1))
                      idp(2)=-idhep(2)
                      inout(2,5)=-1
                    ELSE
                      iorder(8,5)=2
                      iorder(2,5)=3
                      idp(2)=idhep(jtWW(i,1))
                      idp(3)=-idhep(2)
                      inout(3,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=7
                      iorder(6,5)=8
                      idp(7)=idhep(iwm(i,j,1))
                      idp(8)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=7
                      iorder(6,5)=8
                      idp(8)=idhep(iwm(i,j,1))
                      idp(7)=idhep(jwm(i,j,1))
                    ENDIF
                  ELSE
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=7
                      iorder(2,5)=8
                      idp(7)=idhep(jtWW(i,1))
                      idp(8)=-idhep(2)
                      inout(8,5)=-1
                    ELSE
                      iorder(8,5)=8
                      iorder(2,5)=7
                      idp(8)=idhep(jtWW(i,1))
                      idp(7)=-idhep(2)
                      inout(7,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                  ENDIF
                ENDIF
              ELSE IF(idhep(itWW(i,1)).GT.0.AND.Q(itWW(i,1)).LT.0.d0)
     &                THEN
                IF(iFa(itWW(i,1)).EQ.iFa(jtWW(i,1)))THEN
                  iorder(7,5)=3
                  iorder(1,5)=2            
                  idp(3)=idhep(itWW(i,1))
                  idp(2)=-idhep(1)
                  inout(2,5)=-1
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=1
                    iorder(2,5)=4
                    idp(1)=idhep(jtWW(i,1))
                    idp(4)=-idhep(2)
                    inout(4,5)=-1
                  ELSE
                    iorder(8,5)=4
                    iorder(2,5)=1
                    idp(4)=idhep(jtWW(i,1))
                    idp(1)=-idhep(2)
                    inout(1,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,1)))THEN
                  iorder(7,5)=3
                  iorder(1,5)=2            
                  idp(3)=idhep(itWW(i,1))
                  idp(2)=-idhep(1)
                  inout(2,5)=-1
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=1
                    iorder(4,5)=4
                    idp(1)=idhep(iwp(i,j,1))
                    idp(4)=idhep(jwp(i,j,1))
                  ELSE
                    iorder(4,5)=4
                    iorder(3,5)=1
                    idp(4)=idhep(iwp(i,j,1))
                    idp(1)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=5
                    iorder(2,5)=6
                    idp(5)=idhep(jtWW(i,1))
                    idp(6)=-idhep(2)
                    inout(6,5)=-1
                  ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                    iorder(8,5)=6
                    iorder(2,5)=5
                    idp(6)=idhep(jtWW(i,1))
                    idp(5)=-idhep(2)
                    inout(5,5)=-1
                  ENDIF
                ELSE
                  iorder(7,5)=7
                  iorder(1,5)=8            
                  idp(7)=idhep(itWW(i,1))
                  idp(8)=-idhep(1)
                  inout(8,5)=-1
                  IF(iFa(jtWW(i,1)).EQ.iFa(iwm(i,j,1)))THEN
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=1
                      iorder(2,5)=4
                      idp(1)=idhep(jtWW(i,1))
                      idp(4)=-idhep(2)
                      inout(4,5)=-1
                    ELSE
                      iorder(8,5)=4
                      iorder(2,5)=1
                      idp(4)=idhep(jtWW(i,1))
                      idp(1)=-idhep(2)
                      inout(1,5)=-1
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(6,5)=2
                      iorder(5,5)=3
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=7
                      iorder(4,5)=8
                      idp(7)=idhep(iwp(i,j,1))
                      idp(8)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(3,5)=7
                      iorder(4,5)=8
                      idp(8)=idhep(iwp(i,j,1))
                      idp(7)=idhep(jwp(i,j,1))
                    ENDIF
                  ELSE
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=5
                      iorder(2,5)=6
                      idp(7)=idhep(jtWW(i,1))
                      idp(8)=-idhep(2)
                      inout(8,5)=-1
                    ELSE
                      iorder(8,5)=6
                      iorder(2,5)=5
                      idp(8)=idhep(jtWW(i,1))
                      idp(7)=-idhep(2)
                      inout(7,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                  ENDIF
                ENDIF
              ELSE IF(idhep(itWW(i,1)).LT.0.AND.Q(itWW(i,1)).LT.0.d0)
     &                THEN
                IF(iFa(itWW(i,1)).EQ.iFa(jtWW(i,1)))THEN
                  iorder(7,5)=2
                  iorder(1,5)=3            
                  idp(2)=idhep(itWW(i,1))
                  idp(3)=-idhep(1)
                  inout(3,5)=-1
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=1
                    iorder(2,5)=4
                    idp(1)=idhep(jtWW(i,1))
                    idp(4)=-idhep(2)
                    inout(4,5)=-1
                  ELSE
                    iorder(8,5)=4
                    iorder(2,5)=1
                    idp(4)=idhep(jtWW(i,1))
                    idp(1)=-idhep(2)
                    inout(1,5)=-1
                  ENDIF
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(5)=idhep(iwp(i,j,1))
                    idp(6)=idhep(jwp(i,j,1))
                  ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                    iorder(3,5)=5
                    iorder(4,5)=6
                    idp(6)=idhep(iwp(i,j,1))
                    idp(5)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(5,5)=7
                    iorder(6,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                ELSE IF(iFa(itWW(i,1)).EQ.iFa(iwp(i,j,1)))THEN
                  iorder(7,5)=2
                  iorder(1,5)=3            
                  idp(2)=idhep(itWW(i,1))
                  idp(3)=-idhep(1)
                  inout(3,5)=-1
                  IF(idhep(iwp(i,j,1)).GT.0)THEN
                    iorder(5,5)=1
                    iorder(6,5)=4
                    idp(1)=idhep(iwp(i,j,1))
                    idp(4)=idhep(jwp(i,j,1))
                  ELSE
                    iorder(6,5)=4
                    iorder(5,5)=1
                    idp(4)=idhep(iwp(i,j,1))
                    idp(1)=idhep(jwp(i,j,1))
                  ENDIF
                  IF(idhep(iwm(i,j,1)).GT.0)THEN
                    iorder(3,5)=7
                    iorder(4,5)=8
                    idp(7)=idhep(iwm(i,j,1))
                    idp(8)=idhep(jwm(i,j,1))
                  ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                    iorder(3,5)=7
                    iorder(4,5)=8
                    idp(8)=idhep(iwm(i,j,1))
                    idp(7)=idhep(jwm(i,j,1))
                  ENDIF
                  IF(idhep(jtWW(i,1)).GT.0)THEN
                    iorder(8,5)=5
                    iorder(2,5)=6
                    idp(5)=idhep(jtWW(i,1))
                    idp(6)=-idhep(2)
                    inout(6,5)=-1
                  ELSE IF(idhep(jtWW(i,1)).LT.0)THEN
                    iorder(8,5)=6
                    iorder(2,5)=5
                    idp(6)=idhep(jtWW(i,1))
                    idp(5)=-idhep(2)
                    inout(5,5)=-1
                  ENDIF
                ELSE
                  iorder(7,5)=8
                  iorder(1,5)=7            
                  idp(8)=idhep(itWW(i,1))
                  idp(7)=-idhep(1)
                  inout(7,5)=-1
                  IF(iFa(jtWW(i,1)).EQ.iFa(iwm(i,j,1)))THEN
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=1
                      iorder(2,5)=4
                      idp(1)=idhep(jtWW(i,1))
                      idp(4)=-idhep(2)
                      inout(4,5)=-1
                    ELSE
                      iorder(8,5)=4
                      iorder(2,5)=1
                      idp(4)=idhep(jtWW(i,1))
                      idp(1)=-idhep(2)
                      inout(1,5)=-1
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(6,5)=2
                      iorder(5,5)=3
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=5
                      iorder(4,5)=6
                      idp(5)=idhep(iwp(i,j,1))
                      idp(6)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(3,5)=5
                      iorder(4,5)=6
                      idp(6)=idhep(iwp(i,j,1))
                      idp(5)=idhep(jwp(i,j,1))
                    ENDIF
                  ELSE
                    IF(idhep(jtWW(i,1)).GT.0)THEN
                      iorder(8,5)=5
                      iorder(2,5)=6
                      idp(5)=idhep(jtWW(i,1))
                      idp(6)=-idhep(2)
                      inout(6,5)=-1
                    ELSE
                      iorder(8,5)=6
                      iorder(2,5)=5
                      idp(6)=idhep(jtWW(i,1))
                      idp(5)=-idhep(2)
                      inout(5,5)=-1
                    ENDIF
                    IF(idhep(iwp(i,j,1)).GT.0)THEN
                      iorder(3,5)=1
                      iorder(4,5)=4
                      idp(1)=idhep(iwp(i,j,1))
                      idp(4)=idhep(jwp(i,j,1))
                    ELSE
                      iorder(4,5)=4
                      iorder(3,5)=1
                      idp(4)=idhep(iwp(i,j,1))
                      idp(1)=idhep(jwp(i,j,1))
                    ENDIF
                    IF(idhep(iwm(i,j,1)).GT.0)THEN
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(3)=idhep(iwm(i,j,1))
                      idp(2)=idhep(jwm(i,j,1))
                    ELSE
                      iorder(5,5)=3
                      iorder(6,5)=2
                      idp(2)=idhep(iwm(i,j,1))
                      idp(3)=idhep(jwm(i,j,1))
                    ENDIF
                  ENDIF
                ENDIF
              ENDIF
            ENDIF ! mixed
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'W- -> W- W+ W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W- -> W+ W- W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W- -> W- W- W+ ').AND.
     &       ialfa(6).EQ.0)THEN
            ialfa(6)=1
            ia=6
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(7,6)=3
                  iorder(8,6)=4            
                  idp(3)=idhep(iwm(i,j,1))
                  idp(4)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,6)=7
                  iorder(6,6)=8            
                  idp(7)=idhep(iwm(i,j,1))
                  idp(8)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=4            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=4            
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                IF(Le(iwm(i,j,1)).NE.0)THEN
                  iorder(7,6)=3
                  iorder(8,6)=4            
                  idp(4)=idhep(iwm(i,j,1))
                  idp(3)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,6)=7
                  iorder(6,6)=8            
                  idp(8)=idhep(iwm(i,j,1))
                  idp(7)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=4            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=4            
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                iorder(3,6)=5
                iorder(4,6)=6
                idp(5)=idhep(iwp(i,j,1))
                idp(6)=idhep(jwp(i,j,1))
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                iorder(3,6)=5
                iorder(4,6)=6
                idp(6)=idhep(iwp(i,j,1))
                idp(5)=idhep(jwp(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,6)=2
                iorder(2,6)=1
                idp(2)=-idhep(1)
                idp(1)=-idhep(2)
                inout(1,6)=-1
                inout(2,6)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,6)=1
                iorder(2,6)=2
                idp(1)=-idhep(1)
                idp(2)=-idhep(2)
                inout(1,6)=-1
                inout(2,6)=-1
              ENDIF
            ELSE      !Mixed
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(7,6)=3
                  iorder(8,6)=2            
                  idp(3)=idhep(iwm(i,j,1))
                  idp(2)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,6)=7
                  iorder(6,6)=8            
                  idp(7)=idhep(iwm(i,j,1))
                  idp(8)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=2            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(2)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=2            
                    idp(2)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(7,6)=3
                  iorder(8,6)=2            
                  idp(2)=idhep(iwm(i,j,1))
                  idp(3)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,6)=7
                    iorder(6,6)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(5,6)=7
                  iorder(6,6)=8            
                  idp(8)=idhep(iwm(i,j,1))
                  idp(7)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=2            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(2)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,6)=3
                    iorder(8,6)=2            
                    idp(2)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF((Le(iwp(i,j,1)).EQ.Le(iwm(i,j,1)).AND.
     &           iFa(iwp(i,j,1)).NE.iFa(iwm(i,j,1))).OR.
     &           (Le(iwp(i,j,1)).EQ.Le(iwm(i,j,2)).AND.
     &           iFa(iwp(i,j,1)).NE.iFa(iwm(i,j,2))))THEN
                IF(idhep(iwp(i,j,1)).GT.0)THEN
                  iorder(3,6)=5
                  iorder(4,6)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,6)=5
                  iorder(4,6)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ENDIF
                IF(idhep(1).GT.0)THEN
                  iorder(1,6)=4
                  iorder(2,6)=1
                  idp(4)=-idhep(1)
                  idp(1)=-idhep(2)
                  inout(1,6)=-1
                  inout(4,6)=-1
                ELSE IF(idhep(1).LT.0)THEN
                  iorder(1,6)=1
                  iorder(2,6)=4
                  idp(1)=-idhep(1)
                  idp(4)=-idhep(2)
                  inout(1,6)=-1
                  inout(4,6)=-1
                ENDIF
              ELSE
                IF(idhep(iwp(i,j,1)).GT.0)THEN
                  iorder(3,6)=1
                  iorder(4,6)=4
                  idp(1)=idhep(iwp(i,j,1))
                  idp(4)=idhep(jwp(i,j,1))
                ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,6)=1
                  iorder(4,6)=4
                  idp(4)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                ENDIF
                IF(idhep(1).GT.0)THEN
                  iorder(1,6)=6
                  iorder(2,6)=5
                  idp(6)=-idhep(1)
                  idp(5)=-idhep(2)
                  inout(5,6)=-1
                  inout(6,6)=-1
                ELSE IF(idhep(1).LT.0)THEN
                  iorder(1,6)=5
                  iorder(2,6)=6
                  idp(5)=-idhep(1)
                  idp(6)=-idhep(2)
                  inout(5,6)=-1
                  inout(6,6)=-1
                ENDIF
              ENDIF
            ENDIF
            ialfa(7)=1
            ia=7
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(5,7)=3
                  iorder(6,7)=4            
                  idp(3)=idhep(iwm(i,j,1))
                  idp(4)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,7)=7
                  iorder(8,7)=8            
                  idp(7)=idhep(iwm(i,j,1))
                  idp(8)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=4            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=4            
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                IF(Le(iwm(i,j,1)).NE.0)THEN
                  iorder(5,7)=3
                  iorder(6,7)=4            
                  idp(4)=idhep(iwm(i,j,1))
                  idp(3)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,7)=7
                  iorder(8,7)=8            
                  idp(8)=idhep(iwm(i,j,1))
                  idp(7)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=4            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(4)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=4            
                    idp(4)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                iorder(3,7)=5
                iorder(4,7)=6
                idp(5)=idhep(iwp(i,j,1))
                idp(6)=idhep(jwp(i,j,1))
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                iorder(3,7)=5
                iorder(4,7)=6
                idp(6)=idhep(iwp(i,j,1))
                idp(5)=idhep(jwp(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,7)=2
                iorder(2,7)=1
                idp(2)=-idhep(1)
                idp(1)=-idhep(2)
                inout(1,7)=-1
                inout(2,7)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,7)=1
                iorder(2,7)=2
                idp(1)=-idhep(1)
                idp(2)=-idhep(2)
                inout(1,7)=-1
                inout(2,7)=-1
              ENDIF
            ELSE      !Mixed
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(5,7)=3
                  iorder(6,7)=2            
                  idp(3)=idhep(iwm(i,j,1))
                  idp(2)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,7)=7
                  iorder(8,7)=8            
                  idp(7)=idhep(iwm(i,j,1))
                  idp(8)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=2            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(2)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=2            
                    idp(2)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                IF(Le(iwm(i,j,1)).EQ.0)THEN
                  iorder(5,7)=3
                  iorder(6,7)=2            
                  idp(2)=idhep(iwm(i,j,1))
                  idp(3)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(7)=idhep(iwm(i,j,2))
                    idp(8)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(7,7)=7
                    iorder(8,7)=8            
                    idp(8)=idhep(iwm(i,j,2))
                    idp(7)=idhep(jwm(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,7)=7
                  iorder(8,7)=8            
                  idp(8)=idhep(iwm(i,j,1))
                  idp(7)=idhep(jwm(i,j,1))
                  IF(idhep(iwm(i,j,2)).GT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=2            
                    idp(3)=idhep(iwm(i,j,2))
                    idp(2)=idhep(jwm(i,j,2))
                  ELSE IF(idhep(iwm(i,j,2)).LT.0)THEN
                    iorder(5,7)=3
                    iorder(6,7)=2            
                    idp(2)=idhep(iwm(i,j,2))
                    idp(3)=idhep(jwm(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF((Le(iwp(i,j,1)).EQ.Le(iwm(i,j,1)).AND.
     &           iFa(iwp(i,j,1)).NE.iFa(iwm(i,j,1))).OR.
     &           (Le(iwp(i,j,1)).EQ.Le(iwm(i,j,2)).AND.
     &           iFa(iwp(i,j,1)).NE.iFa(iwm(i,j,2))))THEN
                IF(idhep(iwp(i,j,1)).GT.0)THEN
                  iorder(3,7)=5
                  iorder(4,7)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,7)=5
                  iorder(4,7)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ENDIF
                IF(idhep(1).GT.0)THEN
                  iorder(1,7)=4
                  iorder(2,7)=1
                  idp(4)=-idhep(1)
                  idp(1)=-idhep(2)
                  inout(1,7)=-1
                  inout(4,7)=-1
                ELSE IF(idhep(1).LT.0)THEN
                  iorder(1,7)=1
                  iorder(2,7)=4
                  idp(1)=-idhep(1)
                  idp(4)=-idhep(2)
                  inout(1,7)=-1
                  inout(4,7)=-1
                ENDIF
	      ELSE
                IF(idhep(iwp(i,j,1)).GT.0)THEN
                  iorder(3,7)=1
                  iorder(4,7)=4
                  idp(1)=idhep(iwp(i,j,1))
                  idp(4)=idhep(jwp(i,j,1))
                ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,7)=1
                  iorder(4,7)=4
                  idp(4)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                ENDIF
                IF(idhep(1).GT.0)THEN
                  iorder(1,7)=6
                  iorder(2,7)=5
                  idp(6)=-idhep(1)
                  idp(5)=-idhep(2)
                  inout(5,7)=-1
                  inout(6,7)=-1
                ELSE IF(idhep(1).LT.0)THEN
                  iorder(1,7)=5
                  iorder(2,7)=6
                  idp(5)=-idhep(1)
                  idp(6)=-idhep(2)
                  inout(5,7)=-1
                  inout(6,7)=-1
                ENDIF
	      ENDIF
            ENDIF
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> W+ W+ W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> W+ W- W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> W- W+ W+ ').AND.
     &       ialfa(8).EQ.0)THEN
            ialfa(8)=1
            ia=8
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(3,ia)=1
                  iorder(4,ia)=2            
                  idp(1)=idhep(iwp(i,j,1))
                  idp(2)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,ia)=5
                  iorder(8,ia)=6            
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(3,ia)=1
                  iorder(4,ia)=2            
                  idp(2)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,ia)=5
                  iorder(8,ia)=6            
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,ia)=4
                iorder(2,ia)=3
                idp(4)=-idhep(1)
                idp(3)=-idhep(2)
                inout(3,ia)=-1
                inout(4,ia)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,ia)=3
                iorder(2,ia)=4
                idp(3)=-idhep(1)
                idp(4)=-idhep(2)
                inout(3,ia)=-1
                inout(4,ia)=-1
              ENDIF
            ELSE      !Mixed
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                IF(iFa(1).NE.iFa(iwp(i,j,1)))THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6            
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=4
                    idp(1)=idhep(iwp(i,j,2))
                    idp(4)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=4
                    idp(4)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,ia)=1
                  iorder(8,ia)=4            
                  idp(1)=idhep(iwp(i,j,1))
                  idp(4)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                IF(iFa(1).NE.iFa(iwp(i,j,1)))THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6            
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=4
                    idp(1)=idhep(iwp(i,j,2))
                    idp(4)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=4
                    idp(4)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(7,ia)=1
                  iorder(8,ia)=4            
                  idp(4)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,ia)=2
                iorder(2,ia)=3
                idp(2)=-idhep(1)
                idp(3)=-idhep(2)
                inout(2,ia)=-1
                inout(3,ia)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,ia)=3
                iorder(2,ia)=2
                idp(3)=-idhep(1)
                idp(2)=-idhep(2)
                inout(2,ia)=-1
                inout(3,ia)=-1
              ENDIF
            ENDIF
            ialfa(9)=1
            ia=9
            IF(igroup.NE.15.AND.igroup.NE.16)THEN
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(7,ia)=1
                  iorder(8,ia)=2            
                  idp(1)=idhep(iwp(i,j,1))
                  idp(2)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6            
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                IF(Le(iwp(i,j,1)).EQ.0)THEN
                  iorder(7,ia)=1
                  iorder(8,ia)=2            
                  idp(2)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=5
                    iorder(4,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6            
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=2
                    idp(1)=idhep(iwp(i,j,2))
                    idp(2)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=1
                    iorder(8,ia)=2
                    idp(2)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,ia)=4
                iorder(2,ia)=3
                idp(4)=-idhep(1)
                idp(3)=-idhep(2)
                inout(3,ia)=-1
                inout(4,ia)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,ia)=3
                iorder(2,ia)=4
                idp(3)=-idhep(1)
                idp(4)=-idhep(2)
                inout(3,ia)=-1
                inout(4,ia)=-1
              ENDIF
            ELSE      !Mixed
              IF(idhep(iwp(i,j,1)).GT.0)THEN
                IF(iFa(1).NE.iFa(iwp(i,j,1)))THEN
                  iorder(7,ia)=5
                  iorder(8,ia)=6            
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=4
                    idp(1)=idhep(iwp(i,j,2))
                    idp(4)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=4
                    idp(4)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(3,ia)=1
                  iorder(4,ia)=4            
                  idp(1)=idhep(iwp(i,j,1))
                  idp(4)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
                IF(iFa(1).NE.iFa(iwp(i,j,1)))THEN
                  iorder(7,ia)=5
                  iorder(8,ia)=6            
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=4
                    idp(1)=idhep(iwp(i,j,2))
                    idp(4)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(3,ia)=1
                    iorder(4,ia)=4
                    idp(4)=idhep(iwp(i,j,2))
                    idp(1)=idhep(jwp(i,j,2))
                  ENDIF
                ELSE
                  iorder(3,ia)=1
                  iorder(4,ia)=4            
                  idp(4)=idhep(iwp(i,j,1))
                  idp(1)=idhep(jwp(i,j,1))
                  IF(idhep(iwp(i,j,2)).GT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(5)=idhep(iwp(i,j,2))
                    idp(6)=idhep(jwp(i,j,2))
                  ELSE IF(idhep(iwp(i,j,2)).LT.0)THEN
                    iorder(7,ia)=5
                    iorder(8,ia)=6
                    idp(6)=idhep(iwp(i,j,2))
                    idp(5)=idhep(jwp(i,j,2))
                  ENDIF
                ENDIF
              ENDIF
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(5,ia)=7
                iorder(6,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
              ENDIF
              IF(idhep(1).GT.0)THEN
                iorder(1,ia)=2
                iorder(2,ia)=3
                idp(2)=-idhep(1)
                idp(3)=-idhep(2)
                inout(2,ia)=-1
                inout(3,ia)=-1
              ELSE IF(idhep(1).LT.0)THEN
                iorder(1,ia)=3
                iorder(2,ia)=2
                idp(3)=-idhep(1)
                idp(2)=-idhep(2)
                inout(2,ia)=-1
                inout(3,ia)=-1
              ENDIF
            ENDIF
          ELSE IF(phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> Z  W+ W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> Z  W- W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> W+ Z  W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> W+ W- Z  '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> W- Z  W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> W- W+ Z  '.
     &       AND.ialfa(10).EQ.0)THEN
            ialfa(10)=1
            ia=10
            IF(idhep(1).GT.0)THEN
              IF((Le(iwp(i,j,1)).EQ.0.AND.iFa(1).NE.iFa(iwp(i,j,1)).AND.
     &           (idhep(1).NE.5.AND.igroup.NE.15)).OR.
     &           ABS(idhep(iz(i,j,1))).EQ.5.OR.
     &           ((igroup.EQ.15.OR.igroup.EQ.16).AND.Q(1).GT.0.d0))THEN
                iorder(1,ia)=2
                iorder(2,ia)=1            
                idp(1)=-idhep(2)
                idp(2)=-idhep(1)
                inout(1,ia)=-1
                inout(2,ia)=-1
	        IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(7,ia)=3
                  iorder(8,ia)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
	        ELSE
                  iorder(7,ia)=3
                  iorder(8,ia)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(1,ia)=4
                iorder(2,ia)=3
                idp(3)=-idhep(2)
                idp(4)=-idhep(1)
                inout(3,ia)=-1
                inout(4,ia)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(7,ia)=1
                  iorder(8,ia)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE
                  iorder(7,ia)=1
                  iorder(8,ia)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ELSE IF(idhep(1).LT.0)THEN
              IF((Le(iwp(i,j,1)).EQ.0.and.iFa(1).NE.iFa(iwp(i,j,1)).
     &           AND.(idhep(1).NE.-5.AND.igroup.NE.15)).OR.
     &           ABS(idhep(iz(i,j,1))).EQ.5.OR.
     &           ((igroup.EQ.15.OR.igroup.EQ.16).AND.Q(1).LT.0.d0))THEN
                iorder(1,ia)=1
                iorder(2,ia)=2
                idp(1)=-idhep(1)
                idp(2)=-idhep(2)
                inout(1,ia)=-1
                inout(2,ia)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(7,ia)=3
                  iorder(8,ia)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE
                  iorder(7,ia)=3
                  iorder(8,ia)=4
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
              ELSE
                iorder(1,ia)=3
                iorder(2,ia)=4
                idp(3)=-idhep(1)
                idp(4)=-idhep(2)
                inout(3,ia)=-1
                inout(4,ia)=-1
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(7,ia)=1
                  iorder(8,ia)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE
                  iorder(7,ia)=1
                  iorder(8,ia)=2
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
              ENDIF
            ENDIF
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(5,ia)=7
              iorder(6,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(5,ia)=7
              iorder(6,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'W- -> Z  Z  W- '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W- -> Z  W- Z '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W- -> W- Z  Z '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> Z  Z  W+ '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> Z  W+ Z '.OR.
     &       phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> W+ Z  Z ').AND.
     &       ialfa(11).EQ.0)THEN
            ialfa(11)=1
            ia=11
            IF(idhep(1).GT.0)THEN
              IF(Q(1).LT.0.d0)THEN
                iorder(1,ia)=6
                iorder(2,ia)=5            
                idp(6)=-idhep(1)
                idp(5)=-idhep(2)
                inout(5,ia)=-1
                inout(6,ia)=-1
              ELSE
                iorder(1,ia)=8
                iorder(2,ia)=7            
                idp(8)=-idhep(1)
                idp(7)=-idhep(2)
                inout(7,ia)=-1
                inout(8,ia)=-1
              ENDIF
            ELSE IF(idhep(1).LT.0)THEN
              IF(Q(1).LT.0.d0)THEN
                iorder(1,ia)=5
                iorder(2,ia)=6            
                idp(5)=-idhep(1)
                idp(6)=-idhep(2)
                inout(5,ia)=-1
                inout(6,ia)=-1
              ELSE
                iorder(1,ia)=7
                iorder(2,ia)=8            
                idp(7)=-idhep(1)
                idp(8)=-idhep(2)
                inout(7,ia)=-1
                inout(8,ia)=-1
              ENDIF
            ENDIF
            IF((ABS(idhep(iz(i,j,1))).EQ.5.AND.
     &         ABS(idhep(iz(i,j,2))).EQ.5).OR.
     &         (ABS(idhep(iz(i,j,1))).NE.5.AND.
     &         ABS(idhep(iz(i,j,2))).NE.5))THEN
              IF(idhep(iz(i,j,1)).GT.0)THEN
                iorder(3,ia)=1
                iorder(4,ia)=2
                idp(1)=idhep(iz(i,j,1))
                idp(2)=idhep(jz(i,j,1))
              ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                iorder(3,ia)=2
                iorder(4,ia)=1
                idp(2)=idhep(iz(i,j,1))
                idp(1)=idhep(jz(i,j,1))
              ENDIF
              IF(idhep(iz(i,j,2)).GT.0)THEN
                iorder(5,ia)=3
                iorder(6,ia)=4
                idp(3)=idhep(iz(i,j,2))
                idp(4)=idhep(jz(i,j,2))
              ELSE IF(idhep(iz(i,j,2)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(iz(i,j,2))
                idp(3)=idhep(jz(i,j,2))
              ENDIF
            ELSE
              IF(ABS(idhep(iz(i,j,1))).EQ.5)THEN
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(5,ia)=3
                  iorder(6,ia)=4
                  idp(3)=idhep(iz(i,j,1))
                  idp(4)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(5,ia)=4
                  iorder(6,ia)=3
                  idp(4)=idhep(iz(i,j,1))
                  idp(3)=idhep(jz(i,j,1))
                ENDIF
                IF(idhep(iz(i,j,2)).GT.0)THEN
                  iorder(3,ia)=1
                  iorder(4,ia)=2
                  idp(1)=idhep(iz(i,j,2))
                  idp(2)=idhep(jz(i,j,2))
                ELSE IF(idhep(iz(i,j,2)).LT.0)THEN
                  iorder(3,ia)=2
                  iorder(4,ia)=1
                  idp(2)=idhep(iz(i,j,2))
                  idp(1)=idhep(jz(i,j,2))
                ENDIF
              ELSE
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  iorder(3,ia)=1
                  iorder(4,ia)=2
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(iz(i,j,1)).LT.0)THEN
                  iorder(3,ia)=2
                  iorder(4,ia)=1
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ENDIF
                IF(idhep(iz(i,j,2)).GT.0)THEN
                  iorder(5,ia)=3
                  iorder(6,ia)=4
                  idp(3)=idhep(iz(i,j,2))
                  idp(4)=idhep(jz(i,j,2))
                ELSE IF(idhep(iz(i,j,2)).LT.0)THEN
                  iorder(5,ia)=4
                  iorder(6,ia)=3
                  idp(4)=idhep(iz(i,j,2))
                  idp(3)=idhep(jz(i,j,2))
                ENDIF
              ENDIF
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(7,ia)=7
              iorder(8,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(7,ia)=7
              iorder(8,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(7,ia)=5
              iorder(8,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(7,ia)=5
              iorder(8,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
            ENDIF
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(W0) t ').AND.
     &       ialfa(12).EQ.0)THEN
            ialfa(12)=1
            ia=12
            IF(idhep(itW0(i,1)).GT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(7,ia)=5
                iorder(1,ia)=6            
                idp(5)=idhep(itW0(i,1))
                idp(6)=-idhep(1)
                inout(6,ia)=-1
              ELSE
                iorder(7,ia)=7
                iorder(1,ia)=8            
                idp(7)=idhep(itW0(i,1))
                idp(8)=-idhep(1)
                inout(8,ia)=-1
              ENDIF
            ELSE IF(idhep(itW0(i,1)).LT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(7,ia)=6
                iorder(1,ia)=5
                idp(6)=idhep(itW0(i,1))
                idp(5)=-idhep(1)
                inout(5,ia)=-1
              ELSE
                iorder(7,ia)=8
                iorder(1,ia)=7            
                idp(8)=idhep(itW0(i,1))
                idp(7)=-idhep(1)
                inout(7,ia)=-1
              ENDIF
            ENDIF
            IF(idhep(jtW0(i,1)).GT.0)THEN
              iorder(8,ia)=1
              iorder(2,ia)=2
              idp(1)=idhep(jtW0(i,1))
              idp(2)=-idhep(2)
              inout(2,ia)=-1
            ELSE IF(idhep(jtW0(i,1)).LT.0)THEN 
              iorder(8,ia)=2
              iorder(2,ia)=1
              idp(2)=idhep(jtW0(i,1))
              idp(1)=-idhep(2)
              inout(1,ia)=-1
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF            
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF            
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(0W) t ').AND.
     &       ialfa(13).EQ.0)THEN
            ialfa(13)=1
            ia=13
            IF(idhep(it0W(i,1)).GT.0)THEN
              iorder(7,ia)=1
              iorder(1,ia)=2            
              idp(1)=idhep(it0W(i,1))
              idp(2)=-idhep(1)
              inout(2,ia)=-1
            ELSE IF(idhep(it0W(i,1)).LT.0)THEN
              iorder(7,ia)=2
              iorder(1,ia)=1
              idp(2)=idhep(it0W(i,1))
              idp(1)=-idhep(1)
              inout(1,ia)=-1
            ENDIF
            IF(idhep(jt0W(i,1)).GT.0)THEN
              IF(Q(jt0W(i,1)).GT.0)THEN
                iorder(8,ia)=5
                iorder(2,ia)=6
                idp(5)=idhep(jt0W(i,1))
                idp(6)=-idhep(2)
                inout(6,ia)=-1
              ELSE
                iorder(8,ia)=7
                iorder(2,ia)=8
                idp(7)=idhep(jt0W(i,1))
                idp(8)=-idhep(2)
                inout(8,ia)=-1
              ENDIF
            ELSE IF(idhep(jt0W(i,1)).LT.0)THEN
              IF(Q(jt0W(i,1)).GT.0)THEN
                iorder(8,ia)=6
                iorder(2,ia)=5
                idp(6)=idhep(jt0W(i,1))
                idp(5)=-idhep(2)
                inout(5,ia)=-1
              ELSE
                iorder(8,ia)=8
                iorder(2,ia)=7
                idp(8)=idhep(jt0W(i,1))
                idp(7)=-idhep(2)
                inout(7,ia)=-1
              ENDIF
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF            
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(5,ia)=4
                iorder(6,ia)=3
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF            
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(W ) Z  t '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'t(W ) t  Z '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'t( W) Z  t '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'t( W) t  Z ').AND.
     &       ialfa(14).EQ.0)THEN
            ialfa(14)=1
            ia=14
            IF(phsp(i,j)(1:l2max(i,j)).EQ.'t(W ) Z  t '.OR.
     &         phsp(i,j)(1:l2max(i,j)).EQ.'t(W ) t  Z ')THEN
              i1=1
              i2=2
            ELSE
              i1=2
              i2=1
            ENDIF
            IF(idhep(itW0(i,1)).GT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(8,ia)=5
                iorder(i1,ia)=6
                idp(5)=idhep(itW0(i,1))
                idp(6)=-idhep(i1)
                inout(6,ia)=-1
              ELSE
                iorder(8,ia)=7
                iorder(i1,ia)=8            
                idp(7)=idhep(itW0(i,1))
                idp(8)=-idhep(i1)
                inout(8,ia)=-1
              ENDIF
            ELSE IF(idhep(itW0(i,1)).LT.0)THEN
              IF(Q(itW0(i,1)).GT.0)THEN
                iorder(8,ia)=6
                iorder(i1,ia)=5
                idp(6)=idhep(itW0(i,1))
                idp(5)=-idhep(i1)
                inout(5,ia)=-1
              ELSE
                iorder(8,ia)=8
                iorder(i1,ia)=7            
                idp(8)=idhep(itW0(i,1))
                idp(7)=-idhep(i1)
                inout(7,ia)=-1
              ENDIF
            ENDIF
            IF(idhep(jtW0(i,1)).GT.0)THEN
              iorder(5,ia)=3
              iorder(i2,ia)=4
              idp(3)=idhep(jtW0(i,1))
              idp(4)=-idhep(i2)
              inout(4,ia)=-1
            ELSE IF(idhep(jtW0(i,1)).LT.0)THEN
              iorder(5,ia)=4
              iorder(i2,ia)=3
              idp(4)=idhep(jtW0(i,1))
              idp(3)=-idhep(i2)
              inout(3,ia)=-1
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0)THEN
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(2)=idhep(iz(i,j,1))
                idp(1)=idhep(jz(i,j,1))
              ELSE
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(1)=idhep(iz(i,j,1))
                idp(2)=idhep(jz(i,j,1))
              ENDIF
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0)THEN
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(2)=idhep(iz(i,j,1))
                idp(1)=idhep(jz(i,j,1))
              ELSE
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(1)=idhep(iz(i,j,1))
                idp(2)=idhep(jz(i,j,1))
              ENDIF
            ENDIF            
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0)THEN
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(2)=idhep(iz(i,j,1))
                idp(1)=idhep(jz(i,j,1))
              ELSE
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(1)=idhep(iz(i,j,1))
                idp(2)=idhep(jz(i,j,1))
              ENDIF
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0)THEN
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(2)=idhep(iz(i,j,1))
                idp(1)=idhep(jz(i,j,1))
              ELSE
                iorder(6,ia)=1
                iorder(7,ia)=2
                idp(1)=idhep(iz(i,j,1))
                idp(2)=idhep(jz(i,j,1))
              ENDIF
            ENDIF            
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(0 ) t  W- '.OR.
     &            phsp(i,j)(1:l2max(i,j)).EQ.'t(0 ) W- t ').AND.
     &            ialfa(15).EQ.0)THEN
            ialfa(15)=1
            ia=15
            IF(iord(i,1,3).NE.0)THEN
              i1=1
              i2=2
            ELSE 
              i1=2
              i2=1
            ENDIF
            IF(idhep(it00(i,1)).GT.0)THEN
              iorder(8,ia)=1
              iorder(i1,ia)=2            
              idp(1)=idhep(it00(i,1))
              idp(2)=-idhep(i1)
              inout(2,ia)=-1
            ELSE IF(idhep(it00(i,1)).LT.0)THEN
              iorder(8,ia)=2
              iorder(i1,ia)=1
              idp(2)=idhep(it00(i,1))
              idp(1)=-idhep(i1)
              inout(1,ia)=-1
            ENDIF !it00
            IF(idhep(jt00(i,1)).GT.0)THEN
              iorder(5,ia)=3
              iorder(i2,ia)=4
              idp(3)=idhep(jt00(i,1))
              idp(4)=-idhep(i2)
              inout(4,ia)=-1
            ELSE IF(idhep(jt00(i,1)).LT.0)THEN
              iorder(5,ia)=4
              iorder(i2,ia)=3
              idp(4)=idhep(jt00(i,1))
              idp(3)=-idhep(i2)
              inout(3,ia)=-1
            ENDIF !jt00
            IF(idhep(jt00(i,1)).GT.0)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(6,ia)=7
                iorder(7,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(6,ia)=7
                iorder(7,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ENDIF
            ELSE IF(idhep(jt00(i,1)).LT.0)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(3,ia)=7
                iorder(4,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(3,ia)=7
                iorder(4,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ENDIF !iwm
            ENDIF !jt00
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'t(0 ) t  W+ '.OR.
     &            phsp(i,j)(1:l2max(i,j)).EQ.'t(0 ) W+ t ').AND.
     &       ialfa(16).EQ.0)THEN
            ialfa(16)=1
            ia=16
            IF(iord(i,1,3).NE.0)THEN
              i1=1
              i2=2
            ELSE 
              i1=2
              i2=1
            ENDIF
            IF(idhep(it00(i,1)).GT.0)THEN
              iorder(8,ia)=1
              iorder(i1,ia)=2            
              idp(1)=idhep(it00(i,1))
              idp(2)=-idhep(i1)
              inout(2,ia)=-1
            ELSE IF(idhep(it00(i,1)).LT.0)THEN
              iorder(8,ia)=2
              iorder(i1,ia)=1
              idp(2)=idhep(it00(i,1))
              idp(1)=-idhep(i1)
              inout(1,ia)=-1
            ENDIF !it00
            IF(idhep(jt00(i,1)).GT.0)THEN
              iorder(5,ia)=3
              iorder(i2,ia)=4
              idp(3)=idhep(jt00(i,1))
              idp(4)=-idhep(i2)
              inout(4,ia)=-1
            ELSE IF(idhep(jt00(i,1)).LT.0)THEN
              iorder(5,ia)=4
              iorder(i2,ia)=3
              idp(4)=idhep(jt00(i,1))
              idp(3)=-idhep(i2)
              inout(3,ia)=-1
            ENDIF !jt00
            IF(idhep(jt00(i,1)).GT.0)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(6,ia)=7
                iorder(7,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(6,ia)=7
                iorder(7,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(3,ia)=5
                  iorder(4,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ENDIF
            ELSE IF(idhep(jt00(i,1)).LT.0)THEN
              IF(idhep(iwm(i,j,1)).GT.0)THEN
                iorder(3,ia)=7
                iorder(4,ia)=8
                idp(7)=idhep(iwm(i,j,1))
                idp(8)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
                iorder(3,ia)=7
                iorder(4,ia)=8
                idp(8)=idhep(iwm(i,j,1))
                idp(7)=idhep(jwm(i,j,1))
                IF(idhep(iwp(i,j,1)).LT.0)THEN
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(6)=idhep(iwp(i,j,1))
                  idp(5)=idhep(jwp(i,j,1))
                ELSE
                  iorder(6,ia)=5
                  iorder(7,ia)=6
                  idp(5)=idhep(iwp(i,j,1))
                  idp(6)=idhep(jwp(i,j,1))
                ENDIF
              ENDIF !iwm
            ENDIF !jt00
          ELSE IF((phsp(i,j)(1:l2max(i,j)).EQ.'W- -> Z  t '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'W- -> t  Z '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> Z  t '.OR.
     &             phsp(i,j)(1:l2max(i,j)).EQ.'W+ -> t  Z ').AND.
     &       ialfa(17).EQ.0)THEN
            ialfa(17)=1
            ia=17
            IF(idhep(1).GT.0)THEN
              IF(Q(1).GT.0)THEN
                iorder(1,ia)=8
                iorder(2,ia)=7            
                idp(7)=-idhep(2)
                idp(8)=-idhep(1)
                inout(7,ia)=-1
                inout(8,ia)=-1
              ELSE
                iorder(1,ia)=6
                iorder(2,ia)=5            
                idp(6)=-idhep(1)
                idp(5)=-idhep(2)
                inout(5,ia)=-1
                inout(6,ia)=-1
              ENDIF
            ELSE IF(idhep(1).LT.0)THEN
              IF(Q(1).GT.0)THEN
                iorder(1,ia)=7
                iorder(2,ia)=8
                idp(7)=-idhep(1)
                idp(8)=-idhep(2)
                inout(7,ia)=-1
                inout(8,ia)=-1
              ELSE
                iorder(1,ia)=5
                iorder(2,ia)=6            
                idp(5)=-idhep(1)
                idp(6)=-idhep(2)
                inout(5,ia)=-1
                inout(6,ia)=-1
              ENDIF
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0.OR.idhep(ib(i,j,2)).LT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).LT.0)THEN
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).LT.0)THEN
                  idp(2)=idhep(ib(i,j,2))
                  idp(1)=idhep(jb(i,j,2))
                ENDIF
              ELSE IF(idhep(iz(i,j,1)).GT.0.OR.
     &                idhep(ib(i,j,2)).GT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).GT.0)THEN
                  idp(1)=idhep(ib(i,j,2))
                  idp(2)=idhep(jb(i,j,2))
                ENDIF
              ENDIF
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(7,ia)=4
                iorder(8,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(8,ia)=3
                iorder(7,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(3,ia)=7
              iorder(4,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0.OR.idhep(ib(i,j,2)).LT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).LT.0)THEN
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).LT.0)THEN
                  idp(2)=idhep(ib(i,j,2))
                  idp(1)=idhep(jb(i,j,2))
                ENDIF
              ELSE IF(idhep(iz(i,j,1)).GT.0.OR.
     &                idhep(ib(i,j,2)).GT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).GT.0)THEN
                  idp(1)=idhep(ib(i,j,2))
                  idp(2)=idhep(jb(i,j,2))
                ENDIF
              ENDIF
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(7,ia)=4
                iorder(8,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(8,ia)=3
                iorder(7,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF            
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0.OR.idhep(ib(i,j,2)).LT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).LT.0)THEN
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).LT.0)THEN
                  idp(2)=idhep(ib(i,j,2))
                  idp(1)=idhep(jb(i,j,2))
                ENDIF
              ELSE IF(idhep(iz(i,j,1)).GT.0.OR.
     &                idhep(ib(i,j,2)).GT.0)THEN
                iorder(5,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).GT.0)THEN
                  idp(1)=idhep(ib(i,j,2))
                  idp(2)=idhep(jb(i,j,2))
                ENDIF
              ENDIF
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(8,ia)=4
                iorder(7,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
              IF(idhep(iz(i,j,1)).LT.0.OR.idhep(ib(i,j,2)).LT.0)THEN
                iorder(7,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).LT.0)THEN
                  idp(2)=idhep(iz(i,j,1))
                  idp(1)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).LT.0)THEN
                  idp(2)=idhep(ib(i,j,2))
                  idp(1)=idhep(jb(i,j,2))
                ENDIF
              ELSE IF(idhep(iz(i,j,1)).GT.0.OR.
     &                idhep(ib(i,j,2)).GT.0)THEN
                iorder(7,ia)=1
                iorder(6,ia)=2
                IF(idhep(iz(i,j,1)).GT.0)THEN
                  idp(1)=idhep(iz(i,j,1))
                  idp(2)=idhep(jz(i,j,1))
                ELSE IF(idhep(ib(i,j,2)).GT.0)THEN
                  idp(1)=idhep(ib(i,j,2))
                  idp(2)=idhep(jb(i,j,2))
                ENDIF
              ENDIF
              IF(idhep(ib(i,j,1)).LT.0)THEN
                iorder(8,ia)=4
                iorder(7,ia)=3
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ELSE
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
              ENDIF
            ENDIF
          ELSE IF(phsp(i,j)(1:l2max(i,j)).EQ.'Z  -> t  t '.AND.
     &       ialfa(18).EQ.0)THEN
            ialfa(18)=1
            ia=18
            IF(idhep(1).GT.0)THEN
              iorder(1,ia)=2
              iorder(2,ia)=1            
              idp(1)=-idhep(2)
              idp(2)=-idhep(1)
              inout(1,ia)=-1
              inout(2,ia)=-1
	      IF(idhep(ib(i,j,1)).GT.0)THEN
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
	      ELSE
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ENDIF
            ELSE
              iorder(1,ia)=1
              iorder(2,ia)=2            
              idp(2)=-idhep(2)
              idp(1)=-idhep(1)
              inout(1,ia)=-1
              inout(2,ia)=-1
	      IF(idhep(ib(i,j,1)).GT.0)THEN
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(3)=idhep(ib(i,j,1))
                idp(4)=idhep(jb(i,j,1))
	      ELSE
                iorder(7,ia)=3
                iorder(8,ia)=4
                idp(4)=idhep(ib(i,j,1))
                idp(3)=idhep(jb(i,j,1))
              ENDIF
            ENDIF
            IF(idhep(iwp(i,j,1)).GT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(5)=idhep(iwp(i,j,1))
              idp(6)=idhep(jwp(i,j,1))
            ELSE IF(idhep(iwp(i,j,1)).LT.0)THEN
              iorder(3,ia)=5
              iorder(4,ia)=6
              idp(6)=idhep(iwp(i,j,1))
              idp(5)=idhep(jwp(i,j,1))
            ENDIF
            IF(idhep(iwm(i,j,1)).GT.0)THEN
              iorder(5,ia)=7
              iorder(6,ia)=8
              idp(7)=idhep(iwm(i,j,1))
              idp(8)=idhep(jwm(i,j,1))
            ELSE IF(idhep(iwm(i,j,1)).LT.0)THEN
              iorder(5,ia)=7
              iorder(6,ia)=8
              idp(8)=idhep(iwm(i,j,1))
              idp(7)=idhep(jwm(i,j,1))
            ENDIF
          ENDIF
        ENDDO
      ENDDO

      n_kphs=0
      do i=1,mxnphs
        if(ialfa(i).ne.0)n_kphs=n_kphs+1
      enddo

      ncount=0

      print*,'phase space number=',n_kphs
      
      do i=1,mxnphs
        if(ialfa(i).ne.0)then
          i1=i
          goto 600
        endif
      enddo

* for identical particles in the final states, when there are more phase spaces
* switched on, it might happen that different phase spaces have the initial
* particles (if identical) in different positions of the amplitude order.
* In order to avoid that, the vector iorder is fixed to only one possibility. 

 600  do i=i1+1,mxnphs
        if(ialfa(i).eq.1.and.iorder(2,i).ne.iorder(2,i1))then
          do j=1,8
            if(iorder(j,i).eq.iorder(2,i1))then
              iorder(j,i)=iorder(2,i)
            endif
          enddo
          iorder(2,i)=iorder(2,i1)
        endif
        if(ialfa(i).eq.1.and.iorder(1,i).ne.iorder(1,i1))then
          do j=1,8
            if(iorder(j,i).eq.iorder(1,i1))then
              iorder(j,i)=iorder(1,i)
            endif
          enddo
          iorder(1,i)=iorder(1,i1)
        endif
      enddo

      do i=1,8
        idp_inout(i)=inout(i,i1)
      enddo

      ENDIF !ionesh

      RETURN
      END
